Type: Package
Package: hce
Title: Design and Analysis of Hierarchical Composite Endpoints
Version: 0.9.4
Authors@R: 
    person("Samvel B.", "Gasparyan", , "gasparyan.co@gmail.com", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-4797-2208"))
Description: Simulate and analyze hierarchical composite endpoints with univariate distributions by Gasparyan, Koch, Brunner in (2025) in “The Univariate Distribution of Hierarchical Composite Endpoints and the Condorcet Non-transitivity Paradox.” (Biometrical Journal 68 (3), <doi:10.1002/bimj.70140>). Includes implementation for the kidney hierarchical composite endpoint as defined in Heerspink HL et al (2023) “Development and validation of a new hierarchical composite end point for clinical trials of kidney disease progression” (Journal of the American Society of Nephrology 34 (2): 2025–2038, <doi:10.1681/ASN.0000000000000243>). Win odds, also called Wilcoxon-Mann-Whitney or success odds, is the main analysis method, but other win statistics (win probability, win ratio, net benefit) are also implemented in the univariate case. The win probability analysis is based on the Brunner-Munzel test and uses the DeLong-DeLong-Clarke-Pearson variance estimator, as described by Brunner and Konietschke (2025) in “An unbiased rank-based estimator of the Mann–Whitney variance including the case of ties” (Statistical Papers 66 (1): 20, <doi:10.1007/s00362-024-01635-0>). Includes implementation of a new Wilson-type, compatible confidence interval for the win odds, as proposed by Schüürhuis, Konietschke, Brunner (2025) in “A new approach to the nonparametric Behrens–Fisher problem with compatible confidence intervals.” (Biometrical Journal 67 (6), <doi:10.1002/bimj.70096>). Stratification and covariate adjustment are performed based on the methodology presented by Koch GG et al. in “Issues for covariance analysis of dichotomous and ordered categorical data from randomized clinical trials and non-parametric strategies for addressing them” (Statistics in Medicine 17 (15-16): 1863–92). For a review, see Gasparyan SB et al (2021) “Adjusted win ratio with stratification: Calculation methods and interpretation” (Statistical Methods in Medical Research 30 (2): 580–611, <doi:10.1177/0962280220942558>).
License: MIT + file LICENSE
BugReports: https://github.com/Samve/hce/issues
Depends: R (>= 3.5.0)
Imports: base, stats
Suggests: knitr, rmarkdown
Encoding: UTF-8
LazyData: true
VignetteBuilder: knitr
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-06-26 18:08:20 UTC; r461986
Author: Samvel B. Gasparyan [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-4797-2208>)
Maintainer: Samvel B. Gasparyan <gasparyan.co@gmail.com>
Repository: CRAN
Date/Publication: 2026-06-26 19:00:02 UTC
Built: R 4.5.2; ; 2026-06-26 19:15:22 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(as_formulae,formula)
S3method(as_hce,data.frame)
S3method(as_hce,default)
S3method(calcWINS,data.frame)
S3method(calcWINS,formula)
S3method(calcWINS,hce)
S3method(calcWO,data.frame)
S3method(calcWO,formula)
S3method(calcWO,hce)
S3method(deltaWO,adhce)
S3method(plot,hce)
S3method(plot,hce_results)
S3method(print,hce_results)
S3method(regWO,data.frame)
S3method(regWO,formula)
S3method(stratWO,data.frame)
S3method(stratWO,formula)
S3method(summaryWO,adhce)
S3method(summaryWO,data.frame)
S3method(summaryWO,formula)
S3method(summaryWO,hce)
export(HGLL)
export(IWP)
export(as_formulae)
export(as_hce)
export(calcWINS)
export(calcWO)
export(dGLL)
export(deltaWO)
export(hGLL)
export(hce)
export(minWO)
export(pGLL)
export(powerWO)
export(propWINS)
export(qGLL)
export(rGLL)
export(regWO)
export(rweibullGF)
export(simHCE)
export(simKHCE)
export(simORD)
export(simTTE)
export(sizeWO)
export(sizeWR)
export(stratWO)
export(summaryWO)

# hce <img src="man/figures/logo.png" align="right" width="200" />

![Maturity level-Stable](https://img.shields.io/badge/Maturity%20Level-Stable-orange)
[![CRAN status](https://www.r-pkg.org/badges/version-last-release/hce)](https://CRAN.R-project.org/package=hce)
[![Downloads](https://cranlogs.r-pkg.org/badges/hce)](https://CRAN.R-project.org/package=hce)
[![All Downloads](https://cranlogs.r-pkg.org/badges/grand-total/hce)](https://CRAN.R-project.org/package=hce)

Simulate and analyze hierarchical composite endpoints

# Download

You can install hce from CRAN:

```
install.packages("hce")
```

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup--------------------------------------------------------------------
library(hce)
packageVersion("hce")

## ----echo=FALSE---------------------------------------------------------------
HCE <- data.frame(Order = c("I", "II", "III", "IV", "V"), 
                  Category = c("Death", 
                               "More than one new or worsened organ dysfunction events", 
                               "One new or worsened organ dysfunction event", 
                               "Hospitalized at the end of follow-up (Day 30)", 
                               "Discharged from hospital before Day 30")
)
HCE

## -----------------------------------------------------------------------------
table(COVID19)

## -----------------------------------------------------------------------------
COVID19HCE <- hce(GROUP = COVID19$GROUP, TRTP = COVID19$TRTP)
SUM <- summaryWO(COVID19HCE, ref = "Placebo")$summary
SUM$Ptie <- round(SUM$TIE/SUM$TOTAL, 2)
SUM

## ----echo=FALSE---------------------------------------------------------------
HCE2 <- data.frame(Order = c("I", "II", "III", "IV", "V", "VI", "VII"), 
                  Category = c("Death", 
                               "Dialysis or kidney transplantation", 
                               "Sustained GFR < 15 ml/min per 1.73 m2", 
                               "Sustained GFR decline from baseline of >= 57%", 
                               "Sustained GFR decline from baseline of >= 50%",
                               "Sustained GFR decline from baseline of >= 40%",
                               "Individual GFR slope")
)
HCE2

## -----------------------------------------------------------------------------
dat <- KHCE
Order <- c("Death (adj)", "Chronic dialysis (adj) >=90 days", 
           "Sustained eGFR<15 (mL/min/1.73 m2)", "Sustained >=57% decline in eGFR", 
           "Sustained >=50% decline in eGFR", "Sustained >=40% decline in eGFR", "eGFR slope")   
dat$GROUP <- factor(dat$GROUP, levels = Order)
table(dat$GROUP, dat$TRTP)

## ----echo=FALSE---------------------------------------------------------------
knitr::kable(head(ADSL), caption = "ADSL dataset structure\\label{adsl}")

## ----echo=FALSE---------------------------------------------------------------
knitr::kable(head(ADLB), caption = "ADLB dataset structure\\label{adlb}")

## ----echo=FALSE---------------------------------------------------------------
knitr::kable(head(ADET), caption = "ADET dataset structure\\label{adet}")

## ----echo=FALSE---------------------------------------------------------------
HCE3 <- data.frame(Order = c("I", "II", "III", "IV"), 
                  Category = c("Cardiovascular death", 
                               "Total (first and recurrent) HF hospitalizations", 
                               "Total urgent HF visits", 
                               "Improvement/deterioration in KCCQ-TSS"))
HCE3

#' Coerce a data frame to an `hce` object
#'
#' @param x an object.
#' @param ... additional parameters.
#'
#' @return an `hce` object.
#' @export
#' @md
#' @seealso [hce::as_hce()], [hce::as_hce.data.frame()].
#' @examples
#' dat <- KHCE
#' class(dat) <- "moo" # non-existent class
#' as_hce(dat) # tries to convert to an hce object
#' ## It still works because the inheritance converted it to a data frame
as_hce.default <- function(x, ...){
  ##Sometimes, it is not possible to convert an object directly to a data frame. 
  ##However, you can often add inheritance to an object so that it behaves like a data frame. 
  x <- base::structure(x, class = c(class(x), "data.frame"))
  ## call the data frame method of the function
  as_hce(x)
}

#' Helper function for `hce` objects
#'
#' @param GROUP a character vector or a factor containing events. 
#' If a factor, its levels are used to define the hierarchy. Otherwise, the vector is
#' converted to a factor.
#' @param TRTP a character vector of the same length as `GROUP`, indicating assigned treatment groups.
#' @param AVAL0 a numeric vector of the same length as `GROUP`, indicating containing analysis values within each category. The default is 0.
#' @param PADY numeric specifying the length of follow-up in years.
#' @returns an object of class `hce` or `adhce` (if `AVAL0` is provided). The result 
#' is a subject-level data frame, where each row corresponds to one subject, 
#' @export
#' @md
#' @seealso [hce::as_hce()] for coercing to `hce` objects.
#' @examples
#' # Example 1 - Both `AVAL0` and `PADY` are provided. The output is an `adhce` object.
#' GROUP <- COVID19$GROUP
#' TRTP <- rep(c("A", "P"), each = 531)
#' dat <- hce(GROUP, TRTP, PADY = 10, AVAL0 = rnorm(1062))
#' class(dat)
#' calcWO(dat)
#' summaryWO(dat) # Uses the `GROUP` variable for summary.
#' # Example 2 - Only `AVAL0` is provided, `PADY` is calculated as the maximum of `AVAL0`. 
#' # The output is an `adhce` object.
#' set.seed(2022)
#' d <- hce(GROUP = sample(x = c("A", "B", "C"), size = 10, replace = TRUE), 
#' TRTP = rep(c("Active", "Control"), each = 5), 
#' AVAL0 = c(rnorm(5, mean = 1), rnorm(5)))
#' calcWO(d, ref = "Control")
#' ## modify the hierarchy by proving a factor for the GROUP variable. 
#' ## calcWO() applied to an hce rederives `AVAL` based on the `GROUP` variable.
#' d$GROUP <- factor(d$GROUP, levels = c("C", "B", "A"))
#' calcWO(d, ref = "Control")
#' # Example 3 - Provide only `PADY` and not `AVAL0` will not make any difference.
#' GROUP <- COVID19$GROUP
#' TRTP <- rep(c("A", "P"), each = 531)
#' dat <- hce(GROUP, TRTP, PADY = 10)
#' class(dat)
#' calcWO(dat)
#' dat <- hce(GROUP, TRTP)
#' class(dat)
#' calcWO(dat)
hce <- function (GROUP, TRTP, AVAL0 = NULL, PADY = NULL) 
{
  if (length(TRTP) == 0) 
    stop("TRTP is required.")
  if (length(GROUP) == 0) 
    stop("GROUP is required.")
  d <- data.frame(TRTP = TRTP, GROUP = GROUP)
  PADY <- PADY[1]
  d$AVAL0 <- AVAL0
  if(is.null(PADY) & !is.null(AVAL0))
    PADY <- max(AVAL0)
  d$GROUP <- as.factor(d$GROUP)
  d$AVAL <- d$PARAMN <- as.numeric(d$GROUP)
  d$PADY <- PADY
  as_hce(d)
}

#' A generic function for calculating win odds based on a threshold
#'
#' @param x an object used to select a method.
#' @param delta a numeric threshold. 
#' @param ... further arguments passed to or from other methods.
#' @return a data frame containing calculated values.
#' @export
#' @md
#' @seealso [hce::deltaWO.adhce()] method.
#'
deltaWO <- function (x, delta, ...) 
{
  UseMethod("deltaWO")
}

#' Win odds summary for `hce` objects
#'
#' @param x an `hce` object.
#' @param ... additional parameters.
#' @returns a list containing the summary of wins, losses, and ties. It contains the following named objects:
#' * summary a data frame containing number of wins, losses, and ties by treatment group and the overall number of comparisons.
#' * WO calculated WO (win odds) and WP (win probability) and their standard errors.
#' @export
#' @md
#' @seealso [hce::calcWO()], [hce::summaryWO()], [hce::summaryWO.data.frame()], [hce::summaryWO.formula()], [hce::summaryWO.adhce()] methods.
#' @examples
#' COVID19HCE <- hce(GROUP = COVID19$GROUP, TRTP = COVID19$TRTP)
#' summaryWO(COVID19HCE, ref = "Placebo")
summaryWO.hce <- function(x, ...){
  Args <- base::list(...)
  x <- as_hce(x)
  x <- base::as.data.frame(x)
  if(!is.null(Args[["ref"]])) ref <- Args[["ref"]]
  else if ("P" %in% unique(x$TRTP)) ref <- "P"
  else ref <- unique(x$TRTP)[1]
  summaryWO.data.frame(x = x, AVAL = "AVAL", TRTP = "TRTP", ref = ref)
}

#' Coerce a data frame to an `hce` object
#'
#' @param x a data frame.
#' @param ... additional parameters.
#'
#' @return an `hce` object.
#' @export
#' @md
#' @seealso [hce::as_hce()], [hce::as_hce.default()].
#' @examples
#' # The case when all required variables `AVAL0`, `GROUP`, `PADY`, and `TRTP` are present.
#' KHCE <- as_hce(KHCE)
#' ## Converts to an `adhce` object
#' class(KHCE)
#' calcWO(KHCE)
#' # The case when only `AVAL` and `TRTP`.
#' ## Converts to an `hce` object
#' dat <- KHCE[, c("TRTP", "AVAL")]
#' dat <- as_hce(dat)
#' class(dat)
#' summaryWO(dat)
as_hce.data.frame <- function(x, ...){
  ## Validator function for hce objects
  .validate_hce <- function(x){
    # This step applies to objects that inherit from data frames.
    # It removes other types to ensure only 'data.frame' remains as the class.
    x <- base::as.data.frame(x)
    Names <- base::names(x)
    if(base::any(base::dim(x) == 0))
      stop("The dimension of the dataset should be non-zero.")
    if(!"AVAL" %in% Names)
      stop("The dataset should contain the column AVAL for the analysis values.")
    if(! "TRTP" %in% Names)
      stop("The dataset should contain the column TRTP for the planned treatment.")
    if(base::length(base::unique(x$TRTP)) != 2)
      stop("The TRTP column should have exactly 2 levels.")
    if(!is.numeric(x$AVAL))
      stop("AVAL must be numeric.")
    x
  }
  ## Validator function for adhce objects
  .validate_adhce <- function(x){
    Names <- base::names(x)
    C1 <- c("GROUP") %in% Names & is.numeric(x$AVAL0) & is.numeric(x$PADY) & length(unique(x$PADY)) == 1
    if(C1) {
      C0 <- !is.factor(x$GROUP) && is.character(x$GROUP) &&  all(startsWith(x$GROUP[x$GROUP != "C"], "TTE")) && any(x$GROUP == "C")
      if(C0){
        x$GROUP <- as.factor(x$GROUP)
        UNIQ <- unique(x$GROUP)
        LEV <- c(sort(UNIQ[UNIQ != "C"]), UNIQ[UNIQ == "C"])
        x$GROUP <- factor(x$GROUP, levels = LEV)
      } else {
        x$GROUP <- as.factor(x$GROUP)  
      }
      x$PARAMN <- as.numeric(x$GROUP)
      LEVELS <- levels(x$GROUP)
      LAST <- LEVELS[length(LEVELS)]
      stopifnot("All timepoints in `AVAL0` for the time-to-event endpoints should be less than or equal to the evaluation timeframe specified in `PADY`." = all(x$AVAL0[x$GROUP != LAST] <= x$PADY[x$GROUP != LAST]))
      m <- min(x$AVAL0[x$GROUP == LAST], na.rm = TRUE)
      x$GROUPN <- x$PARAMN*x$PADY
      x$AVAL <- ifelse(x$GROUP != LAST, x$AVAL0 + x$GROUPN, x$AVAL0 - m + 1 + x$GROUPN)
      return(x)
    } else {
      return(x)
    }
  }
  ## Constructor function for hce and adhce objects
  .new_hce <- function(x = data.frame()){
    base::stopifnot(base::is.data.frame(x))
    x1 <- .validate_adhce(x)
    if(identical(x, x1)){
      x <- .validate_hce(x)
      y <- base::structure(x, class = c("hce", "data.frame"))  
    } else {
      x1 <- .validate_hce(x1)
      y <- base::structure(x1, class = c("adhce", "hce", "data.frame"))  
    }
    y
  }
  
  .new_hce(x)
}

#' Win statistics calculation for `hce` objects
#'
#' @param x an `hce` object.
#' @param ... additional parameters.
#'
#' @returns a list containing win statistics and their confidence intervals. It contains the following named data frames: 
#' * summary a data frame containing number of wins, losses, and ties of the active treatment group and the overall number of comparisons. 
#' * WP a data frame containing the win probability and its confidence interval. 
#' * NetBenefit a data frame containing the net benefit and its confidence interval. This is just a `2x-1` transformation of WP and its CI.
#' * WO a data frame containing the win odds and its confidence interval. 
#' * WR1 a data frame containing the win ratio and its confidence interval, using the transformed standard error of the `gamma` statistic. 
#' * WR2 a data frame containing the win ratio and its confidence interval, using the standard error calculated using `Pties`. 
#' * gamma a data frame containing Goodman Kruskal's `gamma` and its confidence interval. 
#' * SE a data frame containing standard errors used to calculated the Confidence intervals for win statistics. 
#' 
#' When `SE_WP_Type = "unbiased"`, the `WP`, `WO` and `NetBenefit` estimators use the unbiased variance estimator of `WP`. Additionally, a Wilson-type range-preserving confidence interval is provided: 
#' * WP_W a data frame containing the win probability and its range-preserving confidence interval. 
#' * NetBenefit_W a data frame containing the net benefit and its range-preserving confidence interval. 
#' * WO_W a data frame containing the win odds and its range-preserving confidence interval. 
#' @export
#' @md
#' @seealso [hce::calcWINS()], [hce::calcWINS.formula()], [hce::calcWINS.data.frame()].
#' @references
#' The theory of win statistics is covered in the following papers: 
#' * Win proportion and win odds confidence interval calculation: 
#' \cr \cr Somers RH (1962) “A New Asymmetric Measure of Association for Ordinal Variables." American Sociological Review 27.6: 799-811. <doi:10.2307/2090408>.
#' \cr \cr Bamber D (1975) "The area above the ordinal dominance graph and the area below the receiver operating characteristic graph." Journal of Mathematical Psychology 12.4: 387-415. <doi:10.1016/0022-2496(75)90001-2>.
#' \cr \cr DeLong ER et al. (1988) "Comparing the Areas Under Two or More Correlated Receiver Operating Characteristic Curves: A Nonparametric Approach." Biometrics 44.3: 837-845. <doi:10.2307/2531595>.
#' \cr \cr Brunner E et al. (2021) "Win odds: an adaptation of the win ratio to include ties." Statistics in Medicine 40.14: 3367-3384. <doi:10.1002/sim.8967>.
#' \cr \cr Gasparyan SB et al. (2021) "Adjusted win ratio with stratification: calculation methods and interpretation." Statistical Methods in Medical Research 30.2: 580-611. <doi:10.1177/0962280220942558>.
#' \cr \cr Gasparyan SB et al. (2021) "Power and sample size calculation for the win odds test: application to an ordinal endpoint in COVID-19 trials." Journal of Biopharmaceutical Statistics 31.6: 765-787. <doi:10.1080/10543406.2021.1968893>.
#' \cr \cr Brunner E, Konietschke F. (2025) "An unbiased rank-based estimator of the Mann–Whitney variance including the case of ties." Statistical Papers 66.20. <doi:10.1007/s00362-024-01635-0>.
#' * Win ratio: the first CI utilizes the standard error derived from the `gamma` statistic standard error as outlined by:
#' \cr \cr Gasparyan SB, Kowalewski EK, Buenconsejo J, Koch GG. (2023) "Hierarchical Composite Endpoints in COVID-19: The DARE-19 Trial." In Case Studies in Innovative Clinical Trials, Chapter 7, 95–148. Chapman; Hall/CRC. <doi:10.1201/9781003288640-7>.
#' * Win ratio: the second CI utilizes the standard error presented by:
#' \cr \cr Yu RX, Ganju J. (2022) "Sample size formula for a win ratio endpoint." Statistics in Medicine 41.6: 950-63. <doi:10.1002/sim.9297>.
#' * Goodman Kruskal's `gamma` and CI: matches implementation in `DescTools::GoodmanKruskalGamma()` and based on:
#' \cr \cr Agresti A. (2002) Categorical Data Analysis. John Wiley & Sons, pp. 57-59. <doi:10.1002/0471249688>.
#' \cr \cr Brown MB, Benedetti JK. (1977) "Sampling Behavior of Tests for Correlation in Two-Way Contingency Tables." Journal of the American Statistical Association 72, 309-315. <doi:10.1080/01621459.1977.10480995>.
#' \cr \cr Goodman LA, Kruskal WH. (1954) "Measures of association for cross classifications." Journal of the American Statistical Association 49, 732-764. <doi:10.1080/01621459.1954.10501231>.
#' \cr \cr Goodman LA, Kruskal WH. (1963) "Measures of association for cross classifications III: Approximate sampling theory." Journal of the American Statistical Association 58, 310-364. <doi:10.1080/01621459.1963.10500850>.
#' * Unbiased variance estimator for `WP` and Wilson-type range -preserving confidence intervals are based on:
#' \cr \cr Brunner E, Konietschke F. (2025) "An unbiased rank-based estimator of the Mann–Whitney variance including the case of ties." Statistical Papers 66: 20. <doi:10.1007/s00362-024-01635-0>.
#' \cr \cr Schüürhuis S, Konietschke F, Brunner E. (2025) "A New Approach to the Nonparametric Behrens–Fisher Problem With Compatible Confidence Intervals." Biometrical Journal 67.6. <doi:10.1002/bimj.70096>.
#' @examples
#' # Example 1
#' COVID19HCE <- hce(GROUP = COVID19$GROUP, TRTP = COVID19$TRTP)
#' calcWINS(COVID19HCE)
#' # Example 2
#' COVID19bHCE <- hce(GROUP = COVID19b$GROUP, TRTP = COVID19b$TRTP)
#' calcWINS(COVID19bHCE, ref = "Placebo", WOnull = 1.1, alpha = 0.01)
#' # Example 3
#' calcWINS(COVID19HCE, SE_WP_Type = "unbiased")$WP
#' calcWINS(COVID19HCE, SE_WP_Type = "biased")$WP
calcWINS.hce <- function(x, ...){
  Args <- base::list(...)
  x <- as_hce(x)
  x <- base::as.data.frame(x)
  
  if(!is.null(Args[["ref"]])) ref <- Args[["ref"]]
  else if ("P" %in% unique(x$TRTP)) ref <- "P"
  else ref <- unique(x$TRTP)[1]
  if(!base::is.null(Args[["alpha"]])) alpha <- Args[["alpha"]]
  else alpha <- 0.05
  if(!base::is.null(Args[["WOnull"]])) WOnull <- Args[["WOnull"]]
  else WOnull <- 1
  
  res <- calcWINS.data.frame(x = x, AVAL = "AVAL", TRTP = "TRTP", 
                             ref = ref, alpha = alpha, WOnull = WOnull, SE_WP_Type = Args[["SE_WP_Type"]])
  res$ref <- base::paste(unique(x$TRTP)[unique(x$TRTP) != ref],"vs", ref)
  res$Input <- data.frame(alpha = alpha, WOnull = WOnull)
  res
}

#' Win odds summary for a data frame
#'
#' @param x a data frame containing subject-level data.
#' @param AVAL variable in the data with ordinal analysis values.
#' @param TRTP the treatment variable in the data.
#' @param ref the reference treatment group.
#' @param GROUP an optional variable for grouping.
#' @param ... additional parameters.
#' @returns a list containing the summary of wins, losses, and ties. It contains the following named objects:
#' * summary a data frame containing number of wins, losses, and ties by treatment group and the overall number of comparisons.
#' * summary_by_GROUP (if `GROUP` variable is specified) a summary data frame by `GROUP`.
#' * WO calculated WO (win odds) and WP (win probability) and their standard errors.
#' @export
#' @md
#' @seealso [hce::calcWO()], [hce::summaryWO()], [hce::summaryWO.formula()], [hce::summaryWO.hce()]   methods.
#' @examples
#' summaryWO(x = HCE3, AVAL = "AVAL", TRTP = "TRTP", ref = "P", GROUP = "GROUP")
summaryWO.data.frame <- function(x, AVAL, TRTP, ref, GROUP = NULL, ...){
  data <- as.data.frame(x)
  ref <- ref[1]
  AVAL <- AVAL[1]
  TRTP <- TRTP[1]
  GROUP <- GROUP[1]
  if(! AVAL %in% base::names(data))
    stop(paste0("The variable ", AVAL, " is not in the dataset."))
  if(! TRTP %in% base::names(data))
    stop(paste0("The variable ", TRTP, " is not in the dataset."))
  data$AVAL <- data[, base::names(data) == AVAL]
  data$TRTP <- data[, base::names(data) == TRTP]
  if(!is.null(GROUP)){
    if(! GROUP %in% base::names(data))
      stop(paste0("The variable ", GROUP, " is not in the dataset."))
    data$GROUP <- data[, base::names(data) == GROUP]
   }
  if (length(unique(data$TRTP)) != 2) 
    stop("The dataset should contain two treatment groups.")
  if(! ref %in% unique(data$TRTP)) stop("Choose the reference from the values in TRTP.")

  data$TRTP <- base::ifelse(data$TRTP == ref, "P", "A")
  A <- base::rank(c(data$AVAL[data$TRTP == "A"], data$AVAL[data$TRTP == "P"]), ties.method = "average")
  B <- base::tapply(data$AVAL, data$TRTP, base::rank, ties.method = "average")
  n <- base::tapply(data$AVAL, data$TRTP, base::length)
  n1 <- n[["A"]]
  n0 <- n[["P"]]
  d <- base::data.frame(R1 = A, R2 = base::c(B$A, B$P), TRTP = base::c(base::rep("A", n1), base::rep("P", n0)), AVAL = data$AVAL)
  
  d$IWTN <- d$R1 - d$R2
  a1 <- base::rank(c(data$AVAL[data$TRTP == "A"], data$AVAL[data$TRTP == "P"]), ties.method = "max")
  a2 <- base::rank(c(data$AVAL[data$TRTP == "A"], data$AVAL[data$TRTP == "P"]), ties.method = "min")
  b1 <- base::tapply(data$AVAL, data$TRTP, base::rank, ties.method = "max")
  b2 <- base::tapply(data$AVAL, data$TRTP, base::rank, ties.method = "min")
  
  d$ITN <- (a1 - a2) - c(b1$A - b2$A, b1$P - b2$P)
  d$IWN <- d$IWTN - d$ITN*0.5
  d$ILN <- ifelse(d$TRTP == "A", n0 - d$IWN - d$ITN, n1 - d$IWN - d$ITN)
  
  DATA <- rbind(data[data$TRTP == "A", ], data[data$TRTP == "P", ])
  DATA[, c("IWN", "ITN", "ILN")] <- rbind(d[d$TRTP == "A", c("IWN", "ITN", "ILN")], d[d$TRTP == "P", c("IWN", "ITN", "ILN")])
  DATA$IWP <- base::ifelse(d$TRTP == "A", d$IWTN/n0, d$IWTN/n1)
  
  res <- stats::aggregate(x = DATA[, c("IWN", "ILN", "ITN")], 
                          by = list (TRTP = DATA$TRTP),  FUN = "sum")
  names(res) <- c("TRTP", "WIN", "LOSS", "TIE")
  res$TOTAL <- res$WIN + res$LOSS + res$TIE
  res$WR <- res$WIN / res$LOSS
  res$WO <- (res$WIN + 0.5*res$TIE)/ (res$LOSS + 0.5*res$TIE)
  
  WP0 <- base::tapply(DATA$IWP, DATA$TRTP, base::mean)
  VAR <- base::tapply(DATA$IWP, DATA$TRTP, function(x) (base::length(x) - 1)*stats::var(x)/base::length(x))
  SE_WP <- base::sqrt(base::sum(VAR/n))
  WP = WP0[["A"]]
  WO = WP/(1 - WP)
  SE <- SE_WP/(WP*(1 - WP))
  out <- base::data.frame(WO = WO, SE = SE, WP = WP, SE_WP = SE_WP)
  
  
  
  if(!is.null(GROUP)){
    res0 <- stats::aggregate(x = DATA[, c("IWN", "ILN", "ITN")], 
                             by = list (TRTP = DATA$TRTP, GROUP = DATA$GROUP),  FUN = "sum")
    names(res0) <- c("TRTP", "GROUP", "WIN", "LOSS", "TIE")
    res0$TOTAL <- res0$WIN + res0$LOSS + res0$TIE
    l <- list(summary = res, summary_by_GROUP = res0, WO = out)
  } else 
    l <- list(summary = res, WO = out)
  
  l
}

#' A generic function for win odds regression
#'
#' @param x an object used to select a method.
#' @param ... further arguments passed to or from other methods.
#'
#' @return a data frame containing calculated values.
#' @export
#' @md
#' @seealso [hce::regWO.data.frame()], [hce::regWO.formula()] methods.
regWO <- function(x, ...) {
  UseMethod("regWO")
}

test_that("WO for KHCE, hce - SAS", {
  expect_equal(calcWO(as_hce(KHCE))$WO, 1.3199846571681)
})
test_that("WO for KHCE 2, hce - SAS", {
  expect_equal(calcWINS(as_hce(KHCE))$WO$WO, 1.3199846571681)
})

test_that("WO for KHCE 3, hce - direct", {
  expect_equal(calcWINS(as_hce(KHCE))$WO$WO, (319841 + 0.5*401)/(242258 + 0.5*401))
})


test_that("WR for KHCE, hce - direct", {
  expect_equal(calcWINS(as_hce(KHCE))$WR1$WR, (319841 )/(242258 ))
})

test_that("WP for KHCE, hce - SAS", {
  expect_equal(calcWO(as_hce(KHCE))$WP, 0.5689626666667)
})

test_that("WP for KHCE 2, hce - SAS", {
  expect_equal(calcWINS(as_hce(KHCE))$WP$WP, 0.5689626666667)
})

test_that("WO for KHCE 3, hce - direct", {
  expect_equal(calcWINS(as_hce(KHCE))$WP$WP, (319841 + 0.5*401)/(562500))
})

test_that("WINS for KHCE, hce - direct", {
  expect_equal(as.numeric(calcWINS(as_hce(KHCE))$summary[c("WIN", "LOSS", "TIE")]), 
               c(319841, 242258, 401))
})

test_that("SE for WP for KHCE, hce - SAS", {
  expect_equal(calcWO(as_hce(KHCE))$SE_WP, 0.0147431746263)
})

test_that("SE for WP for KHCE 2, hce - SAS", {
  expect_equal(calcWINS(as_hce(KHCE))$SE$WP_SE, 0.0147431746263)
})

test_that("p-value for WP for KHCE, hce - SAS", {
  expect_equal(calcWO(as_hce(KHCE))$Pvalue, 0.000002902527050)
})

test_that("p-value for WP for KHCE 2, hce - SAS", {
  expect_equal(calcWINS(as_hce(KHCE))$WP$Pvalue, 0.000002902527050)
})

test_that("p-value for WP for KHCE, hce - hce", {
  expect_equal(calcWINS(as_hce(KHCE), SE_WP_Type = "unbiased")$WP$Pvalue, calcWINS(x = KHCE, AVAL = "AVAL", TRTP = "TRTP", ref = "P", SE_WP_Type = "unbiased")$WP$Pvalue)
})


test_that("p-value for WP for KHCE 2, hce - hce", {
  expect_equal(calcWINS(data = COVID19, GROUP ~ TRTP, ref = "Placebo", SE_WP_Type = "unbiased")$WP$Pvalue, 
               calcWINS(x = COVID19, AVAL = "GROUP", TRTP = "TRTP", ref = "Placebo", SE_WP_Type = "unbiased")$WP$Pvalue)
})

test_that("gamma for WP for KHCE, SAS", {
  expect_equal(calcWINS(as_hce(KHCE))$gamma$gamma, 0.1380237289161)
})

test_that("SE for gamma for WP for KHCE, SAS", {
  expect_equal(calcWINS(as_hce(KHCE))$SE$gamma_SE, 0.0295069564073)
})

test_that("Order does not affect the WO regression", {
  expect_equal(regWO(x = KHCE, AVAL = "GROUP", TRTP = "TRTP", COVAR = "EGFRBL", ref = "P"), 
               regWO(x = KHCE[sample(1:nrow(KHCE), replace = FALSE), ], AVAL = "GROUP", TRTP = "TRTP", COVAR = "EGFRBL", ref = "P"))
})

test_that("Order does not affect the WO stratified regression", {
  expect_equal(stratWO(x = KHCE, AVAL = "AVAL", COVAR = "EGFRBL", 
                       TRTP = "TRTP", STRATA = "STRATAN", ref = "P"), 
               stratWO(x = KHCE[sample(1:nrow(KHCE), replace = FALSE),], AVAL = "AVAL", COVAR = "EGFRBL", 
                       TRTP = "TRTP", STRATA = "STRATAN", ref = "P"))
})


test_that("Order does not affect the WO stratification", {
  expect_equal(stratWO(x = KHCE, AVAL = "AVAL",  
                       TRTP = "TRTP", STRATA = "STRATAN", ref = "P"), 
               stratWO(x = KHCE[sample(1:nrow(KHCE), replace = FALSE),], AVAL = "AVAL", 
                       TRTP = "TRTP", STRATA = "STRATAN", ref = "P"))
})

test_that("Order does not affect the IWP calculations", {
  expect_equal(
    IWP(data = KHCE, AVAL = "EGFRBL", TRTP = "TRTPN", ref = 2),
    IWP(data = KHCE[sample(1:nrow(KHCE), replace = FALSE),], 
                AVAL = "EGFRBL", TRTP = "TRTPN", ref = 2)
  )
})

test_that("WP based on the IWP calculation matches the calcWO results", {
  expect_equal(
    mean(IWP(data = KHCE, AVAL = "EGFRBL", TRTP = "TRTPN", ref = 2)$EGFRBL_[IWP(data = KHCE, AVAL = "EGFRBL", TRTP = "TRTPN", ref = 2)$TRTP=="A"]),
    calcWO(EGFRBL ~ TRTP, data = KHCE)[c("WP", "SE_WP")]$WP
  )
})



test_that("SE for the adjusted WP comparison to the sanon package", {
  expect_equal(
    regWO(AVAL ~ TRTP + EGFRBL, data = KHCE[sample(1:nrow(KHCE), replace = FALSE),])$SE_beta*sqrt(nrow(KHCE)/(nrow(KHCE) - 1)),
    0.0147476911574
  )
})


test_that("Adjusted WP comparison to the sanon package", {
  expect_equal(
    regWO(AVAL ~ TRTP + EGFRBL, data = KHCE)$beta,
    0.569107747079
  )
})

FREQ <- c(16, 5, 0, 1, 0, 4, 1, 5, 7, 2)
dat0 <- data.frame(AVAL = rep(5:1, 2), TRTP = rep(c('A', 'P'), each = 5))
dat <- dat0[rep(row.names(dat0), FREQ),]

test_that("Brunner-Konietschke variance comparison with a published value", {
  expect_equal(
    calcWINS(AVAL ~ TRTP, data = dat,  SE_WP_Type = "unbiased")$SE$WP_SE^2,
    0.0041708562
  )
})

#' Simulate a kidney disease `hce` dataset
#'
#' Simulate a kidney disease `hce` dataset, capturing eGFR (Estimated Glomerular Filtration Rate) progression over time, along with 
#' a competing and dependent terminal event: KFRT (Kidney Failure Replacement Therapy)
#' 
#' @param n sample size in the active treatment group.
#' @param CM_A annualized eGFR slope in the active group.
#' @param CM_P annualized eGFR slope in the control group.
#' @param n0 sample size in the control treatment group.
#' @param TTE_A event rate per year in the active group for KFRT.
#' @param TTE_P event rate per year in the placebo group for KFRT. 
#' @param fixedfy length of follow-up in years.
#' @param Emin lower limit of eGFR at baseline.
#' @param Emax upper limit of eGFR at baseline.
#' @param sigma within-patient standard deviation.
#' @param Sigma between-patient standard deviation.
#' @param m number of equidistant visits.
#' @param theta coefficient of dependence of eGFR values and the risk of KFRT.
#' @param phi coefficient of proportionality (between 0 and 1) of the treatment effect. The case of 0 corresponds to the uniform treatment effect.
#' @param two_meas determines whether to use duplicate measurements at baseline and/or at `fixedfy`. The default is to use a single measurement.
#' @param df_sigma degrees of freedom for the measurement error distribution. The default `Inf` gives normal measurement errors; must be more than 2.
#' @details
#' The default setting is `TTE_A = TTE_P` because, conditional on eGFR level,
#' the treatment effect does not influence the event rate of KFRT. In this model,
#' the effect of treatment on KFRT operates entirely through its impact on eGFR decline.
#'
#' Let `CM_A` and `CM_P` be denoted by \eqn{\beta_A} and \eqn{\beta_P}, respectively.
#' Let `TTE_A` and `TTE_P` be denoted by \eqn{\lambda_A} and \eqn{\lambda_P}, respectively.
#' Let `theta` be denoted by \eqn{\theta}, `phi` by \eqn{\phi}, `sigma` by \eqn{\sigma},
#' `Sigma` by \eqn{\Sigma}, and `df_sigma` by \eqn{\nu}.
#'
#' The parameters `TTE_A` and `theta` are chosen so that when GFR is 15, the event rate
#' is 1 per patient per year, and when GFR is 25, the event rate is 0.01 per patient per year. These
#' parameter values are obtained by solving
#' \deqn{
#' \lambda_g \exp(\theta \cdot \mathrm{GFR}) = \mathrm{rate}
#' }
#' for the group-specific baseline rate \eqn{\lambda_g} and \eqn{\theta}. When the observed eGFR is above 30,
#' the event rate is set to a very low value (`10e-7`), while when the observed eGFR is below or equal to 7,
#' the event rate is set to a very high value (`10e5`). This ensures that patients with observed low eGFR values
#' always experience KFRT, while those with high eGFR values do not.
#'
#' By default, the within-patient standard deviation, `sigma`, is set to `NULL`. When left as `NULL`, `sigma`
#' is calculated as
#' \deqn{
#' \sigma_{ij} = \sqrt{0.67 \cdot \mu_{ij}},
#' }
#' where \eqn{\mu_{ij}} denotes the predicted eGFR for patient \eqn{i} at visit \eqn{j}. This yields
#' time-dependent measurement variability, with lower predicted eGFR values corresponding to lower variability.
#' In the implementation, to prevent the variance from becoming negative or too small, the quantity
#' \eqn{0.67 \cdot \mu_{ij}} is truncated below at `0.1` before taking the square root.
#'
#' When `df_sigma = Inf` (the default), measurement errors are normally distributed:
#' \deqn{
#' \varepsilon_{ij} \sim N(0, \sigma_{ij}^2).
#' }
#' When `df_sigma` is finite and greater than 2, measurement errors follow a Student t distribution
#' with heavier tails:
#' \deqn{
#' \varepsilon_{ij} = \sigma_{ij}\sqrt{\frac{\nu - 2}{\nu}} T_{\nu},
#' }
#' where \eqn{\nu =} `df_sigma` and \eqn{T_{\nu}} is a standard t random variable with \eqn{\nu}
#' degrees of freedom. This scaling preserves the same time-dependent standard deviation defined by `sigma`,
#' so that
#' \deqn{
#' \mathrm{Var}(\varepsilon_{ij}) = \sigma_{ij}^2.
#' }
#'
#' Given the overall effect `Delta` and the placebo progression rate `CM_P`, a fully uniform (purely additive)
#' treatment effect—meaning the same average effect applies to all patients regardless of baseline progression—is
#' implemented by setting \eqn{\phi = 0} and
#' \deqn{
#' \beta_A = \Delta + \beta_P.
#' }
#' A fully proportional treatment effect—no additive component, the effect scales with the baseline rate—is
#' implemented by setting
#' \deqn{
#' \beta_A = \beta_P
#' }
#' and
#' \deqn{
#' \phi = \frac{\Delta}{|\beta_P|}.
#' }
#' A more relativistic intermediate effect (half additive and half proportional) is obtained by setting
#' \deqn{
#' \phi = \frac{\Delta}{2|\beta_P|}
#' }
#' and
#' \deqn{
#' \beta_A = \frac{\Delta}{2} + \beta_P.
#' }
#'
#' The kidney hierarchical composite endpoint is defined in the following order:
#' (1) Kidney Failure Replacement Therapy (KFRT); (2) Sustained eGFR < 15;
#' (3) Sustained 57 percent or more decline in eGFR; (4) Sustained 50 percent or more decline in eGFR;
#' (5) Sustained 40 percent or more decline in eGFR; and (6) Change in eGFR. In practice, because KFRT is
#' frequently initiated when true eGFR is very low, sustained eGFR < 15 events are rarely observed.
#'
#' If `two_meas` is not `"no"`, a second measurement is generated using the same latent baseline eGFR and
#' subject-specific slope, but with an independent measurement error draw from the same distribution as above.
#' The two measurements are then averaged at baseline when `two_meas = "base"`, at the final visit when
#' `two_meas = "postbase"`, or at both baseline and the final visit when `two_meas = "both"`. This implementation
#' preserves correlation between the duplicate measurements because they share the same underlying true trajectory.
#' @export
#' @md
#' @seealso [hce::simHCE()] for a general function of simulating `hce` datasets.
#' @examples
#' # Example - Specifying the most important variables
#' set.seed(2022)
#' ## The overall treatment effect
#' Delta <- 0.75
#' ## The placebo progression rate
#' CM_P <- - 4.5
#' ## Intermediate effect (half additive and half proportional)
#' delta <- Delta/2
#' CM_A <- delta + CM_P 
#' phi <- Delta / (2*abs(CM_P))
#' L <- simKHCE(n = 1000, CM_A = CM_A, CM_P = CM_P, 
#' fixedfy = 4, Emin = 25, Emax = 75, phi = phi)
#' dat <- L$HCE
#' calcWO(dat)
simKHCE <- function(n, CM_A, CM_P = - 4, n0 = n, TTE_A = 1000, TTE_P = TTE_A,  
                    fixedfy = 2, Emin = 20, Emax = 100, 
                    sigma = NULL, Sigma = 3,
                    m = 10, theta = - .4605, phi = 0, two_meas = c("no", "base", "postbase", "both"), df_sigma = Inf){
  two_meas <- match.arg(two_meas)
  n <- n[1] 
  CM_A <- CM_A[1] 
  CM_P <- CM_P[1] 
  n0 <- n0[1]
  TTE_A <- TTE_A[1]
  TTE_P <- TTE_P[1]  
  fixedfy <- fixedfy[1] 
  Emin <- Emin[1]
  Emax <- Emax[1]
  sigma <- sigma[1]
  Sigma <- Sigma[1]
  m <- round(m)[1]
  theta <- theta[1]
  phi <- phi[1]
  df_sigma <- df_sigma[1]
  stopifnot("The standard deviation `Sigma` should be positive." 
            = all(c(Sigma > 0)), 
            "`phi` should be in the interval [0, 1]" = all(c(phi >= 0, phi <= 1)),
            "GFR range `Emin` and `Emax` should be non-negative and `Emin` < `Emax`." = all(c(Emin >= 0, Emax >= 0, Emin < Emax)),
            "`m` must be >= 1" = m >= 1,
            "The measurement error degrees of freedom, `df_sigma`, must be numeric and greater than 2, or `Inf` for normal errors." = 
              is.numeric(df_sigma) && df_sigma > 2)
  if(!is.null(sigma)) stopifnot("The standard deviation `sigma` must be positive or left `NULL` for data-driven estimates." = sigma > 0)
  
  N <- n0 + n
  b0 <- CM_P
  b1 <- CM_A - CM_P
  c0 <- log(TTE_P)
  c1 <- log(TTE_A/TTE_P)
  c2 <- theta
  # Patient ID and treatment group
  d <- data.frame(ID = 1:N, TRTPN = c( rep(1, n),  rep(0, n0)))
  # Random patient-level slope with a given between-patient variability Sigma^2
  d$SLOPE <- b0 + stats::rnorm(N, mean = 0, sd = Sigma)
  # Implement proportionality for patients in the active group and negative (true) random slope
  d$SLOPE <- ifelse(d$TRTPN == 1 & d$SLOPE < 0, d$SLOPE*(1 - phi), d$SLOPE)
  # Implement additive effect (slope difference) for the patients in the active group
  d$SLOPE <-  d$SLOPE + b1*d$TRTPN 
  # Uniform random (true) baseline GFR from a given range. The observed baseline BASE will be different because of the sampling error (within-patient variability).
  d$BASE0 <- stats::runif(N, Emin, Emax)
  # Equidistant visit times. The number of visits is m + 1. Includes both the baseline visit and the visit at the end of the timeframe.
  tj <- (0:m)*fixedfy/m
  # Generate visits for all patients
  VISITS <- data.frame(ID = rep(1:N, each = m + 1), ADAY = rep(tj, times = N)) 
  # Merge to get the treatment groups
  d1 <- merge(VISITS, d, all.x = TRUE, by = "ID")
  # Random true slope is observed with a measurement error (within-patient variability is sigma^2)
  # sigma, by default, is derived based on the formula 0.67*predicted eGFR, which is time dependent.
  if(is.null(sigma)){
    VAR <- 0.67*(d1$SLOPE * d1$ADAY + d1$BASE0)
    VAR[VAR <= 0.1] <- 0.1 #prevent the variance being negative or too small.
    sigma <- sqrt(VAR)
  }
  if(is.infinite(df_sigma)){
    # If df_sigma is infinite, we use the normal distribution for the measurement error, which is the default.
    BASE <- d1$BASE0 + stats::rnorm(nrow(d1), sd = sigma)
  } else {
    # If df_sigma is finite, we use the t distribution for the measurement error, which has heavier tails than the normal distribution.
    BASE <- d1$BASE0 + sigma*sqrt((df_sigma - 2)/df_sigma)*stats::rt(nrow(d1), df = df_sigma)
  }
  d1$AVAL <- d1$SLOPE*d1$ADAY + ifelse(BASE <= Emin, Emin, ifelse(BASE >= Emax, Emax, BASE))
  d1$AVAL[d1$AVAL < 0] <- 0
  ######## Added a new implementation to handle double measurements at baseline or at PADY. #########
  if(two_meas != "no"){
    # BASE0 and SLOPE are the same for both measurements, hence they are correlated. ########
    if(is.infinite(df_sigma)){
      # If df_sigma is infinite, we use the normal distribution for the measurement error, which is the default.
      BASE1 <- d1$BASE0 + stats::rnorm(nrow(d1), sd = sigma)
    } else {
      # If df_sigma is finite, we use the t distribution for the measurement error, which has heavier tails than the normal distribution.
      BASE1 <- d1$BASE0 + sigma*sqrt((df_sigma - 2)/df_sigma)*stats::rt(nrow(d1), df = df_sigma)
    }
    d1$AVAL1 <- d1$SLOPE * d1$ADAY + ifelse(BASE1 <= Emin, Emin, ifelse(BASE1 >= Emax, Emax, BASE1))  
  }
  if(two_meas == "base"){
    d1$AVAL <- ifelse(d1$ADAY== 0, (d1$AVAL + d1$AVAL1)/2, d1$AVAL)
  } else if(two_meas == "postbase"){
    d1$AVAL <- ifelse(d1$ADAY == fixedfy, (d1$AVAL + d1$AVAL1)/2, d1$AVAL)
  } else if(two_meas == "both"){
    d1$AVAL <- ifelse(d1$ADAY == 0 | d1$ADAY == fixedfy, (d1$AVAL + d1$AVAL1)/2, d1$AVAL)
  }
  ################### END of the addition #########################
  # Extract time 0 measurements as baseline (observed baseline with sampling error)
  d2 <- d1[d1$ADAY == 0, c("ID", "AVAL")]
  names(d2) <- c("ID", "BASE")
  ## Simulate uniform random variables (should be subject specific!)
  d2$U <- stats::runif(nrow(d2))
  # Merge with the baseline variables (observed = BASE, true = BASE0)
  d3 <- merge(d1, d2, by = "ID", all.x = T)
  # Create patient-level KFRT incidence rates based on the true random slope
  ## Constant event-rates per group
  d3$RATE0 <- exp(c0 + c1*d3$TRTPN)
  ## Multiply the group-level incidence rates depending on the patient-level random (true) GFR slope at each visit interval.
  d3$RATE <- d3$RATE0*exp(c2*(d3$SLOPE*d3$ADAY + d3$BASE0))
  ## Make sure that patients with higher than 30 observed eGFR cannot have KFRT
  ## Make surer that patients with lower than 5 observed eGFR always have KFRT
  d3$RATE <- ifelse(d3$AVAL > 30, 10E-7, ifelse(d3$AVAL <= 7, 10E5, d3$RATE))
  ## Calculate the cumulative event rate for each patient
  ### CUMRATE must be aligned with the intended interval indexing per ID; otherwise, hazard accumulation can be wrong.
  d3 <- d3[order(d3$ID, d3$ADAY), ]
  d3$PADY <- fixedfy
  dt <- fixedfy/m
  d3$CUMHAZ <- stats::ave(d3$RATE*dt, d3$ID, FUN = cumsum)
  d3$V <- -log(d3$U)
  d4 <- d3[d3$V <= d3$CUMHAZ, ] #selects the intervals in which events happen
  ## Cumulative event rate minus the event rate in the current interval gives the cumulative event rate up to the current interval
  ## Use the cumulative event rate up to the current interval multiplied by the length of all intervals
  ## Use the log uniform added to it and divided by the rate of the current interval
  ## Add it to the current visit ADAY to initiate the event after the current visit
  if(nrow(d4) > 0){
    ## Order to correctly select the first event
    d4 <- d4[order(d4$ID, d4$ADAY), ]
    ## Select the first event
    d4 <- d4[!duplicated(d4$ID), ] 
    ## Generate the time of the first event
    d4$AVALT <- (d4$V - (d4$CUMHAZ - d4$RATE*dt))/d4$RATE + d4$ADAY
    ## d3$V <= d3$CUMHAZ selection above guarantees that d4$AVALT - d4$ADAY is less than dt
    ## Hence, by construction the below should not be needed, but protects against rounding issues.
    d4$AVALT <- pmin(d4$AVALT, d4$PADY)
    ##########################
    d5 <- merge(d3, d4[, c("ID", "AVALT")], all.x = TRUE, by = "ID")
    ## if event time is missing, then assign censoring.
    d5$EVENT <- NA
    d5$EVENT <- ifelse(is.na(d5$AVALT), 0, 1)
    d5[is.na(d5$AVALT), "AVALT"] <- fixedfy
  } else {
    d5 <- d3
    d5$EVENT <- 0
    d5$AVALT <- fixedfy
  }
  d5[, c("SLOPE", "BASE0", "RATE", "RATE0")] <- NULL
  # Calculate change and percent changes from baseline
  d5$CHG <- d5$AVAL - d5$BASE
  d5$PCHG <- (d5$AVAL / d5$BASE - 1)*100
  # Remove observations after the generated event (the event is kidney failure hence precludes measurements of GFR)
  d5 <- d5[d5$ADAY <= d5$AVALT, ]
  # Sort the dataset
  ADLB <- d5[order(d5$ID, d5$ADAY), ]
  # Derive sustained decline based on the laboratory measurements 
  d <- ADLB
  d$E40 <- ifelse(d$PCHG <= -40, 1, 0)
  d$E50 <- ifelse(d$PCHG <= -50, 1, 0)
  d$E57 <- ifelse(d$PCHG <= -57, 1, 0)
  d$E15 <- ifelse(d$AVAL < 15, 1, 0)
  sustained <- function(dat, x){
    dat <- as.data.frame(dat)
    dat <- dat[order(dat$ADAY), ]
    row.names(dat) <- NULL
    thr0 <- as.numeric(substr(x, 2, 3))
    thr <- ifelse(x == "E15", thr0, - thr0)
    I <- which(dat[, x] == 1)
    if(length(I) == 0)
      return(NULL)
    for(k in 1:length(I)){
      i <- I[k]
      t0 <- dat[i, "ADAY"]
      # use one month for the confirmation of the sustained decline
      t1 <- t0 + 1/12
      j <- which(dat$ADAY >= t1)[1]
      
      if(is.na(i) | is.na(j)) 
        next
      # Requires that all measurements during the window satisfy the criteria
      if(x %in% c("E57", "E50", "E40") & all(dat[i:j, "PCHG"] <= thr)){
        dat0 <- dat[i, c("ID", "ADAY", x)]
        names(dat0) <- c("ID", "ADAY", "EVENT")
        dat0$EVENT <- x
        return(dat0)
      }
      if(x %in% c("E15") & all(dat[i:j, "AVAL"] < thr)){
        dat0 <- dat[i, c("ID", "ADAY", x)]
        names(dat0) <- c("ID", "ADAY", "EVENT")
        dat0$EVENT <- x
        return(dat0)
      }
    }
    return(NULL)
  }
  l <- lapply(split(d, d$ID), sustained, x = "E40")
  E1 <- do.call(rbind, l)
  l <- lapply(split(d, d$ID), sustained, x = "E50")
  E2 <- do.call(rbind, l)
  l <- lapply(split(d, d$ID), sustained, x = "E57")
  E3 <- do.call(rbind, l)
  l <- lapply(split(d, d$ID), sustained, x = "E15")
  E4 <- do.call(rbind, l)
  ## The dataset of all events
  ADET0 <- rbind(E1, E2, E3, E4)
  if(!is.null(ADET0)){
    names(ADET0) <- c("ID", "AVAL0", "PARAMCD")  
  }
  ### filters GFR based events by severity
  E3 <- E3[!E3$ID %in% E4$ID, ]
  E2 <- E2[!E2$ID %in% c(E3$ID, E4$ID), ]
  E1 <- E1[!E1$ID %in% c(E2$ID, E3$ID, E4$ID), ]
  E <- rbind(E3, E2, E1, E4)
  if(!is.null(E)){
    names(E) <- c("ID", "AVAL0", "PARAMCD")  
  }
  ### adds change in GFR for those without a GFR based event
  d0 <- d[!d$ID %in% E$ID & d$ADAY == d$PADY, c("ID", "CHG")]
  if(nrow(d0) > 0){
    names(d0) <- c("ID", "AVAL0")
    d0$PARAMCD <- "GFR"  
  }
  ## Combines patients with GFR based events with patients without events to include their change in GFR at the end of follow-up
  d1 <- rbind(d0, E)
  ### In this step (which could have been much earlier), we select kidney failure replacement therapy events
  d2 <- unique(d[d$EVENT == 1, c("ID", "AVALT")])
  if(nrow(d2) > 0){
    d2$PARAMCD <- "KFRT"
    names(d2) <- c("ID", "AVAL0", "PARAMCD")  
  }
  ### Merges KFRT events in d2 with the other events / change in GFR values in d1 (removing those with KFRT from the other events to avoid duplication)
  d3 <- rbind(d1[!d1$ID %in% d2$ID, ], d2)
  ## Merges to add event names
  PARAM <- data.frame(PARAMCD = c("KFRT", "E15", "E57", "E50", "E40", "GFR"), PARAMN = 1:6)
  d4 <- merge(d3, PARAM, by = "PARAMCD", all.x = TRUE)
  d4 <- d4[, c("ID", "PARAMCD", "PARAMN", "AVAL0")]
  d4 <- d4[order(d4$ID), ]
  ############## Merges to have BASE GFR in the dataset
  d5 <- unique(d[, c("ID", "TRTPN", "PADY", "BASE")])
  d6 <- merge(d5, d4, by = "ID")
  ## Finalize ADLB by creating a character variable for treatment
  ADLB$TRTP <- ifelse(ADLB$TRTPN == 1, "A", "P")
  ######### Create final HCE dataset
  HCE <- d6
  HCE$TRTP <- ifelse(HCE$TRTPN == 1, "A", "P")
  HCE$GROUP <- factor(HCE$PARAMCD, levels = c("KFRT", "E15", "E57", "E50", "E40", "GFR"))
  levels(HCE$GROUP) <- c("Kidney Failure Replacement Therapy", "Sustained eGFR < 15 (mL/min/1.73 m2)", 
                         "Sustained >= 57% decline in eGFR", "Sustained >= 50% decline in eGFR", 
                         "Sustained >= 40% decline in eGFR", "Change in eGFR")
  ####### Create the final event-time dataset
  ADET <- rbind(ADET0, d2)
  if(!is.null(ADET)) ADET <- ADET[order(ADET$ID), ]
  ###############
  ### Clean the final GFR dataset
  ADLB[, c("U", "CUMHAZ", "V")] <- NULL
  ######## Create the final list of results
  L <- list(GFR = ADLB, ADET = ADET, HCE = as_hce(HCE))
  return(L)
}

#' Simulate ordinal variables for two treatment groups using categorization of beta distributions
#'
#' @param n sample size in the active treatment group.
#' @param n0 sample size in the placebo group.
#' @param M number of ordinal values to be simulated.
#' @param alpha1 shape1 parameter for the beta distribution in the active group.
#' @param beta1 shape2 parameter for the beta distribution in the active group.
#' @param alpha0 shape1 parameter for the beta distribution in the placebo group.
#' @param beta0 shape2 parameter for the beta distribution in the placebo group.
#' @return a data frame containing the following columns:
#' * ID subject identifier.
#' * TRTP planned treatment group - "A" for active, "P" for Placebo.
#' * GROUPN ordinal values. The number of unique values is specified by the variable `M0`.
#' * tau the theoretical win odds.
#' * theta the theoretical win probability.
#' @export
#' @md
#' @seealso [hce::simHCE()] for simulating `hce` objects.
#' @examples
#' # Example 1
#' set.seed(2024)
#' alpha1 <- 8
#' beta1 <- 8
#' alpha0 <- 4
#' beta0 <- 5
#' d <- simORD(n = 1500, n0 = 1500, M = 5, alpha1 = alpha1, beta1 = beta1, 
#' alpha0 = alpha0, beta0 = beta0)
#' x <- seq(0, 1, 0.01)
#' plot(x, dbeta(x, shape1 = alpha1, shape2 = beta1), 
#' type = "l", ylab = "Density of beta distribution", col = 2)
#' lines(x, dbeta(x, shape1 = alpha0, shape2 = beta0), col = 3, lty = 2)
#' legend("topleft", lty = c(1, 2), col = c(2, 3), legend = c("Control", "Active"))
#' D <- hce(GROUP = d$GROUPN, TRTP = d$TRTP)
#' table(D$TRTP, D$GROUP)
#' calcWO(D)

#' # Example 2
#' set.seed(2024)
#' d <- simORD(n = 100, n0 = 50, M = 2)
#' d_hce <- hce(GROUP = d$GROUPN, TRTP = d$TRTP)
#' calcWO(d_hce)
#' ### compare with the theoretical values of the continuous distributions 
#' c(tau = unique(d$tau), theta = unique(d$theta))
#' # Example 2 - Convergence of the win odds to its theoretical value
#' set.seed(2024)
#' N <- NULL
#' size <- c(seq(10, 500, 1))
#' for(i in size){
#'   d <- simORD(n = i, M = 2)
#'   d_hce <- hce(GROUP = d$GROUPN, TRTP = d$TRTP)
#'   TAU <- calcWO(d_hce)
#'   D <- data.frame(WO = TAU$WO, n = i, tau = unique(d$tau))
#'   N <- rbind(N, D)
#' }
#' plot(N$n, N$WO, log = "y", ylim = c(0.5, 2), ylab = "Win Odds", xlab = "Sample size", type = "l")
#' lines(N$n, N$tau, col = "darkgreen", lty = 2, lwd = 2)
#' abline(h = 1, lty = 4, col = "red")
#' legend("bottomright", legend = c("Theoretical Win Odds", "Null", "Win Odds Estimate"), 
#' lty = c(4, 2, 1), col = c("darkgreen", "red", "black"))
#' title("Convergence of the win odds to its theoretical value")

simORD <- function(n, n0 = n, M, alpha1 = 8, beta1 = 7, alpha0 = 5, beta0 = 5){
  n <- n[1]
  n0 <- n0[1]
  M <- M[1]
  alpha0 <- alpha0[1]
  alpha1 <- alpha1[1]
  beta0 <- beta0[1]
  beta1 <- beta1[1]

  x <- seq(from = 0, to = 1, length.out = M + 1)
  tildeY <- 1 - (stats::pbeta(x[-1], shape1 = alpha1, shape2 = beta1) + stats::pbeta(x[-length(x)], shape1 = alpha1, shape2 = beta1))/2
  tildeX <- (stats::pbeta(x[-1], shape1 = alpha0, shape2 = beta0) - stats::pbeta(x[-length(x)], shape1 = alpha0, shape2 = beta0))
  theta <- sum(tildeX*tildeY)
  tau <- theta/(1 - theta)
  X0 <- stats::rbeta(n = n0, shape1 = alpha0, shape2 = beta0)
  Y0 <- stats::rbeta(n = n, shape1 = alpha1, shape2 = beta1)
  x0 <- seq(from = 0, to = 1, length.out = M + 1)
  X1 <- cut(X0, breaks = x0)
  levels(X1) <- 1:M
  X1 <- as.numeric(X1)
  Y1 <- cut(Y0, breaks = x0)
  levels(Y1) <- 1:M
  Y1 <- as.numeric(Y1)
  d <- data.frame(ID = 1:(n0 + n), GROUPN = c(Y1, X1), TRTP = c(rep("A", n), rep("P", n0)), tau = tau, theta = theta)
  return(d)
}

#' Win odds summary for `adhce` objects
#'
#' @param x an `adhce` object.
#' @param ... additional parameters.
#' @returns a list containing the summary of wins, losses, and ties. It contains the following named objects:
#' * summary a data frame containing number of wins, losses, and ties by treatment group and the overall number of comparisons.
#' * summary_by_GROUP a summary data frame by `GROUP`.
#' * WO calculated WO (win odds) and WP (win probability) and their standard errors.
#' * cumsummary_by_GROUP a cumulative summary data frame by `GROUP`.
#' @export
#' @md
#' @seealso [hce::calcWO()], [hce::summaryWO()], [hce::summaryWO.data.frame()], [hce::summaryWO.formula()], [hce::summaryWO.hce()] methods.
#' @examples
#' ## Example 1 - using an `hce` object
#' HCE5 <- HCE4
#' HCE5$PADY <- NULL
#' dat <- as_hce(HCE5)
#' ## `PADY` is not present in the dataset, hence converts it to an `hce` object 
#' ## instead of an `adhce` object.
#' ## Example 2 - Using an `adhce` object
#' class(dat)
#' summaryWO(dat, ref = "P")
#' ## The class is `adhce` hence will use the variable `GROUP`.
#' HCE5$PADY <- 1080
#' dat <- as_hce(HCE4) 
#' class(dat)
#' summaryWO(dat, ref = "P")
#' ## Example 3 - Plotting cumulative components of an `adhce` object
#' dat <- as_hce(KHCE)
#' res0 <- summaryWO(dat, ref = "P")
#' res <- res0$cumsummary_by_GROUP
#' barplot(PROP ~ WINS + GROUPN, data = res, 
#' col = c("darkgreen", "darkred", "darkblue"), 
#' xlab = "Proportions", xlim = c(0, 1), 
#' ylab = "Cumulative components by prioritization", 
#' legend.text = unique(res$WINS), beside = TRUE, horiz = TRUE)
#' grid()
summaryWO.adhce <- function(x, ...){
  Args <- base::list(...)
  x <- as_hce(x)
  x <- base::as.data.frame(x)
  if(!is.null(Args[["ref"]])) ref <- Args[["ref"]]
  else if ("P" %in% unique(x$TRTP)) ref <- "P"
  else ref <- unique(x$TRTP)[1]
  r0 <- summaryWO.data.frame(x = x, AVAL = "AVAL", TRTP = "TRTP", ref = ref, GROUP = "GROUP")
  r1_ <- r0$summary_by_GROUP
  # Check that when some GROUPS levels are not present, they are added as zeros
  rx <- expand.grid(TRTP = c("A", "P"), GROUP = levels(x$GROUP))
  r1 <- merge(rx, r1_, by = c("TRTP", "GROUP"), all.x = TRUE)
  r1[is.na(r1)] <- 0
  ###########
  r2 <- tapply(r1$LOSS, r1$TRTP, cumsum)
  TOTAL <- r0$summary$TOTAL[1]
  GROUP <- levels(x$GROUP)
  LAB <- c(paste(unique(x$TRTP)[unique(x$TRTP) != ref], "wins"), paste(ref, "wins"), "Ties")
  r3 <- data.frame(GROUPN = rep(1:length(GROUP), GROUP = rep(GROUP, times = 3), times = 3), 
                   WINS = rep(LAB, each = length(GROUP)),
                   COUNT = c(r2$P, r2$A, TOTAL - r2$A - r2$P), TOTAL = TOTAL)
  r3$PROP <- r3$COUNT/r3$TOTAL
  r3 <- r3[order(r3$GROUPN), ]
  r0$cumsummary_by_GROUP <- r3
  r0
}

#' Sample size calculation for the win ratio test (with WR = 1 null hypothesis)
#'
#' @param WR a numeric vector of win odds values.
#' @param power the given power. A numeric vector of length 1.
#' @param WO win odds. Should be specified only if `Pties` is not specified. A numeric vector of length 1.
#' @param Pties probability of ties. A numeric vector of length 1.
#' @param k proportion of active group in the overall sample size. Default is 0.5 (balanced randomization). A numeric vector of length 1.
#' @param alpha the significance level for the 2-sided test. Default is 0.05. A numeric vector of length 1.
#'
#' @return a data frame containing the sample size with input values.
#' @export
#' @md
#' @seealso [hce::sizeWO()] for WO sample size calculation.
#' @references Yu RX, Ganju J. (2022) "Sample size formula for a win ratio endpoint." Statistics in Medicine, 41.6: 950-63. <doi:10.1002/sim.9297>.
#' @examples
#' sizeWR(WR = 1.35, Pties = 0.125, power = 0.8)
#' sizeWR(WR = 1.35, WO = 1.3, power = seq(0.5, 0.9, 0.05))

sizeWR <- function(WR, power, WO = NULL, Pties = NULL, k = 0.5, alpha = 0.05){
  alpha <- alpha[1]
  k <- k[1]
  Ca <- stats::qnorm(1 - alpha/2)
  Cpow <- stats::qnorm(power)
  if(k < 0 | k > 1) stop("k should be between 0 and 1.")
  if(is.null(WO) & is.null(Pties)) stop("Either win odds `WO` or the probability of ties `Pties` should be specified.")
  if(!is.null(WO) & !is.null(Pties)) stop("Either win odds `WO` or the probability of ties `Pties` should be specified.")
  if(is.null(Pties)) {
    WP <- WO/(WO + 1)
    Pties <- 1 - (WR + 1)*(2 * WP - 1)/(WR - 1)
  }
  if(is.null(WO)) {
    NB <- (WR - 1)/(WR + 1)*(1 - Pties)
    WP <- (NB + 1)/2
    WO <- WP/(1 - WP)
  }
  coef <- 4*(1 + Pties)/(3*k*(1 - k)*(1 - Pties))
  N <- coef * (Ca + Cpow)^2 / log(WR)^2
  base::data.frame(WR = WR, Pties = Pties, WO = WO, power = power, 
                   SampleSize = base::ceiling(N), 
                   alpha = alpha)
}

#' Win Odds Regression Using a Data Frame
#'
#' This function performs regression analysis for the win odds using a single numeric covariate.
#'
#' @param x a data frame containing subject-level data.
#' @param AVAL a variable in the data with ordinal analysis values.
#' @param TRTP the treatment variable in the data.
#' @param COVAR a numeric covariate.
#' @param ref the reference treatment group.
#' @param alpha the significance level, with a default value of 0.05.
#' @param WOnull the null hypothesis value for win odds. The default is 1.
#' @param ... additional parameters.
#' @returns a data frame containing the calculated win odds and its confidence interval, including: 
#' * WO_beta adjusted win odds.
#' * LCL lower confidence limit for adjusted WO.
#' * UCL upper confidence limit for adjusted WO.
#' * SE standard error of the adjusted win odds.
#' * WOnull win odds of the null hypothesis (specified in the `WOnull` argument).
#' * alpha two-sided significance level for calculating the confidence interval (specified in the `alpha` argument).
#' * Pvalue p-value associated with testing the null hypothesis.
#' * N total number of patients in the analysis.
#' * beta adjusted win probability.
#' * LCL_beta lower confidence limit for adjusted win probability.
#' * UCL_beta upper confidence limit for adjusted win probability.
#' * SE_beta standard error for the adjusted win probability.
#' * SD_beta standard deviation for the adjusted win probability.
#' * WP (non-adjusted) win probability.
#' * SE_WP standard error of the non-adjusted win probability.
#' * SD_WP standard deviation of the non-adjusted win probability.
#' * WO non-adjusted win odds.
#' * COVAR_MEAN_DIFF mean difference between two treatment groups of the numeric covariate.
#' * COVAR_VAR sum of variances of two treatment groups of the numeric covariate.
#' * COVAR_COV covariance between the response and the numeric covariate.
#' @export
#' @md
#' @seealso [hce::regWO()], [hce::regWO.formula()].
#' @references Gasparyan SB et al. (2021) "Adjusted win ratio with stratification: calculation methods and interpretation." Statistical Methods in Medical Research 30.2: 580-611. <doi:10.1177/0962280220942558>.
#' @examples
#' # A baseline covariate that is highly correlated with the outcome
#' set.seed(2023)
#' dat <- COVID19
#' n <- nrow(dat)
#' dat$Severity <- ifelse(dat$GROUP > 4, rnorm(n, 0), rnorm(n, 100))
#' tapply(dat$Severity, dat$TRTP, mean)
#' regWO(x = dat, AVAL = "GROUP", TRTP = "TRTP", COVAR = "Severity", ref = "Placebo")
#' # Without adjustment
#' calcWO(x = dat, AVAL = "GROUP", TRTP = "TRTP", ref = "Placebo")
regWO.data.frame <- function(x, AVAL, TRTP, COVAR, ref, alpha = 0.05, WOnull = 1, ...){
  data <- as.data.frame(x)
  alpha <- alpha[1]
  WOnull <- WOnull[1]
  TRTP <- TRTP[1]
  COVAR <- COVAR[1]
  AVAL <- AVAL[1]
  ref <- ref[1]
  WPnull <- WOnull/(WOnull + 1)
  Ca <- stats::qnorm(1 - alpha/2)
  
  data$AVAL <- data[, base::names(data) == AVAL]
  data$TRTP <- data[, base::names(data) == TRTP]
  data$COVAR <- data[, base::names(data) == COVAR]
  if(length(unique(data$TRTP)) != 2) {
    message1 <- base::paste("The variable", TRTP, "should have exactly 2 unique values, but has", length(unique(data$TRTP)))
    stop(message1)
  }
  if(!ref %in% unique(data$TRTP)) stop("Choose the reference from the values in TRTP.")
  data$TRTP <- base::ifelse(data$TRTP == ref, "P", "A")
  if(!is.numeric(data$COVAR)) {
    message2 <- base::paste("The variable", COVAR, "should be numeric.")
    stop(message2)
  }
 
  A <- base::rank(c(data$AVAL[data$TRTP == "A"], data$AVAL[data$TRTP == "P"]), ties.method = "average")
  B <- base::tapply(data$AVAL, data$TRTP, base::rank, ties.method = "average")
  n <- base::tapply(data$AVAL, data$TRTP, base::length)
  n1 <- n[["A"]]
  n0 <- n[["P"]]
  N <- n0 + n1
  d <- base::data.frame(R1 = A, R2 = base::c(B$A, B$P), 
                        TRTP = base::c(base::rep("A", n1), base::rep("P", n0)), 
                        COVAR = c(data$COVAR[data$TRTP == "A"], data$COVAR[data$TRTP == "P"]))
  d$R <- d$R1 - d$R2
  d$R0 <- base::ifelse(d$TRTP == "A", d$R/n0, d$R/n1)
  
  WP0 <- base::tapply(d$R0, d$TRTP, base::mean)
  WP <- WP0[["A"]]
  VAR <- base::tapply(d$R0, d$TRTP, function(x) (base::length(x)-1)*stats::var(x)/base::length(x))
  SE_WP <- base::sqrt(base::sum(VAR/n))
  
  
  MEAN_COVAR <- base::tapply(d$COVAR, d$TRTP, base::mean)
  VAR_COVAR <- base::tapply(d$COVAR, d$TRTP, function(x) (base::length(x)-1)*stats::var(x)/base::length(x))
  C0 <- sapply(split(d[, c("COVAR", "R0")], d$TRTP), 
               function(d) (length(d$COVAR) - 1)*stats::cov(d$COVAR, d$R0)/length(d$COVAR))
  C1 <- sum(C0/c(n1, n0))
  
  beta <- WP - C1*(MEAN_COVAR["A"] - MEAN_COVAR["P"])/sum(VAR_COVAR/c(n1, n0))
  SE_beta2 <-  SE_WP^2 - C1^2/sum(VAR_COVAR/c(n1, n0))
  SE_beta <- sqrt(SE_beta2)
  
  WO_beta <- beta/(1 - beta)
  SE <- SE_beta/(beta*(1 - beta))
  LCL <- WO_beta*base::exp(- Ca*SE)
  UCL <- WO_beta*base::exp(Ca*SE)
  threshold <- base::abs(beta - WPnull)/SE_beta
  P <- 2*(1 - stats::pnorm(threshold))
  
  out <- base::data.frame(WO_beta = WO_beta, LCL = LCL, UCL = UCL, 
                          SE = SE, WOnull = WOnull, 
                          alpha = alpha, Pvalue = P, N = N,
                          beta = beta, LCL_beta = beta - Ca*SE_beta, UCL_beta = beta + Ca*SE_beta, 
                          SE_beta = SE_beta, SD_beta = SE_beta*sqrt(N),
                          WP = WP, SE_WP = SE_WP, SD_WP = SE_WP*sqrt(N), WO = WP/(1 - WP))
  
  out$COVAR_MEAN_DIFF <- MEAN_COVAR["A"] - MEAN_COVAR["P"]
  out$COVAR_VAR <- sum(VAR_COVAR/c(n1, n0))
  out$COVAR_COV <- C1
  
  return(out)
}

#' Win odds calculation based on a threshold for `adhce` objects
#'
#' @param x an `adhce` object.
#' @param delta a numeric threshold. 
#' @param ref the reference treatment group.
#' @param alpha  significance level. The default is 0.05.
#' @param WOnull the null hypothesis. The default is 1.
#' @param ...  additional parameters.
#' @returns a data frame containing the win odds and its confidence interval. It contains the following columns:
#' * WO calculated win odds.
#' * LCL lower confidence limit.
#' * UCL upper confidence limit.
#' * SE standard error of the win odds.
#' * WOnull win odds of the null hypothesis (specified in the `WOnull` argument).
#' * alpha two-sided significance level for calculating the confidence interval (specified in the `alpha` argument).
#' * Pvalue p-value associated with testing the null hypothesis.
#' * WP calculated win probability.
#' * LCL_WP lower confidence limit for `WP`.
#' * UCL_WP upper confidence limit for `WP`.
#' * SE_WP standard error of the win probability.
#' * SD_WP standard deviation of the win probability, calculated as `SE_WP` multiplied by `sqrt(N)`.
#' * N total number of patients in the analysis.
#' * ref the reference group.
#' * delta the threshold.
#' @export
#' @md
#' @seealso [hce::deltaWO()], [hce::calcWO()].
#' @examples
#' # Example using the kidney dataset
#' dat <- as_hce(KHCE)
#' calcWO(dat, ref = "P")
#' ## The exact same result
#' deltaWO(dat, delta = 0, ref = "P")
#' ## A large threshold
#' deltaWO(dat, delta = 10, ref = "P")
#' 
deltaWO.adhce <- function (x, delta, ref = unique(x$TRTP)[1], alpha = 0.05, WOnull = 1, ...) 
{
  # Even if the object has the class `adhce`, still recreate it to catch all issues.
  dat <- as_hce(x)
  delta <- delta[1]
  
  base::stopifnot("The threshold `delta` must be nonnegative." = delta >= 0)

  alpha <- alpha[1]
  WOnull <- WOnull[1]
  WPnull <- WOnull/(WOnull + 1)
  ref <- ref[1]
  SEV <- base::levels(base::factor(dat$GROUP))
  LAST <- SEV[base::length(SEV)]
  
  dat$AVAL01 <- dat$AVAL02 <- dat$AVAL0
  dat$AVAL01[dat$GROUP == LAST & dat$TRTP == ref] <- dat$AVAL0[dat$GROUP == LAST & dat$TRTP == ref] - delta
  dat$AVAL02[dat$GROUP == LAST & dat$TRTP == ref] <- dat$AVAL0[dat$GROUP == LAST & dat$TRTP == ref] + delta
  mcont1 <- base::min(dat[dat$GROUP == LAST, "AVAL01"])
  mcont2 <- base::min(dat[dat$GROUP == LAST, "AVAL02"])
  dat$AVAL1 <- base::ifelse(dat$GROUP != LAST, dat$AVAL01 + dat$GROUPN, 
                      dat$AVAL01 - mcont1 + 1 + dat$GROUPN)
  dat$AVAL2 <- base::ifelse(dat$GROUP != LAST, dat$AVAL02 + dat$GROUPN, 
                      dat$AVAL02 - mcont2 + 1 + dat$GROUPN)
  
  dat1 <- IWP(data = dat, AVAL = "AVAL1", TRTP = "TRTP", ref = ref)
  ## Brings back the original treatment assignment given that IWP() converts the treatment to "A" and "P"
  dat1_ <- merge(dat1[names(dat) != "TRTP"], dat[, c("ID", "TRTP")], all.x = TRUE, by = "ID")
  dat2 <- IWP(data = dat1_, AVAL = "AVAL2", TRTP = "TRTP", ref = ref)
  dat2$AVAL_ <- (dat2$AVAL1_ + dat2$AVAL2_)/2
  
  n <- base::tapply(dat2$AVAL1_, dat2$TRTP, base::length)
  SE_WP <- sqrt(sum(tapply(dat2$AVAL_, dat2$TRTP, function(x) (length(x) - 1)*stats::var(x)/base::length(x))/n))
  WP1 <- base::tapply(dat2$AVAL1_, dat2$TRTP, base::mean)
  WP2 <- base::tapply(dat2$AVAL2_, dat2$TRTP, base::mean)
  ## Will always be "A" and "P" because of IWP()
  WP <- (WP1["A"] + WP2["A"])/2
  WO <- WP/(1 - WP)
  
  SD_WP <- SE_WP*base::sqrt(sum(n))
  SE <- SE_WP/(WP*(1 - WP))
  Ca <- stats::qnorm(1 - alpha/2)
  LCL <- WO*base::exp(- Ca*SE)
  UCL <- WO*base::exp(Ca*SE)
  
  LCL_WP <- WP - Ca*SE_WP
  UCL_WP <- WP + Ca*SE_WP
  threshold <- base::abs(WP - WPnull)/SE_WP
  P <- 2*(1 - stats::pnorm(threshold))
  
  out <- base::data.frame(WO = WO, LCL = LCL, UCL = UCL, SE = SE, WOnull = WOnull, alpha = alpha, 
                          Pvalue = P, WP = WP, LCL_WP  = LCL_WP, UCL_WP = UCL_WP, SE_WP = SE_WP, 
                          SD_WP = SD_WP, N = sum(n), ref = ref, delta = delta)
  return(out)
  
}

#' Stratified win odds with adjustment
#'
#' @param x a data frame containing subject-level data.
#' @param AVAL variable in the data with ordinal analysis values.
#' @param TRTP the treatment variable in the data.
#' @param STRATA a character variable for stratification.
#' @param ref the reference treatment group.
#' @param COVAR a numeric covariate.
#' @param alpha the reference treatment group.
#' @param WOnull the null hypothesis. The default is 1.
#' @param ... additional parameters.
#' @returns a data frame containing the following columns:
#' * WO stratified (or adjusted/stratified) win odds.
#' * LCL lower confidence limit for adjusted (or adjusted/stratified) WO.
#' * UCL upper confidence limit for adjusted (or adjusted/stratified) WO.
#' * SE standard error of the adjusted (or adjusted/stratified) win odds.
#' * WOnull win odds of the null hypothesis (specified in the `WOnull` argument).
#' * alpha two-sided significance level for calculating the confidence interval (specified in the `alpha` argument).
#' * Pvalue p-value associated with testing the null hypothesis.
#' * WP adjusted (or adjusted/stratified) win probability.
#' * LCL_WP lower confidence limit for adjusted (or adjusted/stratified) WP.
#' * UCL_WP upper confidence limit for adjusted (or adjusted/stratified) WP.
#' * SE_WP standard error for the adjusted (or adjusted/stratified) win probability.
#' * SD_WP standard deviation of the adjusted (or adjusted/stratified) win probability.
#' * N total number of patients in the analysis.
#' * Type "STRATIFIED" or "STRATIFIED/ADJUSTED" depending on whether `COVAR` is specified.
#' @export
#' @md
#' @seealso [hce::stratWO()].
#' @references Gasparyan SB et al. (2021) "Adjusted win ratio with stratification: calculation methods and interpretation." Statistical Methods in Medical Research 30.2: 580-611. <doi:10.1177/0962280220942558>.
#' @examples
#' # Stratified win odds
#' res <- stratWO(x = KHCE, AVAL = "AVAL", TRTP = "TRTP", 
#'                STRATA = "STRATAN", ref = "P")
#' res
#' ## Compare with the non-stratified win odds
#' res0 <- calcWO(AVAL ~ TRTP, data = KHCE,  ref = "P")
#' res0
#' ## Compare with the win odds in each stratum separately
#' l <- lapply(split(KHCE, KHCE$STRATAN), calcWO, AVAL = "AVAL", TRTP = "TRTP", ref = "P")
#' l <- do.call(rbind, l)
#' l <- l[, c("WO", "LCL", "UCL", "N")]
#' l$STRATA <- as.numeric(row.names(l))
#' plot(y = l$WO, x = l$STRATA, ylim = c(0.5, 2.5), log = "y", xlim = c(0, 6),
#'       ylab = "Win Odds", xlab = "", xaxt = "n")
#' axis(1, at = 1:6, labels = c(paste0("STR = ", l$STRATA), "Stratified", "Non-stratified"))
#' arrows(l$STRATA, l$LCL, l$STRATA, 
#'        l$UCL, angle = 90, code = 3, length = 0.05, col = "darkgreen")
#' points(5, res$WO)
#' arrows(5, res$LCL, 5, res$UCL, angle = 90, code = 3, 
#'        length = 0.05, col = "darkblue")
#' abline(h = c(1, res$WO), col = "red", lty = 4)
#' points(6, res0$WO)
#' arrows(6, res0$LCL, 6, res0$UCL, angle = 90, code = 3, 
#'        length = 0.05, col = "darkred")
#' # Stratified and adjusted win odds
#' res <- stratWO(x = KHCE, AVAL = "AVAL", COVAR = "EGFRBL", 
#'               TRTP = "TRTP", STRATA = "STRATAN", ref = "P")
#' res
#' 
stratWO.data.frame <- function(x, AVAL, TRTP, STRATA, ref, COVAR = NULL, alpha = 0.05, WOnull = 1, ...){
  data <- as.data.frame(x)
  alpha <- alpha[1]
  ref <- ref[1]
  WOnull <- WOnull[1]
  AVAL <- AVAL[1]
  STRATA <- STRATA[1]
  COVAR <- COVAR[1]
  WPnull <- WOnull/(WOnull + 1)
  Ca <- stats::qnorm(1 - alpha/2)
  

  if(! AVAL %in% base::names(data))
    stop(paste0("The variable ", AVAL, " is not in the dataset."))
  if(! TRTP %in% base::names(data))
    stop(paste0("The variable ", TRTP, " is not in the dataset."))
  if(! STRATA %in% base::names(data))
    stop(paste0("The variable ", STRATA, " is not in the dataset."))
  
      
  data$AVAL <- data[, base::names(data) == AVAL]
  data$TRTP <- data[, base::names(data) == TRTP]
  data$STRATA <- data[, base::names(data) == STRATA]
  
  if (length(unique(data$TRTP)) != 2) 
    stop("The dataset should contain two treatment groups.")
  if (!ref %in% unique(data$TRTP)) 
    stop("Choose the reference from the values in TRTP.")
  data$TRTP <- base::ifelse(data$TRTP == ref, "P", "A")

  l <- split(data, data$STRATA)
  
  if(!is.null(COVAR)){
    data$COVAR <- data[, base::names(data) == COVAR]
    
    if(! COVAR %in% base::names(data))
      stop(paste0("The variable ", COVAR, " is not in the dataset."))
    if(!is.numeric(data$COVAR)) 
      stop("COVAR should be numeric.") 
    
    ll2 <- lapply(l, regWO, AVAL = "AVAL", TRTP = "TRTP", ref = "P", COVAR = COVAR, alpha = alpha, WOnull = WOnull)
    STR2 <- do.call(rbind, ll2)
    STR2$STRATA <- row.names(STR2)
    row.names(STR2) <- NULL
    
    STR_tab <- table(data$STRATA, data$TRTP)
    STR_frame <- data.frame(STRATA = row.names(STR_tab), TRT_A = STR_tab[, "A"], TRT_P = STR_tab[, "P"])
    row.names(STR_frame) <- NULL
    STR0 <- merge(STR2, STR_frame, by = "STRATA")
    STR0$omega <- STR0$TRT_A*STR0$TRT_P/STR0$N
    STR0$omega0 <- sum(STR0$omega)
    STR0$weight <- STR0$omega/STR0$omega0
    
    WP_str <- sum(STR0$WP * STR0$weight)
    SE_WP_str <- sqrt(sum((STR0$SE_WP * STR0$weight)^2))
    adjust <- sum(STR0$weight * STR0$COVAR_MEAN_DIFF)*sum(STR0$weight^2 * STR0$COVAR_COV)/sum(STR0$weight^2 * STR0$COVAR_VAR)
    adjust_VAR <- sum(STR0$weight^2 * STR0$COVAR_COV)^2/sum(STR0$weight^2 * STR0$COVAR_VAR)
    WP <- WP_str - adjust
    SE_WP <- sqrt(SE_WP_str^2 - adjust_VAR)
   
    threshold <- abs(WP - WPnull)/SE_WP
    P <- 2 * (1 - stats::pnorm(threshold))
    WO <- WP/(1 - WP) 
    SE <-  SE_WP/(WP*(1 - WP))
    LCL <- WO*exp(- Ca *SE)
    UCL <- WO*exp(Ca *SE)
    
    res <- data.frame( WO = WO, LCL = LCL, UCL = UCL, SE = SE, WOnull = WOnull, alpha = alpha, Pvalue = P, 
                       WP = WP, LCL_WP = WP - Ca*SE_WP, UCL_WP = WP + Ca*SE_WP, SE_WP = SE_WP, SD_WP = SE_WP*sqrt(sum(STR0$N)), N = sum(STR0$N))
    res$Type <- "STRATIFIED/ADJUSTED"
    
  } else{
    ll <- lapply(l, calcWO, AVAL = "AVAL", TRTP = "TRTP", ref = "P", alpha = alpha, WOnull = WOnull)
    
    STR <- do.call(rbind, ll)
    STR$STRATA <- row.names(STR)
    row.names(STR) <- NULL
    
    STR_tab <- table(data$STRATA, data$TRTP)
    STR_frame <- data.frame(STRATA = row.names(STR_tab), TRT_A = STR_tab[, "A"], TRT_P = STR_tab[, "P"])
    row.names(STR_frame) <- NULL
    STR0 <- merge(STR, STR_frame, by = "STRATA")
    STR0$omega <- STR0$TRT_A*STR0$TRT_P/STR0$N
    STR0$omega0 <- sum(STR0$omega)
    STR0$weight <- STR0$omega/STR0$omega0
    
    WP_str <- sum(STR0$WP * STR0$weight)
    SE_WP_str <- sqrt(sum((STR0$SE_WP * STR0$weight)^2))
    threshold <- abs(WP_str - WPnull)/SE_WP_str
    P <- 2 * (1 - stats::pnorm(threshold))
    WO_str <- WP_str/(1 - WP_str) 
    SE_str <-  SE_WP_str/(WP_str*(1 - WP_str))
    LCL_str <- WO_str*exp(- Ca *SE_str)
    UCL_str <- WO_str*exp(Ca*SE_str)
    
    res <- data.frame( WO = WO_str, LCL = LCL_str, UCL = UCL_str, SE = SE_str, WOnull = WOnull, alpha = alpha, Pvalue = P, 
                       WP = WP_str, LCL_WP = WP_str - Ca*SE_WP_str, UCL_WP = WP_str + Ca*SE_WP_str, SE_WP = SE_WP_str, SD_WP = SE_WP_str*sqrt(sum(STR0$N)), N = sum(STR0$N))
    res$Type <- "STRATIFIED"
  }
  
  return(res)
  
}

#' Win odds calculation using formula syntax
#'
#' @param x an object of class formula.
#' @param data a data frame.
#' @param ... additional parameters.
#' @returns a data frame containing the win odds and its confidence interval. It contains the following columns:
#' * WO calculated win odds.
#' * LCL lower confidence limit.
#' * UCL upper confidence limit.
#' * SE standard error of the win odds.
#' * WOnull win odds of the null hypothesis (specified in the `WOnull` argument).
#' * alpha two-sided significance level for calculating the confidence interval (specified in the `alpha` argument).
#' * Pvalue p-value associated with testing the null hypothesis.
#' * WP calculated win probability.
#' * LCL_WP lower confidence limit for `WP`.
#' * UCL_WP upper confidence limit for `WP`.
#' * SE_WP standard error of the win probability.
#' * SD_WP standard deviation of the win probability, calculated as `SE_WP` multiplied by `sqrt(N)`.
#' * N total number of patients in the analysis.
#' * formula returning the specified formula in the `x` argument.
#' * ref showing how the reference group was selected. Can be modifying by specifying the `ref` argument.
#' @export
#' @md
#' @seealso [hce::calcWO()], [hce::calcWO.hce()], [hce::calcWO.data.frame()].
#' @references Gasparyan SB et al. "Adjusted win ratio with stratification: calculation methods and interpretation." Statistical Methods in Medical Research 30.2 (2021): 580-611. <doi:10.1177/0962280220942558>.
#' @examples
#' #Example 1
#' data(HCE1)
#' calcWO(AVAL ~ TRTP, data = HCE1)
#'
#'#Example 2
#' calcWO(GROUP ~ TRTP, data = COVID19, ref = "Placebo", alpha = 0.01)
#'
calcWO.formula <- function(x, data, ...){
  Args <- base::list(...)
  formulavars <- base::all.vars(x)
  formula0 <- stats::reformulate(formulavars[2], response = formulavars[1])
  mf <- stats::model.frame(formula = formula0, data = data)
  Level <- base::unique(mf[[formulavars[2]]])
  Levellength <- base::length(Level)

    if(Levellength != 2){
    message1 <- base::paste("The variable", formulavars[2], "should have exactly 2 unique values, but has", Levellength)
    stop(message1)
  }
  if(!base::is.null(Args[["ref"]])) ref <- Args[["ref"]]
  else if (base::all(Level == c("A", "P"))) ref <- "P"
  else ref <- Level[1]
  if(! ref %in% Level) stop(base::paste("Choose the reference from the values",
                                  base::paste(Level, collapse = ", ")))

  if(!base::is.null(Args[["alpha"]])) alpha <- Args[["alpha"]]
  else alpha <- 0.05
  if(!base::is.null(Args[["WOnull"]])) WOnull <- Args[["WOnull"]]
  else WOnull <- 1
  
  res <- calcWO.data.frame(x = mf, AVAL = formulavars[1], TRTP = formulavars[2], 
                           ref = ref, alpha = alpha, WOnull = WOnull)
  res$formula <- base::deparse(formula0)
  res$ref <- base::paste(Level[Level != ref],"vs", ref)
  res
}

#' Win Odds Regression Using a Formula Syntax
#'
#' This function performs regression analysis for the win odds using a single numeric covariate.
#' 
#' @param x an object of class formula.
#' @param data a data frame.
#' @param ... additional parameters.
#' @returns a data frame containing the calculated win odds and its confidence interval, including: 
#' * WO_beta adjusted win odds.
#' * LCL lower confidence limit for adjusted WO.
#' * UCL upper confidence limit for adjusted WO.
#' * SE standard error of the adjusted win odds.
#' * WOnull win odds of the null hypothesis (specified in the `WOnull` argument).
#' * alpha two-sided significance level for calculating the confidence interval (specified in the `alpha` argument).
#' * Pvalue p-value associated with testing the null hypothesis.
#' * N total number of patients in the analysis.
#' * beta adjusted win probability.
#' * LCL_beta lower confidence limit for adjusted win probability.
#' * UCL_beta upper confidence limit for adjusted win probability.
#' * SE_beta standard error for the adjusted win probability.
#' * SD_beta standard deviation for the adjusted win probability.
#' * WP (non-adjusted) win probability.
#' * SE_WP standard error of the non-adjusted win probability.
#' * SD_WP standard deviation of the non-adjusted win probability.
#' * WO non-adjusted win odds.
#' * COVAR_MEAN_DIFF mean difference between two treatment groups of the numeric covariate.
#' * COVAR_VAR sum of variances of two treatment groups of the numeric covariate.
#' * COVAR_COV covariance between the response and the numeric covariate.
#' * formula returning the specified formula in the `x` argument.
#' * ref showing how the reference group was selected. Can be modifying by specifying the `ref` argument.
#' @export
#' @md
#' @seealso [hce::regWO()], [hce::regWO.data.frame()].
#' @references Gasparyan SB et al. (2021) "Adjusted win ratio with stratification: calculation methods and interpretation." Statistical Methods in Medical Research 30.2: 580-611. <doi:10.1177/0962280220942558>.
#' @examples
#' regWO(AVAL ~ TRTP, data = KHCE)
#' regWO(AVAL ~ TRTP + EGFRBL, data = KHCE)
regWO.formula <- function (x, data, ...) 
{ # Extra arguments to modify the default values of the data frame call
  Args <- base::list(...)
  # The names in the formula x
  formulavars <- base::all.vars(x)
  # Recreate a formula from a character vector by dropping unnecessary terms
  if (length(formulavars) > 2){
    formula0 <- stats::reformulate(formulavars[2:3], response = formulavars[1])
  } else {
    formula0 <- stats::reformulate(formulavars[2], response = formulavars[1])
  }
  # Subsets the data frame to keep only the variables in the formula
  mf <- stats::model.frame(formula = formula0, data = data)
  if (length(formulavars) > 2){
    Level <- base::unique(mf[[formulavars[2]]])
    if (!base::is.null(Args[["ref"]])) 
      ref <- Args[["ref"]]
    else if (base::all(Level == c("A", "P"))) 
      ref <- "P"
    else ref <- Level[1]
    if (!base::is.null(Args[["alpha"]])) 
      alpha <- Args[["alpha"]]
    else alpha <- 0.05
    if (!base::is.null(Args[["WOnull"]])) 
      WOnull <- Args[["WOnull"]]
    else WOnull <- 1
    res <- regWO.data.frame(x = mf, AVAL = formulavars[1], TRTP = formulavars[2], COVAR = formulavars[3], 
                 ref = ref, alpha = alpha, WOnull = WOnull)
    res$formula <- base::deparse(formula0)
    res$ref <- base::paste(Level[Level != ref], "vs", ref)
  } else {
    res <- calcWO.formula(x = formula0, data = data, ...)
    res$formula <- base::deparse(formula0)
  }
  return(res)
}

#' A generic function for stratified win odds with adjustment
#'
#' @param x an object used to select a method.
#' @param ... further arguments passed to or from other methods.
#'
#' @return a list containing the stratified results and results by strata.
#' @export
#' @md
#' @seealso [hce::stratWO.data.frame()]  methods.
stratWO <- function(x, ...) {
  UseMethod("stratWO")
}

#' Simulate an `hce` object 
#'
#' Simulate an `hce` object with multiple, possibly correlated (Hougaard copula) Weibull time‑to‑event outcomes and a single continuous endpoint (normal or log‑normal)
#'
#'
#' @param n sample size in the active treatment group.
#' @param n0 sample size in the placebo group.
#' @param TTE_A event rates per year in the active group for the time-to-event outcomes.
#' @param TTE_P event rates per year in the placebo group for the time-to-event outcomes. Should have the same length as `TTE_A`.
#' @param CM_A mean value for the continuous outcome of the active group.
#' @param CM_P mean value for the continuous outcome of the placebo group.
#' @param CSD_A standard deviation for the continuous outcome of the active group.
#' @param CSD_P standard deviation for the continuous outcome of the placebo group.
#' @param fixedfy length of follow-up in years.
#' @param yeardays number of days in a year.
#' @param pat scale of provided event rates (per `pat`-years).
#' @param shape shape of the Weibull distribution for time-to-event outcomes. Default is exponential distribution with `shape = 1`.
#' @param theta Gumbel dependence coefficient of the Weibull distributions for time-to-event outcomes. Default is `theta = 1` which assumes independence of time-to-event outcomes. Must be above or equal to 1.
#' @param logC logical, whether to use log-normal distribution for the continuous outcome.
#' @param seed for generating random numbers.
#' @param dec decimal places for the continuous outcome used for rounding. The default is `dec = 2`.
#' @param all_data logical, whether to return source datasets `ADET` (an event-time dataset for all time-to-event outcomes per patient) and `BDS` (a basic data structure for the continuous outcome for all patients).
#' @return an object of class `hce` containing the following columns:
#' * ID subject identifier.
#' * TRTP planned treatment group - "A" for active, "P" for Placebo.
#' * GROUP type of the outcome, either "TTE" for time-to-event outcomes or "C" for continuous. 
#' Only one continuous outcome is possible, but no restriction on the number of "TTE" outcomes.
#' * GROUPN order of outcomes in `GROUP`, with a higher value signifying a better outcome.
#' * AVALT the timing of the time-to-event outcomes.
#' * AVAL0 numeric values of the continuous outcome and the timing of "TTE" outcomes.
#' * AVAL analysis values derived as `AVAL0 + GROUPN`. For the continuous outcome the values of `AVAL0` are shifted to start always from 0.
#' * seed the seed of the random sample. If not specified in `seed` argument will be selected based on system time.
#' * PADY primary analysis day, the length of fixed follow-up in days calculated as `yeardays` multiplied by `fixedfy`.
#' 
#' If `all_data = TRUE`, the function returns a list containing the `hce` dataset, along with its source datasets: `ADET` (an event-time dataset for all time-to-event outcomes per patient) and `BDS` (a basic data structure for the continuous outcome for all patients).
#' @export
#' @md
#' @seealso [hce::hce()], [hce::as_hce()] for the helper a coerce function to `hce` objects.
#' @examples
#' # Example 1
#' Rates_A <- c(1.72, 1.74, 0.58, 1.5, 1)
#' Rates_P <- c(2.47, 2.24, 2.9, 4, 6)
#' dat <- simHCE(n = 2500, TTE_A = Rates_A, TTE_P = Rates_P,
#'               CM_A = -3, CM_P = -6, CSD_A = 16, CSD_P = 15, fixedfy = 3)
#' head(dat)
#' 
#'# Example 2
#' Rates_A <- 10
#' Rates_P <- 15
#' dat <- simHCE(n = 1000, n0 = 500, TTE_A = Rates_A, TTE_P = Rates_P, 
#'               CM_A = 0.1, CM_P = 0, seed = 5, shape = 0.2, logC = TRUE, dec = 0)
#' summaryWO(dat)
#' 
#' # Example 3: Comparison of dependent and independent outcomes
#' Rates_A <- c(10, 20)
#' Rates_P <- c(20, 20)
#' dat1 <- simHCE(n = 2500, TTE_A = Rates_A, TTE_P = Rates_P, 
#' CM_A = -3, CM_P = -6, CSD_A = 15, fixedfy = 3, theta = 1, seed = 1)
#' dat2 <- simHCE(n = 2500, TTE_A = Rates_A, TTE_P = Rates_P, 
#' CM_A = -3, CM_P = -6, CSD_A = 15, fixedfy = 3, theta = 1.0001, seed = 1)
#' calcWO(dat1)
#' calcWO(dat2)


simHCE <- function(n, n0 = n, TTE_A, TTE_P, CM_A, CM_P, CSD_A = 1, CSD_P = CSD_A, fixedfy = 1, yeardays = 360, pat = 100, shape = 1, theta = 1, logC = FALSE, seed = NULL, dec = 2, all_data = FALSE){
  if(base::length(TTE_P) != base::length(TTE_A))
    stop("Event rate vectors for active and placebo groups should have the same length.")
  stopifnot("`theta` must be greater than or equal to 1." = theta >= 1)
  n <- n[1]
  n0 <- n0[1]
  CM_A <- CM_A[1]
  CM_P <- CM_P[1]
  CSD_A <-  CSD_A[1]
  CSD_P <-  CSD_P[1]
  fixedfy <- fixedfy[1]
  yeardays <- yeardays[1]
  pat <- pat[1]
  seed <- seed[1]
  logC <- as.logical(logC[1])
  all_data <- as.logical(all_data[1])
  shape <- shape[1]
  dec <- dec[1]
  theta <- theta[1]

  fixedf <- yeardays*fixedfy
  TTE_P <- TTE_P/yeardays/pat
  TTE_A <- TTE_A/yeardays/pat
  l <- base::length(TTE_A)
  if(is.null(seed)){
    seed <- base::as.numeric(base::Sys.time())
    seed <- base::round(seed)
  }
  seed <- seed[1]
  base::set.seed(seed)
  invW <- function(u, scale, shape) {
    scale * (- log( 1 - u))^(1/shape)
  }
  if(theta == 1){
    M_P <- base::sapply(1/TTE_P, stats::rweibull, n = n0, shape = shape)
    M_A <- base::sapply(1/TTE_A, stats::rweibull, n = n, shape = shape)
  } else if(theta > 1){
    Y <- rGumbel(n, theta = theta, dim = l)
    X <- rGumbel(n0, theta = theta, dim = l)
    
    d <- data.frame(AVAL = as.numeric(Y), RATE = rep(TTE_A, each = n), TTE = rep(1:l, each = n))
    d0 <- lapply(split(d, d$TTE), function(x) invW(x[, 1], scale = 1/x[, 2], shape = shape))
    M_A <- do.call(cbind, d0)
    dimnames(M_A) <- NULL
    d <- data.frame(AVAL = as.numeric(X), RATE = rep(TTE_P, each = n0), TTE = rep(1:l, each = n0))
    d0 <- lapply(split(d, d$TTE), function(x) invW(x[, 1], scale = 1/x[, 2], shape = shape))
    M_P <- do.call(cbind, d0)
    dimnames(M_P) <- NULL
  }
  M_P <- round(M_P)
  M_P <- base::as.data.frame(M_P)
  base::names(M_P) <- base::paste0("TTE", 1:l)
  M_P$TRTP <- "P"
  M_P$ID <- (n + 1):(n0 + n)
  M_A <- round(M_A)
  M_A <- base::as.data.frame(M_A)
  base::names(M_A) <- base::paste0("TTE", 1:l)
  M_A$TRTP <- "A"
  M_A$ID <- 1:n
  dat <- base::rbind(M_A, M_P)
  dat$PADY <- fixedf
  dat$seed <- seed
  
  dat$AVALT <- base::apply(dat[, base::startsWith(names(dat), "TTE"), drop = FALSE], 1, function(x) x[x <= fixedf][1])
  dat$AVALT[is.na(dat$AVALT)] <- fixedf
  dat$EVENTN <- base::apply(dat[, base::startsWith(names(dat), "TTE"), drop = FALSE], 1, function(x) which(x <= fixedf)[1])
  dat$EVENTN[is.na(dat$EVENTN)]  <- l + 1

  dat$EVENT <- base::factor(dat$EVENTN, levels = 1:(l+1))
  base::levels(dat$EVENT) <- base::c(paste0("TTE", 1:l), "C")
  dat$GROUP <- base::as.character(dat$EVENT)
  dat$GROUPN <- dat$EVENTN*dat$PADY

  lXA <- round(stats::rlnorm(n, meanlog = CM_A, sdlog = CSD_A), dec)
  lXP <- round(stats::rlnorm(n0, meanlog = CM_P, sdlog = CSD_P), dec)
  XA <- round(stats::rnorm(n, mean = CM_A, sd = CSD_A), dec)
  XP <- round(stats::rnorm(n0, mean = CM_P, sd = CSD_P), dec)
  
  if(logC) {
    dat[dat$TRTP == "A", "AVAL0"] <- lXA
    dat[dat$TRTP == "P", "AVAL0"] <- lXP
  } else {
    dat[dat$TRTP == "A", "AVAL0"] <- XA
    dat[dat$TRTP == "P", "AVAL0"] <- XP
  }
  ## Needed only for all_data = TRUE
  BDS <- dat[, c("ID", "TRTP", "AVAL0")]
  names(BDS)[names(BDS) == "AVAL0"] <- "AVAL"
  ###############
  dat[dat$GROUP != "C", "AVAL0"] <- dat[dat$GROUP != "C", "AVALT"]
  mcont <- min(dat[dat$GROUP == "C", "AVAL0"])
  dat$AVAL <- ifelse(dat$GROUP != "C", dat$AVAL0 + dat$GROUPN, dat$AVAL0 - mcont + 1 + dat$GROUPN)
  if(!all_data){
    dat <- dat[, c("ID", "TRTP", "GROUP", "GROUPN", "AVALT", "AVAL0", "AVAL", "seed")]
    dat$PADY <- fixedf
    dat <- as_hce(dat)
    return(dat)  
  } else{
    TRANSPOSE <- function(x) {
      d <- as.data.frame(t(x))
      d$ID <- as.numeric(names(d))
      names(d)[1] <- "AVAL"
      d$EVENT <- row.names(d)
      row.names(d) <- NULL
      d  
    }
    l <- lapply(split(dat[, base::startsWith(names(dat), "TTE"), drop = FALSE], dat$ID), TRANSPOSE)
    res <- do.call(rbind, l)
    row.names(res) <- NULL
    res <- res[res$AVAL <= fixedf, c("ID", "AVAL", "EVENT")]
    res$PADY <- fixedf
    dat <- dat[, c("ID", "TRTP", "GROUP", "GROUPN", "AVALT", "AVAL0", "AVAL", "seed")]
    dat$PADY <- fixedf
    dat <- as_hce(dat)
    l <- list(ADET = res, BDS = BDS, HCE = dat)
    return(l)
  }
}

phi <- function(t, theta = 1) {
  exp(- t^(1/theta))
}

#' Simulate from a Gumbel copula (version 1)
#'
#' @param n number of observations. 
#' @param dim dimension of the copula.
#' @param theta coefficient of dependence. 
#'
#' @return a random vector.
#' @noRd
rgumbel <- function(n, dim = 2, theta = 1){
  stopifnot("`theta` must be greater than or equal to 1." = theta >= 1)
  if(length(n) > 1)
    n <- length(n)
  if(length(dim) > 1)
    dim <- length(dim)
  exprand <- matrix(stats::rexp(dim * n), c(n, dim))
  unifpirand <- stats::runif(n, 0, pi)
  exprand2 <- stats::rexp(n)
  beta <- 1/theta
  stablerand <- sin((1 - beta) * unifpirand)^((1 - beta)/beta) * 
    (sin(beta * unifpirand))/(sin(unifpirand))^(1/beta)
  stablerand <- stablerand/(exprand2^(theta - 1))
  unifrand <- phi(exprand/stablerand, theta)
  return(unifrand)  
}

#' Simulate from a Gumbel copula (version 2)
#'
#' @param n number of observations. 
#' @param dim dimension of the copula.
#' @param theta coefficient of dependence. 
#'
#' @return a random vector.
#' @noRd
rGumbel <- function(n, dim = 2, theta = 1) {
  stopifnot("`theta` must be greater than or equal to 1." = theta >= 1)
  if(length(dim) > 1)
    dim <- length(dim)
  if(length(n) > 1)
    n <- length(n)
  rSTBL <- function(n, theta){
    Theta <- stats::runif(n, -pi/2, pi/2)
    W <- stats::rexp(n)
    beta <- 1
    alpha <- 1/theta
    gamma <- (cos(pi/(2*theta)))^theta
    delta <- ifelse(theta == 1, 1, 0)
    btan <- beta*tan(alpha*pi/2)
    theta0 <- atan(btan)/alpha
    Ctan <- (1 + btan^2)^(1/(2*alpha))
    alpha0 <- alpha*(theta0 + Theta)
    if(alpha != 1){
      Z <- sin(alpha0)*Ctan/cos(Theta)^(1/alpha)*(cos(alpha0 - Theta)/W)^((1 - alpha)/alpha)
    } else {
      pb <- pi/2 + beta*Theta
      Z <- 2/pi*(pb*tan(Theta) - beta*log(pi/2*W*cos(Theta)/pb))
    }
    Z * gamma + delta
  }
  exprand <- matrix(stats::rexp(dim * n), c(n, dim))
  stablerand <- rSTBL(n, theta = theta)
  unifrand <- phi(exprand/stablerand, theta)
  return(unifrand)
}

#' Calculates patient-level individual win proportions
#'
#' @param data a data frame containing subject-level data.
#' @param AVAL variable in the data with ordinal analysis values.
#' @param TRTP the treatment variable in the data.
#' @param ref the reference treatment group.
#' @return the input data frame with a new column of individual win proportions named using the input `AVAL` value with `_`.   
#' @export
#' @md
#' @seealso [hce::calcWO()], [hce::calcWO.hce()], [hce::calcWO.formula()].
#' @references Gasparyan SB et al. "Adjusted win ratio with stratification: calculation methods and interpretation." Statistical Methods in Medical Research 30.2 (2021): 580-611. <doi:10.1177/0962280220942558>.
#' @examples
#' KHCE1 <- IWP(data = KHCE, AVAL = "EGFRBL", TRTP = "TRTPN", ref = 2)
#' WP <- tapply(KHCE1$EGFRBL_, KHCE1$TRTPN, mean)
#' VAR <- tapply(KHCE1$EGFRBL_, KHCE1$TRTPN, function(x) (length(x)-1)*var(x)/length(x))
#' N <- tapply(KHCE1$EGFRBL_, KHCE1$TRTPN, length)
#' SE <- sqrt(sum(VAR/N))
#' c(WP = WP[[1]], SE = SE)
#' calcWO(EGFRBL ~ TRTP, data = KHCE)[c("WP", "SE_WP")]
IWP <- function(data, AVAL, TRTP, ref){
  data <- as.data.frame(data)
  AVAL <- AVAL[1]
  ref <- ref[1]
  TRTP <- TRTP[1]
  
  if (!AVAL %in% base::names(data)) 
    stop(paste0("The variable ", AVAL, " is not in the dataset."))
  if (!TRTP %in% base::names(data)) 
    stop(paste0("The variable ", TRTP, " is not in the dataset."))
  data$AVAL <- data[, AVAL]
  data$TRTP <- data[, TRTP]
  if (length(unique(data$TRTP)) != 2) 
    stop("The dataset should contain two treatment groups.")
  if (!ref %in% unique(data$TRTP)) 
    stop("Choose the reference from the values in TRTP.")
  data$TRTP <- base::ifelse(data$TRTP == ref, "P", "A")
  A <- base::rank(c(data$AVAL[data$TRTP == "A"], data$AVAL[data$TRTP == 
                                                             "P"]), ties.method = "average")
  B <- base::tapply(data$AVAL, data$TRTP, base::rank, ties.method = "average")
  n <- base::tapply(data$AVAL, data$TRTP, base::length)
  n1 <- n[["A"]]
  n0 <- n[["P"]]
  d <- base::data.frame(R1 = A, R2 = base::c(B$A, B$P), TRTP = base::c(base::rep("A", 
                                                                                 n1), base::rep("P", n0)))
  d$R <- d$R1 - d$R2
  d$R0 <- base::ifelse(d$TRTP == "A", d$R/n0, d$R/n1)
  data <- rbind(data[data$TRTP == "A", ], data[data$TRTP == "P", ])
  data[ , paste0(AVAL, "_")] <- d$R0
  data[order(as.numeric(row.names(data))),]
}

#' A generic function for calculating win statistics
#'
#' @param x an object used to select a method.
#' @param ... further arguments passed to or from other methods.
#'
#' @return a data frame containing calculated values.
#' @export
#' @md
#' @seealso [hce::calcWINS.hce()], [hce::calcWINS.formula()], [hce::calcWINS.data.frame()]  methods.
calcWINS <- function(x, ...) {
  UseMethod("calcWINS")
}

#' A generic function for calculating win odds
#'
#' @param x an object used to select a method.
#' @param ... further arguments passed to or from other methods.
#' @return a data frame containing calculated values.
#' @export
#' @md
#' @seealso [hce::calcWO.hce()], [hce::calcWO.formula()], [hce::calcWO.data.frame()]  methods.
#'
calcWO <- function(x, ...) {
  UseMethod("calcWO")
}

#' Sample size calculation for the win odds test (no ties)
#'
#' @param WO a numeric vector of win odds values.
#' @param power the given power. A numeric vector of length 1.
#' @param SD assumed standard deviation of the win proportion. By default uses the conservative SD. A numeric vector of length 1.
#' @param k proportion of active group in the overall sample size. Default is 0.5 (balanced randomization). A numeric vector of length 1.
#' @param alpha the significance level for the 2-sided test. Default is 0.05. A numeric vector of length 1.
#' @param WOnull the win odds value of the null hypothesis (default is 1). A numeric vector of length 1.
#' @param alternative a character string specifying the class of the alternative hypothesis, must be one of `"shift"` (default), `"max"` or `"ordered"`. You can specify just the initial letter.
#'
#' @details
#' `alternative = "max"` refers to the maximum variance of the win proportion across all possible
#' alternatives. The maximum variance equals `WP*(1 - WP)/k` where the win probability is calculated as `WP = WO/(WO + 1).`
#' `alternative = "shift"` specifies the variance across alternatives from a shifted family of distributions (Wilcoxon test). The variance formula, as suggested by Noether, is calculated based on the null hypothesis as follows `1/(12*k*(1 - k)).` 
#' `alternative = "ordered"` specifies the variance across alternatives from stochastically ordered distributions which include shifted distributions. 
#' 
#' @return a data frame containing the sample size with input values.
#' @export
#' @md
#' @seealso [hce::powerWO()], [hce::minWO()] for WO power or minimum detectable WO calculation.
#' @references 
#' * All formulas were presented in 
#' \cr \cr Bamber D (1975) "The area above the ordinal dominance graph and the area below the receiver operating characteristic graph." Journal of Mathematical Psychology 12.4: 387-415. <doi:10.1016/0022-2496(75)90001-2>.
#' * Noether's formula for shifted alternatives 
#' \cr \cr Noether GE (1987) "Sample size determination for some common nonparametric tests." Journal of the American Statistical Association 82.398: 645-7. <doi:10.1080/01621459.1987.10478478>.
#' * For shift alternatives see also 
#' \cr \cr Gasparyan SB et al. (2021) "Power and sample size calculation for the win odds test: application to an ordinal endpoint in COVID-19 trials." Journal of Biopharmaceutical Statistics 31.6: 765-787. <doi:10.1080/10543406.2021.1968893>.
#' @examples
#' sizeWO(WO = 1.25, power = 0.9)
#' sizeWO(WO = 1.25, power = 0.9, k = 0.75)
#' sizeWO(WO = seq(1.05, 1.5, 0.05), power = 0.9)
#' # Comparison of different alternatives
#' x <- seq(1.05, 5, 0.05)
#' N1 <- sizeWO(WO = x, power = 0.9, alternative = "m")$SampleSize
#' N2 <- sizeWO(WO = x, power = 0.9, alternative = "o")$SampleSize
#' N3 <- sizeWO(WO = x, power = 0.9, alternative = "s")$SampleSize
#' d <- data.frame(WO = x, N_m = N1, N_o = N2, N_s = N3)
#' ## Check the power for the ordered alternative
#' check <- c()
#' for(i in seq_along(x)){
#' check[i] <- powerWO(N = d[i, "N_o"], WO = d[i, "WO"], alternative = "o")$power
#' }
#' d$power_check_o <- check
#' print(d)

sizeWO <- function(WO, power, SD = NULL, k = 0.5, alpha = 0.05, WOnull = 1, alternative = c("shift", "max", "ordered")){
  power <- power[1]
  SD <- SD[1]
  alpha <- alpha[1]
  WOnull <- WOnull[1]
  WP <- WO/(WO + 1)
  k <- k[1]
  WPnull <- WOnull/(WOnull + 1)
  if(k < 0 | k > 1) stop("k should be between 0 and 1.")
  if (any(WO <= 1)) 
    stop("WO should be > 1 for superiority.")
  
  C <- stats::qnorm(1 - alpha/2)
  k <- min(k, 1- k)
  alternative <- match.arg(alternative)
  VAR <- switch(alternative,
                shift = 1/(12 * k * (1 - k)),
                max = WP*(1 - WP)/k ,
                ordered = 1/(k*(1 - k))*(
                  2*k*WP*(1 - WP) - 
                    (1 - 2*k)*(1 - WP)^2 + 
                    1/3*(1 - 3*k)*( 1 - (2*WP - 1)^1.5)
                ))
  if (is.null(SD)){
    SD <- sqrt(VAR)
  }
  SS <- SD^2/(WP - WPnull)^2*(C - stats::qnorm(1 - power))^2
  d <- base::data.frame(WO = WO, WP = WP, power = power, SampleSize = base::ceiling(SS), SD = SD, alpha = alpha, WPnull = WPnull, WOnull = WOnull, k = k)
  l <- base::list(result = d, value = base::ceiling(SS), input = WO, input_name = "Win Odds", value_name = "Sample Size")
  attr(d, "res") <- l
  base::class(d) <- c("hce_results")
  d
}

test_that("size is correct", {
  expect_equal(sizeWO(WO = 1.23, power = 0.9)$SampleSize, 1318)
})
test_that("size is correct, k = 1/3", {
  expect_equal(sizeWO(WO = 1.5, power = 0.9, k = 1/3)$SampleSize, 395)
})

#' A print method for `hce_results ` objects
#'
#' @param x an object of class `hce_results`.
#' @param ... additional arguments to be passed to [base::plot()] function.
#' @return no return value, called for plotting.
#' @export
#' @md
#' @examples
#' WO <- minWO(N = 100:1000)
#' plot(WO)
#' POW <- powerWO(N = 100:1000, WO = 1.2)
#' plot(POW, ylim = c(0, 1))
plot.hce_results <- function(x, ...){
  x <- attr(x, "res")
  Args <- base::list(...)
  Args[["x"]] <- x$input
  Args[["y"]] <- x$value

  if ( is.null(Args[["xlab"]]) & is.null(Args[["ylab"]])) {
    Args[["xlab"]] <- x$input_name
    Args[["ylab"]] <- x$value_name
  } else if (is.null(Args[["xlab"]])) {
    Args[["xlab"]] <- x$input_name
  } else if (is.null(Args[["ylab"]])) {
    Args[["ylab"]] <- x$value_name
  }


 if (is.null(Args[["type"]])) Args[["type"]] <- "l"

 base::do.call(base::plot, Args)

}

#' Power calculation for the win odds test (no ties)
#'
#' @param N a numeric vector of sample size values.
#' @param WO the given win odds for the alternative hypothesis. A numeric vector of length 1.
#' @param SD assumed standard deviation of the win proportion. By default uses the conservative SD. A numeric vector of length 1.
#' @param k proportion of active group in the overall sample size. Default is 0.5 (balanced randomization). A numeric vector of length 1.
#' @param alpha the significance level for the 2-sided test. Default is 0.05. A numeric vector of length 1.
#' @param WOnull the win odds value of the null hypothesis (default is 1). A numeric vector of length 1.
#' @param alternative a character string specifying the class of the alternative hypothesis, must be one of `"shift"` (default), `"max"` or `"ordered"`. You can specify just the initial letter.
#'
#' @details
#' `alternative = "max"` refers to the maximum variance of the win proportion across all possible
#' alternatives. The maximum variance equals `WP*(1 - WP)/k` where the win probability is calculated as `WP = WO/(WO + 1).`
#' `alternative = "shift"` specifies the variance across alternatives from a shifted family of distributions (Wilcoxon test). The variance formula, as suggested by Noether, is calculated based on the null hypothesis as follows `1/(12*k*(1 - k)).` 
#' `alternative = "ordered"` specifies the variance across alternatives from stochastically ordered distributions which include shifted distributions. 
#' 
#' @return a data frame containing the calculated power with input values.
#' @export
#' @md
#' @seealso [hce::sizeWO()], [hce::minWO()] for WO sample size or minimum detectable WO calculation.
#' @references 
#' * All formulas were presented in 
#' \cr \cr Bamber D (1975) "The area above the ordinal dominance graph and the area below the receiver operating characteristic graph." Journal of Mathematical Psychology 12.4: 387-415. <doi:10.1016/0022-2496(75)90001-2>.
#' * Noether's formula for shifted alternatives 
#' \cr \cr Noether GE (1987) "Sample size determination for some common nonparametric tests." Journal of the American Statistical Association 82.398: 645-7. <doi:10.1080/01621459.1987.10478478>.
#' * For shift alternatives see also 
#' \cr \cr Gasparyan SB et al. (2021) "Power and sample size calculation for the win odds test: application to an ordinal endpoint in COVID-19 trials." Journal of Biopharmaceutical Statistics 31.6: 765-787. <doi:10.1080/10543406.2021.1968893>.
#' @examples
#' # Example 1- Use the default standard deviation
#' powerWO(N = 1000, WO = 1.2)
#' powerWO(N = seq(500, 1500, 100), WO = 1.2)
#' # Example 2 - Use data-driven win odds and standard deviation from the COVID19 dataset
#' res <- calcWO(x = COVID19, AVAL = "GROUP", TRTP = "TRTP", ref = "Placebo")
#' print(res)
#' powerWO(N = 500, WO = res$WO, SD = res$SD_WP)
#' powerWO(N = 500, WO = res$WO) # power with the default standard deviation for the win proportion.
#' # Example 3 - Non-balanced 3:1 randomization
#' powerWO(N = 1000, WO = 1.2, k = 0.75)
#' # Example 4 - Comparison of different alternatives
#' powerWO(N = 1000, WO = 1.2, alternative = "m")
#' powerWO(N = 1000, WO = 1.2, alternative = "s")
#' powerWO(N = 1000, WO = 1.2, alternative = "o")
powerWO <- function(N, WO, SD = NULL, k = 0.5, alpha = 0.05, WOnull = 1, alternative = c("shift", "max", "ordered")){
  WO <- WO[1]
  SD <- SD[1]
  alpha <- alpha[1]
  WOnull <- WOnull[1]
  k <- k[1]
  if (k < 0 | k > 1) 
    stop("k should be between 0 and 1.")
  if (WO <= 1) 
    stop("WO should be > 1 for superiority.")
  k <- min(k, 1- k)
  alternative <- match.arg(alternative)
  WP <- WO/(WO + 1)
  WPnull <- WOnull/(WOnull + 1)
  SD_method <- "Input"
  VAR <- switch(alternative,
                shift = 1/(12 * k * (1 - k)),
                max = WP*(1 - WP)/k ,
                ordered = 1/(k*(1 - k)*N)*(
                  (2*k*N - 1)*WP*(1 - WP) - 
                    (1 - 2*k)*N*(1 - WP)^2 + 
                    1/3*((1 - 3*k)*N + 1)*( 1 - (2*WP - 1)^1.5)
                ))
      if (is.null(SD)){
      SD <- sqrt(VAR)
      SD_method <- alternative
  }
  SE <- SD/base::sqrt(N)
  C <- stats::qnorm(1 - alpha/2)
  power <- stats::pnorm(-C + (WP - WPnull)/SE) + stats::pnorm(-C - 
                                                                (WP - WPnull)/SE)
  d <- base::data.frame(power = power, SampleSize = N, WO = WO, 
                        WP = WP, WPnull = WPnull, WOnull = WOnull, alpha = alpha, 
                        SD = SD, SD_method = SD_method, k = k)
  l <- base::list(result = d, value = power, input = N, input_name = "Sample Size", 
                  value_name = "Power")
  attr(d, "res") <- l
  base::class(d) <- c("hce_results")
  d
}

#' Win odds summary using formula syntax
#'
#' @param x an object of class formula.
#' @param data a data frame.
#' @param ... additional parameters.
#' @returns a list containing the summary of wins, losses, and ties. It contains the following named objects:
#' * summary a data frame containing number of wins, losses, and ties by treatment group and the overall number of comparisons.
#' * WO calculated WO (win odds) and WP (win probability) and their standard errors.
#' * formula returning the specified formula in the `x` argument.
#' * ref showing how the reference group was selected. Can be modifying by specifying the `ref` argument.
#' @export
#' @md
#' @seealso [hce::calcWO()], [hce::summaryWO()], [hce::summaryWO.data.frame()], [hce::summaryWO.hce()]  methods.
#' @examples
#' # Example 1
#' summaryWO(data = COVID19, GROUP ~ TRTP)
#' summaryWO(data = COVID19, GROUP ~ TRTP, GROUP = "GROUP", ref = "Placebo")
#' # Example 2 - Individual wins, losses, and ties
#' dat <- COVID19
#' dat$ID <- 1:nrow(dat)
#' summaryWO(data = dat, GROUP ~ TRTP, GROUP = "ID", ref = "Placebo")

summaryWO.formula <- function(x, data, ...){
  Args <- base::list(...)
  formulavars <- base::all.vars(x)
  formula0 <- stats::reformulate(formulavars[2], response = formulavars[1])
  mf <- stats::model.frame(formula = formula0, data = data)
  Level <- base::unique(mf[[formulavars[2]]])
  Levellength <- base::length(Level)
  
  if(Levellength != 2){
    message1 <- base::paste("The variable", formulavars[2], "should have exactly 2 unique values, but has", Levellength)
    stop(message1)
  }
  if(!base::is.null(Args[["ref"]])) ref <- Args[["ref"]]
  else if (base::all(Level == c("A", "P"))) ref <- "P"
  else ref <- Level[1]
  if(! ref %in% Level) stop(base::paste("Choose the reference from the values",
                                        base::paste(Level, collapse = ", ")))
  if(!base::is.null(Args[["GROUP"]])) {
    GROUP <- Args[["GROUP"]]
    mf[, GROUP] <- data[, GROUP, drop = T]
  }
  else GROUP <- NULL
  res <- summaryWO.data.frame(x = mf, AVAL = formulavars[1], TRTP = formulavars[2], ref = ref, GROUP = GROUP)
  res$formula <- base::deparse(formula0)
  res$ref <- base::paste(Level[Level != ref],"vs", ref)
  res
}

#' A generic function for coercing data structures to `hce` objects
#'
#' @param x an object used to select a method.
#' @param ... additional parameters.
#'
#' @return an `hce` object.
#' @export
#' @md
#' @seealso [hce::as_hce.data.frame()], [hce::as_hce.default()].
#' @examples
#' ### data frames 
#' data(HCE1)
#' HCE <- as_hce(HCE1)
#' calcWINS(HCE)
as_hce <- function(x, ...) {
  UseMethod("as_hce")
}

#' A print method for `hce_results` objects
#'
#' @param x an object of class `hce_results`.
#' @param ... additional arguments to be passed to [base::print()] function.
#' @return no return value, called for printing.
#' @export
#'
#' @examples
#' print(powerWO(N = 1000, WO = 1.2))
print.hce_results <- function(x, ...){
  x <- attr(x, "res")
  base::print(x$result, ...)
  
}

#' Minimum detectable or WO for alternative hypothesis for given power (no ties)
#'
#' @param N a numeric vector of sample size values (two arms combined).
#' @param power the given power. The default is 0.5 corresponding to the minimum detectable win odds. A numeric vector of length 1.
#' @param SD assumed standard deviation of the win proportion. By default uses the conservative SD. A numeric vector of length 1.
#' @param k proportion of active group in the overall sample size. Default is 0.5 (balanced randomization). A numeric vector of length 1.
#' @param alpha the significance level for the 2-sided test. Default is 0.05. A numeric vector of length 1.
#' @param WOnull the win odds value of the null hypothesis (default is 1). A numeric vector of length 1.
#' @param digits precision to use for reporting calculated win odds.
#'
#' @return a data frame containing the calculated WO with input values.
#' @export
#' @md
#' @seealso [hce::powerWO()], [hce::sizeWO()]  for WO power and sample size calculation.
#' @references Gasparyan SB et al. (2021) "Power and sample size calculation for the win odds test: application to an ordinal endpoint in COVID-19 trials." Journal of Biopharmaceutical Statistics 31.6: 765-787. <doi:10.1080/10543406.2021.1968893>.
#' @examples
#' minWO(N = 100, digits = 5)
#' minWO(N = 1200, power = 0.9, k = 0.75)
#' # Compare the minimum detectable win odds from shifted alternatives to max and ordered alternatives
#' WO <- minWO(N = 1200, k = 0.5, power = 0.67, digits = 7)$WO
#' powerWO(N = 1200, WO = WO, k = 0.5, alternative = "shift")
#' powerWO(N = 1200, WO = WO, k = 0.5, alternative = "ordered")
#' powerWO(N = 1200, WO = WO, k = 0.5, alternative = "max")
minWO <- function(N, power = 0.5, SD = NULL, k = 0.5, alpha = 0.05, WOnull = 1, digits = 2){
  digits <- digits[1]
  power <- power[1]
  SD <- SD[1]
  alpha <- alpha[1]
  WOnull <- WOnull[1]
  WPnull <- WOnull/(WOnull + 1)
  k <- k[1]
  if(k < 0 | k > 1) stop("k should be between 0 and 1.")
  SD_method <- "Input"
  VAR <- 1/(12*k*(1 - k))
  if(is.null(SD)) {
    SD <- sqrt(VAR)
    SD_method <- "shift"
  }
  SE <- SD/base::sqrt(N)
  C <- stats::qnorm(1 - alpha/2)

  WP <- WPnull + SE*(C - stats::qnorm(1 - power))
  WO <- WP/(1 - WP)
  d <- base::data.frame(WO = base::round(WO, digits), WP = WP, SampleSize = N, power = power, WPnull = WPnull, WOnull = WOnull, alpha = alpha, SD = SD, SD_method = SD_method, k = k)
  l <- base::list(result = d, value = WO, input = N, input_name = "Sample Size", value_name = "Win Odds")
  attr(d, "res") <- l
  base::class(d) <- c("hce_results")
  d
}

#' `HCE1`, `HCE2`, `HCE3`, `HCE4` datasets for 1000 patients with different treatment effects.
#'
#' A simulated dataset containing the ordinal values and other attributes for 1000 patients.
#' HCE1
#'
#' @format a data frame with 1000 rows and 6 variables:
#' \describe{
#'   \item{ID}{subject ID, numbers from 1 to 1000}
#'   \item{TRTP}{treatment values, A Active or P Placebo, character}
#'   \item{GROUP}{type of the event, either Time-To-Event (TTE) or Continuous (C), character}
#'   \item{GROUPN}{type of the event, for the ordering of outcomes in the `GROUP` variable, numeric}
#'   \item{AVALT}{the timing of the time-to-event outcomes, numeric}
#'   \item{AVAL0}{original values for each type of the event, time for TTE outcomes, numeric values for Continuous outcomes, numeric}
#'   \item{AVAL}{`AVAL = AVAL0 + GROUPN`, ordinal analysis values for the HCE analysis. For the continuous outcome the values of `AVAL0` are shifted to start always from 0. Numeric, but caution NOT to apply numeric operations; will give meaningless results}
#'   \item{PADY}{ primary analysis day, the length of fixed follow-up in days, numeric}
#' }
"HCE1"

#' `HCE1`, `HCE2`, `HCE3`, `HCE4` datasets for 1000 patients with different treatment effects.
#'
#' A simulated dataset containing the ordinal values and other attributes for 1000 patients.
#' HCE2
#'
#' @format a data frame with 1000 rows and 6 variables:
#' \describe{
#'   \item{ID}{subject ID, numbers from 1 to 1000}
#'   \item{TRTP}{treatment values, A Active or P Placebo, character}
#'   \item{GROUP}{type of the event, either Time-To-Event (TTE) or Continuous (C), character}
#'   \item{GROUPN}{type of the event, for the ordering of outcomes in the `GROUP` variable, numeric}
#'   \item{AVALT}{the timing of the time-to-event outcomes, numeric}
#'   \item{AVAL0}{original values for each type of the event, time for TTE outcomes, numeric values for Continuous outcomes, numeric}
#'   \item{AVAL}{`AVAL = AVAL0 + GROUPN`, ordinal analysis values for the HCE analysis. For the continuous outcome the values of `AVAL0` are shifted to start always from 0. Numeric, but caution NOT to apply numeric operations; will give meaningless results}
#'   \item{PADY}{ primary analysis day, the length of fixed follow-up in days, numeric}
#' }
"HCE2"

#' `HCE1`, `HCE2`, `HCE3`, `HCE4` datasets for 1000 patients with different treatment effects.
#'
#' A simulated dataset containing the ordinal values and other attributes for 1000 patients.
#' HCE3
#'
#' @format a data frame with 1000 rows and 6 variables:
#' \describe{
#'   \item{ID}{subject ID, numbers from 1 to 1000}
#'   \item{TRTP}{treatment values, A Active or P Placebo, character}
#'   \item{GROUP}{type of the event, either Time-To-Event (TTE) or Continuous (C), character}
#'   \item{GROUPN}{type of the event, for the ordering of outcomes in the `GROUP` variable, numeric}
#'   \item{AVALT}{the timing of the time-to-event outcomes, numeric}
#'   \item{AVAL0}{original values for each type of the event, time for TTE outcomes, numeric values for Continuous outcomes, numeric}
#'   \item{AVAL}{`AVAL = AVAL0 + GROUPN`, ordinal analysis values for the HCE analysis. For the continuous outcome the values of `AVAL0` are shifted to start always from 0. Numeric, but caution NOT to apply numeric operations; will give meaningless results}
#'   \item{PADY}{ primary analysis day, the length of fixed follow-up in days, numeric}
#' }
"HCE3"


#' `HCE1`, `HCE2`, `HCE3`, `HCE4` datasets for 1000 patients with different treatment effects.
#'
#' A simulated dataset containing the ordinal values and other attributes for 1000 patients.
#' HCE4
#'
#' @format a data frame with 1000 rows and 6 variables:
#' \describe{
#'   \item{ID}{subject ID, numbers from 1 to 1000}
#'   \item{TRTP}{treatment values, A Active or P Placebo, character}
#'   \item{GROUP}{type of the event, either Time-To-Event (TTE) or Continuous (C), character}
#'   \item{GROUPN}{type of the event, for the ordering of outcomes in the `GROUP` variable, numeric}
#'   \item{AVALT}{the timing of the time-to-event outcomes, numeric}
#'   \item{AVAL0}{original values for each type of the event, time for TTE outcomes, numeric values for Continuous outcomes, numeric}
#'   \item{AVAL}{`AVAL = AVAL0 + GROUPN`, ordinal analysis values for the HCE analysis. For the continuous outcome the values of `AVAL0` are shifted to start always from 0. Numeric, but caution NOT to apply numeric operations; will give meaningless results}
#'   \item{PADY}{ primary analysis day, the length of fixed follow-up in days, numeric}
#' }
"HCE4"


#' COVID-19 ordinal scale dataset (full report).
#'
#' A dataset with COVID-19 ordinal scale outcomes for 1062 patients.
#'
#' @format a data frame with 1062 rows and 2 variables:
#' \describe{
#'   \item{GROUP}{type of the event, ordinal outcomes 1-8, where a higher value means a better outcome}
#'   \item{TRTP}{treatment values, A Active or P Placebo, character}
#' }
#' @source Beigel JH et al. "Remdesivir for the treatment of Covid-19-final report." New England Journal of Medicine 383.19 (2020): 1813-1836. <doi:10.1056/NEJMoa2007764>.
#' @examples
#' #Frequencies
#' table(COVID19)
#' mosaicplot(table(COVID19), col = c(1, 8, 6, 2, 4, 5, 3, 7), 
#' xlab = "Treatment", ylab = "Ordinal Scale", main = "COVID-19 ordinal scale")
#' # Convert to an hce object
#' COVID19HCE <- hce(GROUP = COVID19$GROUP, TRTP = COVID19$TRTP)
#' # Summary wins, losses, and ties with win odds
#' summaryWO(COVID19HCE, ref = "Placebo")
"COVID19"


#' COVID-19 ordinal scale dataset (preliminary report).
#'
#' A dataset with COVID-19 ordinal scale outcomes for 844 patients.
#'
#' @format a data frame with 844 rows and 2 variables:
#' \describe{
#'   \item{GROUP}{type of the event, ordinal outcomes 1-8, where a higher value means a better outcome}
#'   \item{TRTP}{treatment values, Active or Placebo, character}
#' }
#' @source Beigel JH et al. "Remdesivir for the treatment of Covid-19-final report." New England Journal of Medicine 383.19 (2020): 1813-1836. <doi:10.1056/NEJMoa2007764>.
#' @examples
#' #Frequencies
#' table(COVID19b)
#' mosaicplot(table(COVID19b), col = c(1, 8, 6, 2, 4, 5, 3, 7), 
#' xlab = "Treatment", ylab = "Ordinal Scale", main = "COVID-19 ordinal scale")
#' # Calculate win statistics
#' calcWINS(x = COVID19b, AVAL = "GROUP", TRTP = "TRTP", ref = "Placebo")
"COVID19b"


#' COVID-19 ordinal scale dataset for a combination therapy.
#'
#' A dataset with COVID-19 ordinal scale outcomes for 1033 patients.
#'
#' @format a data frame with 1033 rows and 4 variables:
#' \describe{
#'   \item{ID}{patient identifiers, numeric}
#'   \item{TRTP}{treatment values, A Active or P Placebo, character}
#'   \item{GROUP}{type of the event, ordinal outcomes 1-8, where a higher value means a better outcome}
#'   \item{BASE}{baseline ordinal values}
#' }
#' @source Kalil AC et al. "Baricitinib plus Remdesivir for Hospitalized Adults with Covid-19." New England Journal of Medicine 384.9 (2021): 795-807. <doi:10.1056/NEJMoa2031994>.
#' @examples
#' COVID19HCE <- hce(GROUP = COVID19plus$GROUP, TRTP = COVID19plus$TRTP)
#' # Summary wins, losses, and ties with win odds
#' summaryWO(COVID19HCE, ref = "P")
"COVID19plus"


#' Kidney Hierarchical Composite Endpoint dataset.
#'
#' A dataset with kidney ordinal scale outcomes of 1500 patients in the `ADSL` dataset.
#'
#' @format a data frame with 1500 rows and 11 variables:
#' \describe{
#'   \item{ID}{patient identifiers, numeric}
#'   \item{TRTPN}{treatment values, 1 Active or 2 Placebo, numeric}
#'   \item{AVAL0}{original values for each type of the event, time for TTE outcomes 1-6, numeric values for Continuous outcome 7, numeric}
#'   \item{AVAL}{`AVAL = AVAL0 + GROUPN`, ordinal analysis values for the HCE analysis, numeric, but caution NOT to apply numeric operations; will give meaningless results}
#'   \item{GROUP}{name of the event, character}
#'   \item{GROUPN}{ordinal outcomes corresponding to `PARAMN` values, numeric}
#'   \item{PARAMCD}{coded name of the event, character}
#'   \item{PARAMN}{severity of the event, outcomes 1-7, where a higher value means a better outcome, character}
#'   \item{STRATAN}{strata 1-4, higher value means more severe kidney disease, numeric}
#'   \item{EGFRBL}{Baseline GFR values of patients, numeric}
#'   \item{TRTP}{treatment values, A Active or P Placebo, character}
#'   \item{PADY}{primary analysis day (in years), length of the fixed follow-up, numeric}
#' }
#' @source 
#' Heerspink HL et al "Development and validation of a new hierarchical composite endpoint for clinical trials of kidney disease progression." Journal of the American Society of Nephrology (2023): <doi:10.1681/ASN.0000000000000243>.
#' @examples
#' # Adjusted win odds
#' res <- regWO(x = KHCE, AVAL = "AVAL", TRTP = "TRTP", COVAR = "STRATAN", ref = "P")
#' res
#' # Convert the dataset to an adhce object.
#' ## First check that `GROUP` is a factor with the correct ordering of outcomes.
#' class(KHCE$GROUP) # "factor"
#' levels(KHCE$GROUP)
#' dat1 <- as_hce(KHCE)
#' class(dat1)
#' calcWO(dat1)
#' ## Re-derive individual patient eGFR slopes using a linear regression model, 
#' ## based on the eGFR measurements in the `ADLB` dataset
#' dat2 <- KHCE
#' l <- lapply(
#' split(ADLB, ADLB$ID), 
#' function(x) coef(lm(AVAL ~ ADAY, data = x))[2])
#' new_slopes <- do.call(rbind, l)
#' new_slopes <- as.data.frame(new_slopes)
#' names(new_slopes) <- "LINEAR"
#' new_slopes$ID <- as.numeric(row.names(new_slopes))
#' dat2 <- merge(KHCE, new_slopes, by = "ID", all.x = TRUE)
#' dat2$AVAL0[dat2$PARAMCD == "eGFR"] <- dat2$LINEAR[dat2$PARAMCD == "eGFR"]
#' dat2$AVAL0[is.na(dat2$AVAL0)] <- 0 
#' dat2 <- as_hce(dat2)
#' calcWO(dat2)
"KHCE"



#' Event-Time dataset for kidney outcomes.
#'
#' A dataset with multiple kidney outcomes over time scale outcomes of 1500 patients in the `ADSL` dataset.
#'
#' @format a data frame with 604 rows (events) and 6 variables:
#' \describe{
#'   \item{ID}{patient identifiers, numeric}
#'   \item{AVAL}{occurence time of the event, numeric}
#'   \item{PARAM}{name of the event, character}
#'   \item{PARAMCD}{coded name of the event, character}
#'   \item{PARAMN}{type of the event, outcomes 1-7, where a higher value means a better outcome, numeric}
#'   \item{TRTPN}{treatment values, 1 Active or 2 Placebo, numeric}
#' }
#' @source
#' Heerspink HL et al "Development and validation of a new hierarchical composite endpoint for clinical trials of kidney disease progression." Journal of the American Society of Nephrology (2023): <doi:10.1681/ASN.0000000000000243>. 
#' @examples
#' head(ADET)
#' # Number of unique patients
#' length(unique(ADET$ID)) 
#' # Number of events per event type
#' barplot(table(ADET$PARAM))
"ADET"



#' Laboratory dataset for Glomerular Filtration Rate (GFR) measurements.
#'
#' A dataset of laboratory measurements of kidney function over time for the 1500 patients in the `ADSL` dataset.
#'
#' @format a data frame with 13980 rows and 8 variables:
#' \describe{
#'   \item{ID}{patient identifiers, numeric}
#'   \item{TRTPN}{treatment values, 1 Active or 2 Placebo, numeric}
#'   \item{AVAL}{measurement value, numeric}
#'   \item{ADAY}{measurement day in the study, numeric}
#'   \item{AVISITN}{hospital visit number, numeric}
#'   \item{PARAM}{name of the event, GFR measurements, character}
#'   \item{PARAMCD}{coded name of the event, GFR, character}
#'   \item{PARAMN}{type of the event is set to 7 for all measurements, numeric}
#' }
#' @source 
#' Heerspink HL et al "Development and validation of a new hierarchical composite endpoint for clinical trials of kidney disease progression." Journal of the American Society of Nephrology (2023): <doi:10.1681/ASN.0000000000000243>.
#' @examples
#' head(ADLB)
"ADLB"



#' Baseline characteristics dataset of patients with kidney function assessments.
#'
#' A data frame with baseline characteristics for 1500 patients used to derive `KHCE` dataset.
#'
#' @format a data frame with 1500 rows and 4 variables:
#' \describe{
#'   \item{ID}{patient identifiers, numeric}
#'   \item{TRTPN}{treatment values, 1 Active or 2 Placebo, numeric}
#'   \item{EGFRBL}{Baseline GFR values of patients, numeric}
#'   \item{STRATAN}{strata 1-4, higher value means a higher risk for kidney disease progression, numeric}
#' }
#' @source 
#' Heerspink HL et al "Development and validation of a new hierarchical composite endpoint for clinical trials of kidney disease progression." Journal of the American Society of Nephrology (2023): <doi:10.1681/ASN.0000000000000243>.
#' @examples
#' head(ADSL)
"ADSL"

#' Win odds calculation for `hce` objects
#'
#' @param x an `hce` object.
#' @param ... additional parameters.
#' @returns a data frame containing the win odds and its confidence interval. It contains the following columns:
#' * WO calculated win odds.
#' * LCL lower confidence limit.
#' * UCL upper confidence limit.
#' * SE standard error of the win odds.
#' * WOnull win odds of the null hypothesis (specified in the `WOnull` argument).
#' * alpha two-sided significance level for calculating the confidence interval (specified in the `alpha` argument).
#' * Pvalue p-value associated with testing the null hypothesis.
#' * WP calculated win probability.
#' * LCL_WP lower confidence limit for `WP`.
#' * UCL_WP upper confidence limit for `WP`.
#' * SE_WP standard error of the win probability.
#' * SD_WP standard deviation of the win probability, calculated as `SE_WP` multiplied by `sqrt(N)`.
#' * N total number of patients in the analysis.
#' @export
#' @md
#' @seealso [hce::calcWO()], [hce::calcWO.formula()], [hce::calcWO.data.frame()].
#' @references Gasparyan SB et al. "Adjusted win ratio with stratification: calculation methods and interpretation." Statistical Methods in Medical Research 30.2 (2021): 580-611. <doi:10.1177/0962280220942558>.
#' @examples
#' Rates_A <- c(1, 1.5) 
#' Rates_P <- c(2, 2) 
#' dat <- simHCE(n = 500, TTE_A = Rates_A, TTE_P = Rates_P, CM_A = 1.25, CM_P = 1)
#' calcWO(dat)
#' calcWO(dat, ref = "A", WOnull = 1, alpha = 0.01)
calcWO.hce <- function(x, ...){
  Args <- base::list(...)
  x <- as_hce(x)
  x <- base::as.data.frame(x)
  if(!is.null(Args[["ref"]])) ref <- Args[["ref"]]
  else if ("P" %in% unique(x$TRTP)) ref <- "P"
  else ref <- unique(x$TRTP)[1]
  if(!base::is.null(Args[["alpha"]])) alpha <- Args[["alpha"]]
  else alpha <- 0.05
  if(!base::is.null(Args[["WOnull"]])) WOnull <- Args[["WOnull"]]
  else WOnull <- 1
  
  calcWO.data.frame(x = x, AVAL = "AVAL", TRTP = "TRTP", 
                    ref = ref, WOnull = WOnull, alpha = alpha)
}

#' A plot method for `hce` objects
#' @description
#' Ordinal dominance graph for `hce` objects
#' @param x an object of class `hce` created by `as_hce()`.
#' @param fill logical; if `TRUE` fill the area above the graph.
#' @param ... additional arguments to be passed to [base::plot()] function.
#'
#' @return no return value, called for plotting.
#' @export
#' @md
#' @references Bamber D. "The area above the ordinal dominance graph and the area below the receiver operating characteristic graph." Journal of Mathematical Psychology 12.4 (1975): 387-415. <doi:10.1016/0022-2496(75)90001-2>
#' @examples
#' d <- as_hce(KHCE)
#' d$TRTP <- factor(d$TRTP, levels = c("P", "A"))
#' res <- calcWO(AVAL ~ TRTP, data = d)
#' # Ordinal Dominance Graph
#' plot(d, col = 3, type = 'l')
#' grid()
#' # Area above the Ordinal Dominance Graph
#' plot(d, fill = TRUE, col = "#865A4F", type = 'l', 
#'      lwd = 2, xlab = "Control", ylab = "Active")
#' legend("bottomright", legend = paste0("WP = ", round(res$WP, 5)))
#' abline(a = 0, b = 1, lwd = 2, lty = 2, col = "#999999")
plot.hce <- function(x, fill = FALSE, ...){
  l <- list(...)
  if(is.null(l$type))
    l$type <- "l"
  x <- as_hce(x)
  xran <- seq(min(x$AVAL), max(x$AVAL), length.out = 10000)
  FUN <- tapply(x$AVAL, x$TRTP, stats::ecdf)
  NAM <- names(FUN)
  d <- data.frame(x = FUN[[NAM[1]]](xran), y = FUN[[NAM[2]]](xran))
  d0 <- data.frame(x = c(0), y = c(0))
  d <- rbind(d0, d)
  d <- unique(d)
  y_upper <- rep(max(d$y), length(d$x))
  x_poly <- c(d$x, rev(d$x))
  y_poly <- c(d$y, rev(y_upper))
  names(d) <- NAM
  plot(d, ...)
  if(fill){
    if(!is.null(l$col))
      color <- l$col
    else color <- "#999999"
    graphics::polygon(x_poly, y_poly, col = color)
  }
}

#' Simulate an `adhce` dataset with two correlated outcomes (illness - death model)
#' 
#' Simulate an `adhce` dataset with two correlated outcomes - death and hospitalization - from a heterogeneous population. The correlation between these outcomes arises from population heterogeneity. Models the risk of death following hospitalization as dependent on the timing 
#' of the hospitalization, reflecting strong dependence between the times to the first and second events (i.e., event clustering).
#'
#' @param n sample size in the active treatment group.
#' @param n0 sample size in the placebo treatment group.
#' @param TTE_A event rates in the active group for the time-to-event outcomes; a numeric vector of length two.
#' @param TTE_P event rates in the placebo group for the time-to-event outcomes; a numeric vector of length two.
#' @param shape shape parameter of the Weibull distribution for time-to-event outcomes in the active group. Default is 1 (exponential distribution).
#' @param shape0 shape parameter of the Weibull distribution for time-to-event outcomes in the placebo group. Default is 1 (exponential distribution).
#' @param fixedfy length of follow-up.
#' @param theta heterogeneity coefficient for the first event, modeled via a gamma distribution with mean 1; `theta` controls the variance. When `theta = 0`, there is no heterogeneity, which implies that death and hospitalization are independent.
#' @param alpha0 exponential heterogeneity coefficient for modeling the heterogeneity of risk of death as the first event.
#' @param alpha exponential heterogeneity coefficient for modeling the heterogeneity of risk of death after hospitalization; the heterogeneity of the second event is the inverse of the time of the first event.
#' @param rHR recurrence hazard ratio comparing the active group to the control group for the second event, based on gap time measured from the first event.
#' @param m categorization number used for to discretize time to death and time to hospitalization. Defaults to `Inf` (no discretization).
#' @param hce_type the type of the hierarchical composite endpoint: use `mi` for the most-important outcome or `md` for the move-down approach. This parameter only affects results when `m` is finite.
#' @details
#' The default setting assumes `TTE_A = TTE_P`. Both `TTE_A` and `TTE_P` must be numeric vectors of length two, corresponding to the event rates 
#' (Weibull distribution) for the first event of hospitalization and death. The parameters `shape` and `shape0` identify the shape parameters 
#' of Weibull distributions for the first event, simulated from a distribution with a cumulative hazard of 
#' \deqn{\gamma\cdot rate\cdot t^{shape}} 
#' for hospitalization and 
#' \deqn{\gamma^{\alpha_0}\cdot rate\cdot t^{shape}} 
#' for death, where \eqn{\gamma} (`gamma`) is a patient-specific frailty drawn from a gamma distribution with mean 1 and 
#' variance \eqn{\theta}, shared between death and hospitalization for a given patient. The parameter \eqn{\theta} (`theta`) represents population heterogeneity and also induces 
#' correlation between death and hospitalization as competing first events. The parameter \eqn{\alpha_0} (`alpha0`) controls the heterogeneity of time to death through its 
#' effect on heterogeneity. Death after hospitalization is simulated from an exponential distribution with a constant hazard that depends on the timing \eqn{t_1} 
#' of the first event (hospitalization) as 
#' \deqn{\frac{TTE_A[2] + TTE_P[2]}{2}\cdot \left(\frac{t_1}{fixedfy}\right)^{\alpha}\cdot \gamma^{\alpha_0}}
#' for the placebo arm and 
#' \deqn{\frac{TTE_A[2] + TTE_P[2]}{2}\cdot rHR \cdot \left(\frac{t_1}{fixedfy}\right)^{\alpha}\cdot \gamma^{\alpha_0}}
#' for the active arm where `rHR` is the recurrence hazard ratio. When \eqn{\alpha < 0}, earlier hospitalization (smaller \eqn{t_1}) 
#' increases the risk of death following hospitalization.
#' 
#' By default, events are simulated in continuous time. When `m` is specified as a positive numeric value, the event times are discretized into `m` intervals over the follow-up period.
#' @return an object of class `adhce`.
#' @export
#' @md
#' @seealso [hce::simHCE()] for a general `adhce` dataset simulation, and [hce::simKHCE()] for kidney disease-specific `adhce` simulation.
#' @examples
#' ## Example 1 - positive correlation
#' i <- 1764002323
#' set.seed(i)
#' PADY <- 2
#' D <- simTTE(n = 1000, TTE_A = c(0.1, 0.04), 
#' TTE_P = c(.15, 0.045), theta = 4, alpha0 = 2, alpha = -1, shape = 2, 
#' fixedfy = PADY, rHR = 1)
#' ####### Summary of first events by treatment group ########
#' table(D$EVENT1, D$TRTP)
#' ####### Summary of second events by treatment group ########
#' table(D$EVENT2, D$TRTP)
#' ######## Calculate win odds #########################
#' calcWO(D, ref = "P")
#' ## Plot the ordinal dominance graph ######
#' D$TRTP <- factor(D$TRTP, levels = c("P", "A"))
#' plot(D, type = "l", col = 2, fill = TRUE)
#' abline(a = 0, b = 1, lwd = 2, lty = 3, col = "darkgreen")
#' grid()
#' ################################################################
#' ## Example 2 - Move-down approach (discrete‑time case only)
#' PADY <- 2
#' # Continuous-time
#' set.seed(2)
#' D <- simTTE(n = 1000, TTE_A = c(0.1, 0.04), 
#' TTE_P = c(.15, 0.045), theta = 4, alpha0 = 2, alpha = -1, shape = 2, 
#' fixedfy = PADY, rHR = 1, m = Inf)
#' summaryWO(D, ref = "P")$summary_by_GROUP
#' # Discrete-time (more-ties)
#' D0 <- simTTE(n = 1000, TTE_A = c(0.1, 0.04), 
#' TTE_P = c(.15, 0.045), theta = 4, alpha0 = 2, alpha = -1, shape = 2, 
#' fixedfy = PADY, rHR = 1, m = 5)
#' summaryWO(D0, ref = "P")$summary_by_GROUP
#' # Discrete-time and move-down approach (less ties on death)
#' D1 <- simTTE(n = 1000, TTE_A = c(0.1, 0.04), 
#' TTE_P = c(.15, 0.045), theta = 4, alpha0 = 2, alpha = -1, shape = 2, 
#' fixedfy = PADY, rHR = 1, m = 5, hce_type = "md")
#' summaryWO(D1, ref = "P")$summary_by_GROUP
#' 
simTTE <- function (n, n0 = n, TTE_A, TTE_P = TTE_A, shape = 1, shape0 = shape,
                    fixedfy = 2, theta = 1, alpha0 = 1, alpha = 1, rHR = 1, m = Inf, hce_type = c("mi", "md"))
{ 
hce_type <- match.arg(hce_type)
# Normalize scalar inputs (in case vectors are passed)
n       <- n[1]
n0      <- n0[1]
fixedfy <- fixedfy[1]
theta   <- theta[1]
shape   <- shape[1]
shape0  <- shape0[1]
alpha0   <- alpha0[1]
alpha   <- alpha[1]
rHR <- rHR[1]
m <- m[1]
# Input validation with informative messages
stopifnot(
  `Heterogeneity theta must be non-negative.` = theta >= 0,
  `The recurrence hazard ratio must be non-negative.`               = rHR >= 0,
  `Shape parameters must be positive.`        = all(c(shape > 0, shape0 > 0)),
  `Sample sizes n and n0 must be positive integers.` =
    all(c(n > 0, n0 > 0, n %% 1 == 0, n0 %% 1 == 0)),
  `Follow-up duration fixedfy must be positive.` = fixedfy > 0,
  `TTE_A must contain exactly two elements.` = length(TTE_A) == 2,
  `TTE_P must contain exactly two elements.` = length(TTE_P) == 2,
  `TTE_A and TTE_P must be positive.` = all(c(TTE_A, TTE_P) > 0),
  `The categorization number m must be a positive numeric value.` = 
    is.numeric(m) && m > 0,
  `The recurrence hazard ratio must be a positive numeric value.` = 
    rHR > 0
)

# Parameterisation: convert incoming rate-like TTE_* to Weibull scale parameters
scale_A <- 1 / TTE_A  # Vector length 2: scale_A[1]=HOSP scale, scale_A[2]=DEATH scale for Arm A
scale_P <- 1 / TTE_P  # Same for Arm P

# Construct baseline dataset with treatment assignment
N <- n + n0
d <- data.frame(
  ID   = 1:N,
  TRTP = c(rep("A", n), rep("P", n0))
)

# Subject-level frailty term:
# - If theta = 0, frailty is degenerate at 1 (no heterogeneity).
# - If theta > 0, use gamma frailty with mean 1 and variance theta.
if (theta == 0) {
  Gm <- rep(1, N)
} else {
  Gm <- stats::rgamma(N, shape = 1 / theta, scale = theta)
}
gm  <- Gm[1:n]         # Frailty for Arm A
gm0 <- Gm[(n + 1):N]   # Frailty for Arm P

# Simulate first event: Hospitalisation (HOSP)
# Weibull inverse-CDF: T = ( -log(U) * scale / frailty )^(1/shape)
U  <- stats::runif(n)
U0 <- stats::runif(n0)
X  <- (-log(U)  * scale_A[1] / gm )^(1 / shape)   # Arm A HOSP time
X0 <- (-log(U0) * scale_P[1] / gm0)^(1 / shape0)  # Arm P HOSP time

# Simulate competing event: Death (DEATH)
# Frailty enters as gm^alpha0 for death
U  <- stats::runif(n)
U0 <- stats::runif(n0)
Y  <- (-log(U)  * scale_A[2] / gm^alpha0 )^(1 / shape)    # Arm A DEATH time
Y0 <- (-log(U0) * scale_P[2] / gm0^alpha0)^(1 / shape0)   # Arm P DEATH time

# Store raw event times
d$DEATH <- c(Y,  Y0)
d$HOSP  <- c(X,  X0)

# Determine first observed event within fixed follow-up (fixedfy)
d$EVENT1 <- ifelse(d$DEATH <= pmin(d$HOSP, fixedfy), "DEATH",
                   ifelse(d$HOSP <= fixedfy, "HOSP", "CEN"))

# AVAL1 is the observed time of EVENT1 (or censoring time if CEN)
d$AVAL1 <- ifelse(d$EVENT1 == "DEATH", d$DEATH,
                  ifelse(d$EVENT1 == "HOSP", d$HOSP, fixedfy))

# Keep frailty for second-event modelling
d$FRAILTY <- c(gm, gm0)

# Subset to subjects with first event = HOSP; they are at risk for post-hospitalization death
d0 <- d[d$EVENT1 == "HOSP", ]

if (nrow(d0) > 0) {
  # SECOND EVENT MODEL: Death after hospitalization using individual frailty
  # Individual-specific post-HOSP rate:
  #   rate_i = baseline_rate * (fixedfy / AVAL1_i)^alpha * FRAILTY_i
  # Convert rate to scale (scale = 1 / rate_i) for exponential parameterization.
  # Use a baseline rate similar to original behaviour: mean of the first-event rates (inverse of scales)
  baseline_rate <- mean(c(1 / scale_A[2], 1 / scale_P[2]))
  
  rate3 <- baseline_rate * (d0$AVAL1/fixedfy )^alpha * d0$FRAILTY^alpha0
  scale3 <- 1 /ifelse(d0$TRTP == "A", rHR*rate3, rate3)
  
  # Use exponential distribution (Weibull shape = 1) for time from HOSP to DEATH2
  shape_h <- 1
  DEATH2  <- (-log(stats::runif(nrow(d0))) * scale3)^(1 / shape_h)
  
  # Calendar time of second event
  d0$DEATH2 <- DEATH2 + d0$AVAL1
  
  # Second event indicators and observed time within fixed follow-up
  d0$EVENT2 <- ifelse(d0$DEATH2 <= fixedfy, "DEATH", "CEN")
  d0$AVAL2  <- pmin(d0$DEATH2, fixedfy)
} else {
  # No subjects had hospitalisation first; create empty second-event columns to merge back
  d0 <- d[, c("ID")]
  d0$EVENT2 <- character(0)
  d0$AVAL2  <- numeric(0)
}

# Merge second-event info back to the full cohort
d1 <- merge(d, d0[, c("ID", "EVENT2", "AVAL2")], by = "ID", all.x = TRUE)

# For subjects without second-event records, set as censored at fixedfy
d1[is.na(d1$EVENT2), "EVENT2"] <- "CEN"
d1[is.na(d1$AVAL2),  "AVAL2"]  <- fixedfy

# PADY: planned analysis day = fixed follow-up time
d1$PADY <- fixedfy

## Most-important outcome HCE (by default)
  # Overall endpoint group
  d1$GROUP <- ifelse(d1$EVENT1 == "DEATH" | d1$EVENT2 == "DEATH", "DEATH",
                     ifelse(d1$EVENT1 == "HOSP", "HOSP", "CEN"))
  
  # AVAL0: final observed time relevant for composite endpoint ranking
  d1$AVAL0 <- ifelse(d1$EVENT1 == "DEATH", d1$AVAL1,
                     ifelse(d1$EVENT2 == "DEATH", d1$AVAL2, d1$AVAL1))
  
  # Factorize GROUP to ensure consistent ordering
  d1$GROUP <- factor(d1$GROUP, levels = c("DEATH", "HOSP", "CEN"))


# Drop intermediate columns not needed in final output
d1$DEATH   <- NULL
d1$HOSP    <- NULL
d1$FRAILTY <- NULL
# If discrete time is specified
if(!is.infinite(m)){
  d1$AVAL1 <- fixedfy/m*floor(d1$AVAL1/(fixedfy/m))
  d1$AVAL2 <- fixedfy/m*floor(d1$AVAL2/(fixedfy/m))
  d1$AVAL0 <- ifelse(d1$EVENT1 == "DEATH", d1$AVAL1, ifelse(d1$EVENT2 == 
                                                              "DEATH", d1$AVAL2, d1$AVAL1))
} 
if(hce_type == "md"){
  d1$AVAL0[d1$EVENT2 == "DEATH"] <- d1$AVAL0[d1$EVENT2 == "DEATH"] + d1$AVAL1[d1$EVENT2 == "DEATH"]/(m + 2)
  d1$AVAL0[d1$EVENT1 == "DEATH"] <- d1$AVAL0[d1$EVENT1 == "DEATH"] + d1$PADY[d1$EVENT1 == "DEATH"]/(m + 2)
}
d1$Type <- hce_type
# Convert to an hce dataset
as_hce(d1)
}

#' Win odds calculation using a data frame
#'
#' @param x a data frame containing subject-level data.
#' @param AVAL variable in the data with ordinal analysis values.
#' @param TRTP the treatment variable in the data.
#' @param ref the reference treatment group.
#' @param alpha significance level. The default is 0.05.
#' @param WOnull the null hypothesis. The default is 1.
#' @param ... additional parameters.
#' @returns a data frame containing the win odds and its confidence interval. It contains the following columns:
#' * WO calculated win odds.
#' * LCL lower confidence limit.
#' * UCL upper confidence limit.
#' * SE standard error of the win odds.
#' * WOnull win odds of the null hypothesis (specified in the `WOnull` argument).
#' * alpha two-sided significance level for calculating the confidence interval (specified in the `alpha` argument).
#' * Pvalue p-value associated with testing the null hypothesis.
#' * WP calculated win probability.
#' * LCL_WP lower confidence limit for `WP`.
#' * UCL_WP upper confidence limit for `WP`.
#' * SE_WP standard error of the win probability.
#' * SD_WP standard deviation of the win probability, calculated as `SE_WP` multiplied by `sqrt(N)`.
#' * N total number of patients in the analysis.
#' @export
#' @md
#' @seealso [hce::calcWO()], [hce::calcWO.hce()], [hce::calcWO.formula()].
#' @references Gasparyan SB et al. "Adjusted win ratio with stratification: calculation methods and interpretation." Statistical Methods in Medical Research 30.2 (2021): 580-611. <doi:10.1177/0962280220942558>.
#' @examples
#' data(HCE4)
#' calcWO(x = HCE4, AVAL = "AVAL", TRTP = "TRTP", ref = "P")
calcWO.data.frame <- function(x, AVAL, TRTP, ref, alpha = 0.05, WOnull = 1, ...){
  data <- as.data.frame(x)
  alpha <- alpha[1]
  WOnull <- WOnull[1]
  WPnull <- WOnull/(WOnull + 1)
  ref <- ref[1]
  AVAL <- AVAL[1]
  TRTP <- TRTP[1]
  if(! AVAL %in% base::names(data))
    stop(paste0("The variable ", AVAL, " is not in the dataset."))
  if(! TRTP %in% base::names(data))
    stop(paste0("The variable ", TRTP, " is not in the dataset."))
  data$AVAL <- data[, base::names(data) == AVAL]
  data$TRTP <- data[, base::names(data) == TRTP]
  if(length(unique(data$TRTP)) != 2) {
    message1 <- base::paste("The variable", TRTP, "should have exactly 2 unique values, but has", length(unique(data$TRTP)))
    stop(message1)
  }
  if(!ref %in% unique(data$TRTP)) stop("Choose the reference from the values in TRTP.")
  data$TRTP <- base::ifelse(data$TRTP == ref, "P", "A")

  A <- base::rank(c(data$AVAL[data$TRTP == "A"], data$AVAL[data$TRTP == "P"]), ties.method = "average")
  B <- base::tapply(data$AVAL, data$TRTP, base::rank, ties.method = "average")
  n <- base::tapply(data$AVAL, data$TRTP, base::length)
  n1 <- n[["A"]]
  n0 <- n[["P"]]
  N <- n0 + n1

  d <- base::data.frame(R1 = A, R2 = base::c(B$A, B$P), TRTP = base::c(base::rep("A", n1), base::rep("P", n0)))
  d$R <- d$R1 - d$R2
  d$R0 <- base::ifelse(d$TRTP == "A", d$R/n0, d$R/n1)

  WP0 <- base::tapply(d$R0, d$TRTP, base::mean)
  VAR <- base::tapply(d$R0, d$TRTP, function(x) (base::length(x)-1)*stats::var(x)/base::length(x))
  SE_WP <- base::sqrt(base::sum(VAR/n))
  SD_WP <- SE_WP*base::sqrt(N)

  WP <- WP0[["A"]]
  WO <- WP/(1 - WP)
  SE <- SE_WP/(WP*(1-WP))
  Ca <- stats::qnorm(1 - alpha/2)
  LCL <- WO*base::exp(-Ca*SE)
  UCL <- WO*base::exp(Ca*SE)
  
  LCL_WP <- WP - Ca*SE_WP
  UCL_WP <- WP + Ca*SE_WP
  threshold <- base::abs(WP - WPnull)/SE_WP
  P <- 2*(1 - stats::pnorm(threshold))

  out <- base::data.frame(WO = WO, LCL = LCL, UCL = UCL, SE = SE, WOnull = WOnull, alpha = alpha, Pvalue = P, WP = WP, LCL_WP  = LCL_WP, UCL_WP = UCL_WP, SE_WP = SE_WP, SD_WP = SD_WP, N = N)
  return(out)
}

#' A generic function for summarizing win odds
#'
#' @param x an object used to select a method.
#' @param ... further arguments passed to or from other methods.
#' @return a data frame containing calculated values.
#' @export
#' @md
#' @seealso [hce::summaryWO.adhce()], [hce::summaryWO.hce()], [hce::summaryWO.formula()], [hce::summaryWO.data.frame()]  methods.
#'
summaryWO <- function(x, ...) {
  UseMethod("summaryWO")
}

#' Simulate random numbers from a Weibull distribution with gamma frailty
#'
#' @param n number of observations.
#' @param rate rate parameter of the Weibull distribution.
#' @param shape shape parameter of the Weibull distribution, with a default value of 1 (exponential).
#' @param theta variance parameter of the gamma frailty distribution, with a default value of 1. 
#' @details
#' Let \eqn{\gamma} denote a frailty term following a gamma distribution with
#' \eqn{shape = 1 / \theta} and \eqn{scale = \theta}. Then \eqn{\gamma} has mean 1
#' and variance \eqn{\theta}.
#'
#' Conditional on \eqn{\gamma}, the survival function is
#' \deqn{
#' S(t \mid \lambda, \alpha, \gamma) =
#' \exp\left(-\gamma \lambda t^\alpha\right), \qquad \alpha > 0,\ \lambda > 0,
#' }
#' where \eqn{\alpha} is the shape parameter and \eqn{\lambda} is the rate parameter.
#'
#' Integrating over the gamma frailty distribution gives the marginal survival
#' function
#' \deqn{
#' S(t \mid \lambda, \alpha, \theta) =
#' \left(1 + \theta \lambda t^\alpha\right)^{-1 / \theta},
#' \qquad \alpha > 0,\ \lambda > 0,\ \theta > 0.
#' }
#' This corresponds to the generalized log-logistic distribution (GLL) as implemented in [hce::GLL()].
#' 
#' As \eqn{\theta \to 0}, this reduces to the standard Weibull distribution
#' without frailty, with survival function 
#' \deqn{
#' S(t \mid \lambda, \alpha) = \exp\left(-\lambda t^\alpha\right).
#' }
#' This corresponds to the Weibull distribution in [stats::pweibull()] with `shape = shape` and `scale = rate^(-1 / shape)`.
#' 
#' When \eqn{\theta = 1}, the survival function becomes
#' \deqn{
#' S(t \mid \lambda, \alpha, 1) = \left(1 + \lambda t^\alpha\right)^{-1},
#' }
#' which is the survival function of a log-logistic distribution.
#' @return a vector of random numbers of length `n`.
#' @export
#' @md
#' @references Wienke A. "Frailty Models in Survival Analysis." Chapman and Hall/CRC (2010).
#' @seealso [hce::simTTE()] for simulation of a two-event time-to-event endpoint
#'   from this distribution under the illness-death model.
#'
#'   [hce::pGLL()] provides the cumulative distribution function of the
#'   generalized log-logistic distribution.
#' @examples
#' ## Example 1: each patient has a sampled time with potentially different
#' ## shape, rate, and theta values.
#' set.seed(123)
#' n <- 1000
#' d <- data.frame(ID = 1:(2*n), TRTP = rep(c("A", "P"), each = n), 
#' rate = rep(c(0.1, 0.15), each = n), shape = 1.5, theta = 1)
#' d$time1 <- rweibullGF(n = 2*n, rate = d$rate, shape = d$shape, theta = d$theta)
#' tapply(d$time1, d$TRTP, summary)
#' ## Example 2: all patients share the same parameter values.
#' d$time2 <- rweibullGF(n = 2*n, rate = 0.1, theta = 0)
#' tapply(d$time2, d$TRTP, summary)
rweibullGF <- function(n, rate, shape = 1, theta = 1) {
  n <- n[1]
  stopifnot("`n` must be a positive integer." = is.numeric(n) && n > 0,
            "`shape` must be a positive vector of length 1 or `n`." = is.numeric(shape) && all(shape > 0) && length(shape) %in% c(1, n),
            "`rate` must be a positive vector of length 1 or `n`." = is.numeric(rate) && all(rate > 0) && length(rate) %in% c(1, n),
            "`theta` must be a non-negative vector of length 1 or `n`." = is.numeric(theta) && all(theta >= 0) && length(theta) %in% c(1, n))
  n <- as.integer(n)
  #### Explicitly recycle scalar inputs to length n.
  shape <- rep_len(shape, n)
  rate  <- rep_len(rate, n)
  theta <- rep_len(theta, n)
  #################
  U <- stats::runif(n)                                      
  G <- rep(1, n)
  idx <- theta > 0
  ## Weibull proportional hazards parameterisation with gamma frailty:
  ## S(t | G) = exp(-G * rate * t^shape),
  ## where G is a gamma frailty term with mean 1 and variance theta.
  ## If theta = 0, then G = 1 and the model reduces to the standard Weibull PH model.
  ## In stats::rweibull(), the equivalent scale parameter is:
  ## scale = rate^(-1 / shape)
  G[idx] <- stats::rgamma(sum(idx), shape = 1/theta[idx], scale = theta[idx])   
  (- log(U) / (G * rate))^(1/shape)                 
}

#' Win statistics calculation using a data frame
#'
#' @param x a data frame containing subject-level data.
#' @param AVAL variable in the data with ordinal analysis values.
#' @param TRTP the treatment variable in the data.
#' @param ref the reference treatment group.
#' @param alpha 2-sided significance level. The default is 0.05.
#' @param WOnull the null hypothesis. The default is 1.
#' @param SE_WP_Type biased or unbiased standard error for win probability. The default is biased.
#' @param ... additional parameters.
#' @details When `SE_WP_Type = "unbiased"`, the calculations for win proportion, net benefit, and win odds utilize the unbiased standard error from Brunner-Konietschke (2025) paper which is a reformulation of the original formula proposed by Bamber (1975). In this case, Wilson-type confidence intervals are calculated for the win probability, net benefit, and win odds, following the approach proposed by Schüürhuis, Konietschke, and Brunner (2025).
#' @returns a list containing win statistics and their confidence intervals. It contains the following named data frames: 
#' * summary a data frame containing number of wins, losses, and ties of the active treatment group and the overall number of comparisons. 
#' * WP a data frame containing the win probability and its confidence interval. 
#' * NetBenefit a data frame containing the net benefit and its confidence interval. This is just a `2x-1` transformation of WP and its CI.
#' * WO a data frame containing the win odds and its confidence interval. 
#' * WR1 a data frame containing the win ratio and its confidence interval, using the transformed standard error of the `gamma` statistic. 
#' * WR2 a data frame containing the win ratio and its confidence interval, using the standard error calculated using `Pties`. 
#' * gamma a data frame containing Goodman Kruskal's `gamma` and its confidence interval. 
#' * SE a data frame containing standard errors used to calculated the Confidence intervals for win statistics. 
#' 
#' When `SE_WP_Type = "unbiased"`, the `WP`, `WO` and `NetBenefit` estimators use the unbiased variance estimator of `WP`. Additionally, a Wilson-type range-preserving confidence interval is provided: 
#' * WP_W a data frame containing the win probability and its range-preserving confidence interval. 
#' * NetBenefit_W a data frame containing the net benefit and its range-preserving confidence interval. 
#' * WO_W a data frame containing the win odds and its range-preserving confidence interval. 
#' @export
#' @md
#' @seealso [hce::calcWINS()], [hce::calcWINS.hce()], [hce::calcWINS.formula()].
#' @references
#' The theory of win statistics is covered in the following papers: 
#' * Win proportion and win odds confidence interval calculation: 
#' \cr \cr Somers RH (1962) “A New Asymmetric Measure of Association for Ordinal Variables." American Sociological Review 27.6: 799-811. <doi:10.2307/2090408>.
#' \cr \cr Bamber D (1975) "The area above the ordinal dominance graph and the area below the receiver operating characteristic graph." Journal of Mathematical Psychology 12.4: 387-415. <doi:10.1016/0022-2496(75)90001-2>.
#' \cr \cr DeLong ER et al. (1988) "Comparing the Areas Under Two or More Correlated Receiver Operating Characteristic Curves: A Nonparametric Approach." Biometrics 44.3: 837-845. <doi:10.2307/2531595>.
#' \cr \cr Brunner E et al. (2021) "Win odds: an adaptation of the win ratio to include ties." Statistics in Medicine 40.14: 3367-3384. <doi:10.1002/sim.8967>.
#' \cr \cr Gasparyan SB et al. (2021) "Adjusted win ratio with stratification: calculation methods and interpretation." Statistical Methods in Medical Research 30.2: 580-611. <doi:10.1177/0962280220942558>.
#' \cr \cr Gasparyan SB et al. (2021) "Power and sample size calculation for the win odds test: application to an ordinal endpoint in COVID-19 trials." Journal of Biopharmaceutical Statistics 31.6: 765-787. <doi:10.1080/10543406.2021.1968893>.
#' \cr \cr Brunner E, Konietschke F. (2025) "An unbiased rank-based estimator of the Mann–Whitney variance including the case of ties." Statistical Papers 66.20. <doi:10.1007/s00362-024-01635-0>.
#' * Win ratio: the first CI utilizes the standard error derived from the `gamma` statistic standard error as outlined by:
#' \cr \cr Gasparyan SB, Kowalewski EK, Buenconsejo J, Koch GG. (2023) "Hierarchical Composite Endpoints in COVID-19: The DARE-19 Trial." In Case Studies in Innovative Clinical Trials, Chapter 7, 95–148. Chapman; Hall/CRC. <doi:10.1201/9781003288640-7>.
#' * Win ratio: the second CI utilizes the standard error presented by:
#' \cr \cr Yu RX, Ganju J. (2022) "Sample size formula for a win ratio endpoint." Statistics in Medicine 41.6: 950-63. <doi:10.1002/sim.9297>.
#' * Goodman Kruskal's `gamma` and CI: matches implementation in `DescTools::GoodmanKruskalGamma()` and based on:
#' \cr \cr Agresti A. (2002) Categorical Data Analysis. John Wiley & Sons, pp. 57-59. <doi:10.1002/0471249688>.
#' \cr \cr Brown MB, Benedetti JK. (1977) "Sampling Behavior of Tests for Correlation in Two-Way Contingency Tables." Journal of the American Statistical Association 72, 309-315. <doi:10.1080/01621459.1977.10480995>.
#' \cr \cr Goodman LA, Kruskal WH. (1954) "Measures of association for cross classifications." Journal of the American Statistical Association 49, 732-764. <doi:10.1080/01621459.1954.10501231>.
#' \cr \cr Goodman LA, Kruskal WH. (1963) "Measures of association for cross classifications III: Approximate sampling theory." Journal of the American Statistical Association 58, 310-364. <doi:10.1080/01621459.1963.10500850>.
#' * Unbiased variance estimator for `WP` and Wilson-type range -preserving confidence intervals are based on:
#' \cr \cr Brunner E, Konietschke F. (2025) "An unbiased rank-based estimator of the Mann–Whitney variance including the case of ties." Statistical Papers 66: 20. <doi:10.1007/s00362-024-01635-0>.
#' \cr \cr Schüürhuis S, Konietschke F, Brunner E. (2025) "A New Approach to the Nonparametric Behrens–Fisher Problem With Compatible Confidence Intervals." Biometrical Journal 67.6. <doi:10.1002/bimj.70096>.
#' @examples
#' # Example 1 - Simple use
#' calcWINS(x = COVID19b, AVAL = "GROUP", TRTP = "TRTP", ref = "Placebo")
#' calcWINS(x = COVID19b, AVAL = "GROUP", TRTP = "TRTP", ref = "Placebo", SE_WP_Type = "unbiased")
#' # Example 2 - Different variance estimators  
#' FREQ <- c(16, 5, 0, 1, 0, 4, 1, 5, 7, 2)
#' dat0 <- data.frame(AVAL = rep(5:1, 2), TRTP = rep(c('A', 'P'), each = 5))
#' dat <- dat0[rep(row.names(dat0), FREQ),]
#' ## By default, the variance estimator applies a 1/n weighting to the coefficients
#' ## This approach matches the Somers' D (C|R) estimator, where `C|R` indicates that 
#' ## the column variable Y is treated as the independent variable and the row
#' ## variable X is treated as the dependent variable.
#' calcWINS(AVAL ~ TRTP, data = dat)$WP
#' ## The Brunner-Konietschke estimator
#' UNB <- calcWINS(AVAL ~ TRTP, data = dat,  SE_WP_Type = "unbiased")
#' cbind(UNB$WP, SE = UNB$SE$WP_SE)
#' ## The Brunner-Munzel test, based on the DeLong-Clarke-Pearson formula for the variance estimation,
#' ## applies 1/(n - 1) weighting to the coefficients.
#' dat1 <- IWP(data = dat, AVAL = "AVAL", TRTP = "TRTP", ref = "P")
#' WP <- tapply(dat1$AVAL_, dat1$TRTP, mean)
#' VAR <- tapply(dat1$AVAL_, dat1$TRTP, var)
#' N <- tapply(dat1$AVAL_, dat1$TRTP, length)
#' SE <- sqrt(sum(VAR/N))
#' c(WP = WP[[1]], SE = SE)
#' # Example 3 - Simulations: Biased vs unbiased vs Wilson confidence intervals for Win Probability
#' set.seed(1)
#' n0 <- 5; n1 <- 7; p0 <- 0.2; p1 <- 0.5; x <- 1:20; delta <- 0.15
#' WP0 <- (p1 - p0)/2 + 0.5
#' DAT <- NULL
#' for(i in x){
#'   dat <- data.frame(AVAL = c(rbinom(n1, size = 1, p1), rbinom(n0, size = 1, p0)), 
#'   TRTP = c(rep("A", n1), rep("P", n0)))
#'   CL1 <- calcWINS(x = dat, AVAL = "AVAL", TRTP = "TRTP", ref = "P")$WP
#'   CL1$Type <- "biased"
#'   fit <- calcWINS(x = dat, AVAL = "AVAL", TRTP = "TRTP", 
#'                 ref = "P", SE_WP_Type = "unbiased")
#'   CL2 <- fit$WP
#'   CL2$Type <- "unbiased"
#'   CL3 <- fit$WP_W
#'   CL3$Type <- "Wilson"
#'   DAT <- rbind(DAT, CL1, CL2, CL3)
#'   }
#' WP <- DAT$WP[DAT$Type == "unbiased"]
#' plot(x, WP, pch = 19, xlab = "Simulations", ylab = "Win Probability", 
#'             ylim = c(0., 1.1), xlim = c(0, max(x) + 1))
#' points(x + delta, WP, pch = 19)
#' points(x + 2*delta, WP, pch = 19)
#' arrows(x, DAT$LCL[DAT$Type == "unbiased"], 
#'        x, DAT$UCL[DAT$Type == "unbiased"], angle = 90, code = 3, length = 0.05, "green")
#'        arrows(x + delta, DAT$LCL[DAT$Type == "biased"], 
#'        x + delta, DAT$UCL[DAT$Type == "biased"], angle = 90, code = 3, 
#'        length = 0.05, col = "orange")
#' arrows(x + 2*delta, DAT$LCL[DAT$Type == "Wilson"], 
#'        x + 2*delta, DAT$UCL[DAT$Type == "Wilson"], angle = 90, code = 3, 
#'        length = 0.05, col = "blue")
#' abline(h = c(WP0, 1), col = c("darkgreen", "darkred"), lty = c(3, 4))
#' legend("bottomleft", legend = c("True WP", "UnBiased", "Biased", "Wilson", "Null"), 
#'        col = c("darkgreen", "green", "orange", "blue", "darkred"), 
#'        lty = c(3, 1, 1, 1, 4), cex = 0.75, ncol = 3)
#'        title("Win Probability: Biased vs Unbiased vs Wilson CI")
#' # End of Example 3
calcWINS.data.frame <- function(x, AVAL, TRTP, ref, alpha = 0.05, WOnull = 1, SE_WP_Type = c("biased", "unbiased"), ...){
  #The function relies on calcWO() for the win odds calculation, and summaryWO() for the win counts by group.
  SE_WP_Type <- match.arg(SE_WP_Type)
  data <- as.data.frame(x)
  alpha <- alpha[1]
  Ca <- stats::qnorm(1 - alpha/2)
  WOnull <- WOnull[1]
  WPnull <- WOnull/(WOnull + 1)
  gammanull <- (WOnull - 1)/(WOnull + 1)
  data$AVAL <- data[[AVAL]]
  data$TRTP <- data[[TRTP]]
  if (length(unique(data$TRTP)) != 2) 
    stop("The dataset should contain two treatment groups.")
  if (!ref %in% unique(data$TRTP)) 
    stop("Choose the reference from the values in TRTP.")
   stopifnot("The significance level `alpha` should be between 0 and 1." 
            = all(alpha > 0 & alpha < 1))
   stopifnot("The null hypothesis for the win odds should be positive." 
             = WOnull > 0)
  data$TRTP <- base::ifelse(data$TRTP == ref, "P", "A")
  res0 <- calcWO(x = data, AVAL = "AVAL", TRTP = "TRTP", ref = "P", 
                 alpha = alpha, WOnull = WOnull)
  Res <- summaryWO(x = data, AVAL = "AVAL", TRTP = "TRTP", 
                   ref = "P", GROUP = "AVAL")
  res1 <- Res$summary_by_GROUP
  res2 <- table(data$TRTP, data$AVAL)
  res2 <- as.data.frame(res2)
  names(res2) <- c("TRTP", "GROUP", "N")
  res3 <- merge(res1, res2, by = c("TRTP", "GROUP"))
  res3$C <- ifelse(res3$TRTP == "A", res3$WIN/res3$N, res3$LOSS/res3$N)
  res3$D <- ifelse(res3$TRTP != "A", res3$WIN/res3$N, res3$LOSS/res3$N)
  P <- sum(res3$C * res3$N)
  Q <- sum(res3$D * res3$N)
  WR <- P/Q
  Pties <- Res$summary$TIE[1]/Res$summary$TOTAL[1]
  gamma <- (P - Q)/(P + Q)
  VAR <- 16/(P + Q)^4 * sum((res3$C * Q - res3$D * P)^2 * res3$N)
  gamma_SE <- sqrt(VAR)
  logWR_SE1 <- 2/(WR * (1 - gamma)^2) * gamma_SE
  logWR_SE2 <- 4/sqrt(3 * nrow(data)) * sqrt((1 + Pties)/(1 - 
                                                            Pties))
  threshold0 <- base::abs(gamma - gammanull)/gamma_SE
  P0 <- 2 * (1 - stats::pnorm(threshold0))
  threshold1 <- base::abs(log(WR) - log(WOnull))/logWR_SE1
  P1 <- 2 * (1 - stats::pnorm(threshold1))
  threshold2 <- base::abs(log(WR) - log(WOnull))/logWR_SE2
  P2 <- 2 * (1 - stats::pnorm(threshold2))
  if (SE_WP_Type == "unbiased") {
    WP <- res0[, "WP"]
    WO <- WP/(1 - WP)
    nA <- sum(res2$N[res2$TRTP == "A"])
    nP <- sum(res2$N[res2$TRTP == "P"])
    K <- nA * nP/((nA - 1) * (nP - 1))
    BBK <- K * (res0$SE_WP^2 - (WP * (1 - WP) - 0.25 * Pties)/(nA * nP))
    SE_WP <- sqrt(BBK)
    threshold_WP <- base::abs(WP - WPnull)/SE_WP
    P_WP <- 2 * (1 - stats::pnorm(threshold_WP))
    SE <- SE_WP/(WP * (1 - WP))
    LCL <- WO * base::exp(-Ca * SE)
    UCL <- WO * base::exp(Ca * SE)
  }
  else {
    SE_WP <- res0[, "SE_WP"]
    P_WP <- res0[, "Pvalue"]
  }
  out <- list()
  out$summary <- data.frame(WIN = P/2, LOSS = Q/2, TIE = Res$summary$TIE[1], 
                            TOTAL = Res$summary$TOTAL[1], Pties = Pties)
  out$WP <- data.frame(WP = res0[, "WP"], LCL = res0[, "WP"] - 
                         Ca * SE_WP, UCL = res0[, "WP"] + Ca * SE_WP, Pvalue = P_WP)
  out$NetBenefit <- data.frame(NetBenefit = 2 * res0[, "WP"] - 
                                 1, LCL = 2 * (res0[, "WP"] - Ca * SE_WP) - 1, UCL = 2 * 
                                 (res0[, "WP"] + Ca * SE_WP) - 1, Pvalue = P_WP)
  
  if(SE_WP_Type == "unbiased"){
    out$WO <- data.frame(WO = WO, LCL = LCL, 
                         UCL = UCL, Pvalue =  P_WP)
  } else {
    out$WO <- res0[, c("WO", "LCL", "UCL", "Pvalue")]  
  }
  
  out$WR1 <- data.frame(WR = WR, LCL1 = WR * exp(-Ca * logWR_SE1), 
                        UCL1 = WR * exp(Ca * logWR_SE1), Pvalue1 = P1)
  out$WR2 <- data.frame(WR = WR, LCL2 = WR * exp(-Ca * logWR_SE2), 
                        UCL2 = WR * exp(Ca * logWR_SE2), Pvalue2 = P2)
  out$gamma <- data.frame(gamma = gamma, LCL = gamma - Ca * 
                            gamma_SE, UCL = gamma + Ca * gamma_SE, Pvalue = P0)
  out$SE <- data.frame(WP_SE = SE_WP, NetBenefit_SE = 2 * SE_WP, 
                       logWR_SE1 = logWR_SE1, logWR_SE2 = logWR_SE2, gamma_SE = gamma_SE)
  ### New addition to cover Wilson-type confidence interval
  if (SE_WP_Type == "unbiased"){
    m <- min(nA, nP)
    Ca2 <- Ca^2
    if(WP > 0.01 & WP < 0.99){
      q <- SE_WP^2/(WP * (1 - WP))
      Diff <- sqrt(q^2*Ca2^2 + 4*q*WP*(1 - WP)*Ca2)
      LCL <- 1/(2*(1 + q*Ca^2))*(2*WP + q*Ca^2 - Diff)
      UCL <- 1/(2*(1 + q*Ca^2))*(2*WP + q*Ca^2 + Diff)
      WO <- WP/(1 - WP + q)
      threshold_WP <- base::abs(WP - WPnull)/sqrt(q*WPnull*(1 - WPnull))
    } else if (WP >= 0.99){
      LCL <- m/(m + Ca2)
      UCL <- 1
      threshold_WP <-  sqrt(m)
      WO <- WP/(1 - WP)
    } else {
      LCL <- 0
      UCL <- Ca2/(m + Ca2)
      threshold_WP <- sqrt(m)
      WO <- WP/(1 - WP)
    }
    P_WP <- 2 * (1 - stats::pnorm(threshold_WP))
    out$WP_W <- data.frame(WP = WP, LCL = LCL, UCL = UCL, Pvalue = P_WP)
    LCL_WO <- LCL/(1 - LCL)
    UCL_WO <- UCL/(1 - UCL)
    out$NetBenefit_W <- data.frame(NetBenefit = 2*WP - 1, LCL = 2*LCL - 1, UCL = 2*UCL - 1, Pvalue = P_WP)
    out$WO_W <- data.frame(WO = WO, LCL = LCL_WO, UCL = UCL_WO, Pvalue = P_WP)
  }
  
  return(out)
}

#' Proportion of wins/losses/ties given the win odds and the win ratio
#'
#' @param WO win odds.
#' @param WR win ratio.
#' @param Overall number of comparisons, the sample size of 
#' the active treatment multiplied by the sample size of the placebo. 
#' The default is 1, hence gives the proportion. 
#' @param alpha significance level for the win ratio confidence interval. The default is `NULL` hence the confidence interval is not produced.
#' @param N the combined sample size of two treatment groups. The default is `NULL`. If `alpha` is specified then either `N` should be specified or `Overall > 1`. 
#' For given `Overall`, the pooled sample size is calculated as `N = 2*sqrt(Overall)`.
#' @details 
#' ![](propWINS.png "Calculation formula")
#' @return a data frame with a number (or proportion if `Overall = 1`) of wins/losses/ties. If `alpha` is specified returns also `WR` confidence interval.
#' @export
#' @md
#' @references 
#' * For the relationship between win odds and win ratio see 
#' \cr \cr Gasparyan SB et al. "Hierarchical Composite Endpoints in COVID-19: The DARE-19 Trial". Case Studies in Innovative Clinical Trials, Chapter 7 (2023): 95-148. Chapman and Hall/CRC. <doi:10.1201/9781003288640-7>.
#' * The win ratio CI uses the standard error presented in
#' \cr \cr Yu RX, Ganju J. (2022) "Sample size formula for a win ratio endpoint." Statistics in Medicine 41.6: 950-63. <doi:10.1002/sim.9297>.
#' @examples
#' # Example 1
#' propWINS(WR = 2, WO = 1.5)
#' # Example 2 - Back-calculation 
#' COVID19HCE <- hce(GROUP = COVID19$GROUP, TRTP = COVID19$TRTP)
#' res <- calcWINS(COVID19HCE)
#' WR <- res$WR1$WR
#' WO <- res$WO$WO
#' Overall <- res$summary$TOTAL
#' propWINS(WR = WR, WO = WO, Overall = Overall)
#' ## Verify 
#' res$summary
#' # Example 3 - Confidence interval
#'  propWINS(WR = 1.4, WO = 1.3, alpha = 0.05, Overall = 2500)
#'  propWINS(WR = 2, WO = 1.5, alpha = 0.01, N = 500)
propWINS <- function(WO, WR, Overall = 1, alpha = NULL, N = NULL){
  WO <- WO[1]
  WR <- WR[1]
  Overall <- Overall[1]
  alpha <- alpha[1]
  
  WP <- WO/(WO + 1)
 
  
  if(is.null(alpha)) {
    L <- Overall*(2*WP - 1)/(WR - 1)
    L <- round(L)
    W <- round(WR*L)
    TIE <- Overall - W - L
    d <- data.frame(WIN = W, LOSS = L, TIE = TIE, TOTAL = Overall, WR = WR, WO = WO)
    return(d)
  } else {
    if(is.null(N) & Overall == 1)
      stop("Either Overall should be > 1 or N should be specified.")
    
    if(!is.null(N) & Overall > 1)
      stop("Either Overall and N cannot both be specified.")
    
    if(Overall > 1) 
      N <- 2*sqrt(Overall)
    
    if(Overall == 1 & !is.null(N)) 
      Overall <- (N/2)^2
    
    L <- Overall*(2*WP - 1)/(WR - 1)
    L <- round(L)
    W <- round(WR*L)
    TIE <- Overall - W - L
    d <- data.frame(WIN = W, LOSS = L, TIE = TIE, TOTAL = Overall, WR = WR, WO = WO)
    
    
    k <- 0.5
    Ca <- stats::qnorm(1 - alpha/2)
    VAR <- 1/N*(4/(3*k*(1 - k)))*(1 + TIE/Overall)/(1 - TIE/Overall)
    CL <- WR*exp(Ca*sqrt(VAR)*c(- 1, 1))
    d$WR_LCL <- CL[1]
    d$WR_UCL <- CL[2]
    d$alpha <- alpha
    d$N <- N
    return(d)
  }
  
}

#' The Generalized Log-Logistic Distribution
#'
#' Density, distribution function, quantile function, hazard, cumulative hazard,
#' and random generation for the generalized log-logistic distribution.
#'
#' @param x,q vector of quantiles.
#' @param p vector of probabilities.
#' @param n number of observations. Must be a single non-negative integer.
#' @param rate positive rate parameter of the distribution.
#' @param shape positive shape parameter of the distribution.
#' @param theta non-negative additional parameter of the distribution, with default
#'   value `0`.
#' @param log logical; if `TRUE`, densities are given as logarithms.
#' @name GLL
#'
#' @details
#' The generalized log-logistic distribution with rate parameter \eqn{\lambda > 0},
#' shape parameter \eqn{\alpha > 0}, and parameter \eqn{\theta \ge 0} has survival
#' function
#' \deqn{
#' S(t \mid \lambda, \alpha, \theta) =
#' \left(1 + \theta \lambda t^\alpha\right)^{-1 / \theta},
#' \qquad \theta > 0.
#' }
#'
#' The corresponding cumulative distribution function is
#' \deqn{
#' F(t \mid \lambda, \alpha, \theta) =
#' 1 - \left(1 + \theta \lambda t^\alpha\right)^{-1 / \theta}.
#' }
#'
#' The quantile function is
#' \deqn{
#' Q(p \mid \lambda, \alpha, \theta) =
#' \left(
#' \frac{(1 - p)^{-\theta} - 1}{\theta \lambda}
#' \right)^{1 / \alpha},
#' \qquad 0 \le p \le 1,\ \theta > 0.
#' }
#'
#' The cumulative hazard function is
#' \deqn{
#' H(t \mid \lambda, \alpha, \theta) =
#' \frac{\log\left(1 + \theta \lambda t^\alpha\right)}{\theta}.
#' }
#'
#' The density function is
#' \deqn{
#' f(t \mid \lambda, \alpha, \theta) =
#' \lambda \alpha t^{\alpha - 1}
#' \left(1 + \theta \lambda t^\alpha\right)^{-1 / \theta - 1}.
#' }
#'
#' For `dGLL(..., log = TRUE)`, the logarithm of the density is returned. For
#' \eqn{\theta > 0},
#' \deqn{
#' \log f(t \mid \lambda, \alpha, \theta) =
#' \log(\lambda) + \log(\alpha) + (\alpha - 1)\log(t)
#' - \left(\frac{1}{\theta} + 1\right)\log\left(1 + \theta \lambda t^\alpha\right).
#' }
#'
#' The hazard function is
#' \deqn{
#' h(t \mid \lambda, \alpha, \theta) =
#' \frac{\lambda \alpha t^{\alpha - 1}}
#' {1 + \theta \lambda t^\alpha}.
#' }
#'
#' Random generation is performed by inverse transform sampling. If
#' \eqn{U \sim \mathrm{Uniform}(0,1)}, then
#' \deqn{
#' T = Q(U \mid \lambda, \alpha, \theta)
#' }
#' has the generalized log-logistic distribution.
#'
#' As \eqn{\theta \to 0}, this reduces to the Weibull distribution with survival
#' function
#' \deqn{
#' S(t \mid \lambda, \alpha) = \exp\left(-\lambda t^\alpha\right).
#' }
#' The corresponding quantile function is
#' \deqn{
#' Q(p \mid \lambda, \alpha) =
#' \left(
#' \frac{-\log(1 - p)}{\lambda}
#' \right)^{1 / \alpha}.
#' }
#' In this case, `dGLL(..., log = TRUE)` returns the corresponding Weibull
#' log-density.
#' This corresponds to the Weibull distribution in [stats::dweibull()],
#' [stats::pweibull()], [stats::qweibull()], and [stats::rweibull()] with
#' `shape = shape` and `scale = rate^(-1 / shape)`.
#'
#' When \eqn{\theta = 1}, the survival function becomes
#' \deqn{
#' S(t \mid \lambda, \alpha, 1) = \left(1 + \lambda t^\alpha\right)^{-1},
#' }
#' corresponding to the log-logistic distribution.
#'
#' @return
#' For `dGLL()`, `pGLL()`, `qGLL()`, `hGLL()`, and `HGLL()`, a numeric vector of
#' the same length as the main input (`x`, `q`, or `p`). For `rGLL()`, a numeric
#' vector of length `n`.
#'
#' @md
#'
#' @references
#' Wienke A. *Frailty Models in Survival Analysis*. Chapman and Hall/CRC (2010).
#'
#' @seealso
#' [hce::rweibullGF()] for simulation of a Weibull distribution with gamma frailty.
#'
#' @examples
#' ## Example 1: Compare the density of GLL with different values of `theta`
#' x <- seq(0, 10, length.out = 100)
#' y1 <- pGLL(x, rate = 0.5, shape = 2, theta = 1)
#' y0 <- pGLL(x, rate = 0.5, shape = 2, theta = 0)
#' plot(x, y1, type = "l", ylab = "Cumulative distribution function")
#' lines(x, y0, col = "red", lty = 2)
NULL

#' @export
#' @rdname GLL
dGLL <- function(x, rate, shape = 1, theta = 0, log = FALSE) {
  n <- length(x)
  log <- as.logical(log[1])
  
  stopifnot(
    "`shape` must be a positive vector of length 1 or length of `x`." =
      is.numeric(shape) && all(shape > 0) && length(shape) %in% c(1, n),
    "`rate` must be a positive vector of length 1 or length of `x`." =
      is.numeric(rate) && all(rate > 0) && length(rate) %in% c(1, n),
    "`theta` must be a non-negative vector of length 1 or length of `x`." =
      is.numeric(theta) && all(theta >= 0) && length(theta) %in% c(1, n),
    "`x` quantiles must be a non-negative vector." =
      is.numeric(x) && all(x >= 0)
  )
  
  rate <- rep_len(rate, length.out = n)
  shape <- rep_len(shape, length.out = n)
  theta <- rep_len(theta, length.out = n)
  
  out <- numeric(n)
  i0 <- theta == 0
  i1 <- theta > 0
  
  if (any(i0)) {
    out[i0] <- stats::dweibull(
      x[i0],
      shape = shape[i0],
      scale = rate[i0]^(-1 / shape[i0]),
      log = log
    )
  }
  
  if (any(i1)) {
    if (log) {
      out[i1] <- log(rate[i1]) + log(shape[i1]) -
        (1 / theta[i1] + 1) * log1p(theta[i1] * rate[i1] * x[i1]^shape[i1])
      
      iz <- i1 & x == 0
      ip <- i1 & x > 0
      
      out[ip] <- out[ip] + (shape[ip] - 1) * log(x[ip])
      
      if (any(iz)) {
        out[iz] <- ifelse(
          shape[iz] < 1,
          Inf,
          ifelse(shape[iz] == 1, log(rate[iz]) + log(shape[iz]), -Inf)
        )
      }
    } else {
      out[i1] <- rate[i1] * shape[i1] * x[i1]^(shape[i1] - 1) *
        (1 + theta[i1] * rate[i1] * x[i1]^shape[i1])^(-1 / theta[i1] - 1)
    }
  }
  
  out
}

#' @export
#' @rdname GLL
pGLL <- function(q, rate, shape = 1, theta = 0) {
  n <- length(q)
  stopifnot(
    "`shape` must be a positive vector of length 1 or length of `q`." =
      is.numeric(shape) && all(shape > 0) && length(shape) %in% c(1, n),
    "`rate` must be a positive vector of length 1 or length of `q`." =
      is.numeric(rate) && all(rate > 0) && length(rate) %in% c(1, n),
    "`theta` must be a non-negative vector of length 1 or length of `q`." =
      is.numeric(theta) && all(theta >= 0) && length(theta) %in% c(1, n),
    "`q` quantiles must be a non-negative vector." =
      is.numeric(q) && all(q >= 0)
  )
  
  rate <- rep_len(rate, length.out = n)
  shape <- rep_len(shape, length.out = n)
  theta <- rep_len(theta, length.out = n)
  
  out <- numeric(n)
  i0 <- theta == 0
  i1 <- theta > 0
  
  if (any(i0)) {
    out[i0] <- stats::pweibull(
      q[i0],
      shape = shape[i0],
      scale = rate[i0]^(-1 / shape[i0])
    )
  }
  
  if (any(i1)) {
    out[i1] <- 1 - (1 + theta[i1] * rate[i1] * q[i1]^shape[i1])^(-1 / theta[i1])
  }
  
  out
}

#' @export
#' @rdname GLL
qGLL <- function(p, rate, shape = 1, theta = 0) {
  n <- length(p)
  stopifnot(
    "`shape` must be a positive vector of length 1 or length of `p`." =
      is.numeric(shape) && all(shape > 0) && length(shape) %in% c(1, n),
    "`rate` must be a positive vector of length 1 or length of `p`." =
      is.numeric(rate) && all(rate > 0) && length(rate) %in% c(1, n),
    "`theta` must be a non-negative vector of length 1 or length of `p`." =
      is.numeric(theta) && all(theta >= 0) && length(theta) %in% c(1, n),
    "`p` probabilities must be a numeric vector with values in [0, 1]." =
      is.numeric(p) && all(p >= 0) && all(p <= 1)
  )
  
  rate <- rep_len(rate, length.out = n)
  shape <- rep_len(shape, length.out = n)
  theta <- rep_len(theta, length.out = n)
  
  out <- numeric(n)
  i0 <- theta == 0
  i1 <- theta > 0
  
  if (any(i0)) {
    out[i0] <- stats::qweibull(
      p[i0],
      shape = shape[i0],
      scale = rate[i0]^(-1 / shape[i0])
    )
  }
  
  if (any(i1)) {
    out[i1] <- (((1 - p[i1])^(-theta[i1]) - 1) /
                  (theta[i1] * rate[i1]))^(1 / shape[i1])
  }
  
  out
}

#' @export
#' @rdname GLL
rGLL <- function(n, rate, shape = 1, theta = 0) {
  stopifnot(
    "`n` must be a single non-negative integer." =
      is.numeric(n) && length(n) == 1 && n >= 0 && n == as.integer(n),
    "`shape` must be a positive vector of length 1 or length `n`." =
      is.numeric(shape) && all(shape > 0) && length(shape) %in% c(1, n),
    "`rate` must be a positive vector of length 1 or length `n`." =
      is.numeric(rate) && all(rate > 0) && length(rate) %in% c(1, n),
    "`theta` must be a non-negative vector of length 1 or length `n`." =
      is.numeric(theta) && all(theta >= 0) && length(theta) %in% c(1, n)
  )
  
  qGLL(
    p = stats::runif(n),
    rate = rate,
    shape = shape,
    theta = theta
  )
}

#' @export
#' @rdname GLL
hGLL <- function(x, rate, shape = 1, theta = 0) {
  n <- length(x)
  stopifnot(
    "`shape` must be a positive vector of length 1 or length of `x`." =
      is.numeric(shape) && all(shape > 0) && length(shape) %in% c(1, n),
    "`rate` must be a positive vector of length 1 or length of `x`." =
      is.numeric(rate) && all(rate > 0) && length(rate) %in% c(1, n),
    "`theta` must be a non-negative vector of length 1 or length of `x`." =
      is.numeric(theta) && all(theta >= 0) && length(theta) %in% c(1, n),
    "`x` quantiles must be a non-negative vector." =
      is.numeric(x) && all(x >= 0)
  )
  
  rate <- rep_len(rate, length.out = n)
  shape <- rep_len(shape, length.out = n)
  theta <- rep_len(theta, length.out = n)
  
  out <- numeric(n)
  i0 <- theta == 0
  i1 <- theta > 0
  
  if (any(i0)) {
    out[i0] <- rate[i0] * shape[i0] * x[i0]^(shape[i0] - 1)
  }
  
  if (any(i1)) {
    out[i1] <- rate[i1] * shape[i1] * x[i1]^(shape[i1] - 1) /
      (1 + theta[i1] * rate[i1] * x[i1]^shape[i1])
  }
  
  out
}

#' @export
#' @rdname GLL
HGLL <- function(x, rate, shape = 1, theta = 0) {
  n <- length(x)
  stopifnot(
    "`shape` must be a positive vector of length 1 or length of `x`." =
      is.numeric(shape) && all(shape > 0) && length(shape) %in% c(1, n),
    "`rate` must be a positive vector of length 1 or length of `x`." =
      is.numeric(rate) && all(rate > 0) && length(rate) %in% c(1, n),
    "`theta` must be a non-negative vector of length 1 or length of `x`." =
      is.numeric(theta) && all(theta >= 0) && length(theta) %in% c(1, n),
    "`x` quantiles must be a non-negative vector." =
      is.numeric(x) && all(x >= 0)
  )
  
  rate <- rep_len(rate, length.out = n)
  shape <- rep_len(shape, length.out = n)
  theta <- rep_len(theta, length.out = n)
  
  out <- numeric(n)
  i0 <- theta == 0
  i1 <- theta > 0
  
  if (any(i0)) {
    out[i0] <- rate[i0] * x[i0]^shape[i0]
  }
  
  if (any(i1)) {
    out[i1] <- log1p(theta[i1] * rate[i1] * x[i1]^shape[i1]) / theta[i1]
  }
  
  out
}

---
title: "Hierarchical composite endpoints"
date: "`r format(as.Date('2023-11-01'), '%d %B, %Y')`"
author:
  - name: "Samvel B. Gasparyan"
    affiliation: https://gasparyan.co/
output:
  rmarkdown::html_document:
          theme: "darkly"
          highlight: "zenburn"
          toc: true
          toc_float: true
          link-citations: true 
bibliography: REFERENCES.bib
vignette: >
  %\VignetteIndexEntry{Hce}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

## Introduction


### Setup

Load the package `hce` and check the version

```{r setup}
library(hce)
packageVersion("hce")
```

For citing the package run `citation("hce")` [@hce].

### Definitions

Prioritized outcome composite endpoints (POCE) are a general class of endpoints combining different clinical outcomes of patients into a composite so as to preserve their different natures. A particular case of these endpoints is the hierarchical composite endpoint (HCE). It is evaluated in a fixed follow-up period and accounts for the patient’s clinically most important outcome for the analysis. HCEs are analyzed using win odds and other win statistics [@gasparyanhierarchical].

## Examples

Here we provide examples of HCE using in clinical trials from different therapeutic areas. General considerations for creating HCEs can be found in @gasparyan2022design.

### COVID-19

The DARE-19 [@kosiborod2021effects;@kosiborod2021dapagliflozin] trial used an HCE to assess outcomes in patients hospitalized for COVID-19 and treated for 30 days. The COVID-19 HCE is presented below. It combines death, in hospital organ dysfunction events with clinical status at Day 30 for patients alive, still hospitalized but without previous organ dysfunction events, and hospital discharge as the most favorable outcome for patients discharging without organ dysfunction events and being alive at Day 30. 

Below a higher category signifies a better outcome. Patients are ranked into one and only one category based on their clinically most severe event. For example, patients experiencing an in-hospital new or worsening organ dysfunction event then dying will be included in the category `I`.

```{r echo=FALSE}
HCE <- data.frame(Order = c("I", "II", "III", "IV", "V"), 
                  Category = c("Death", 
                               "More than one new or worsened organ dysfunction events", 
                               "One new or worsened organ dysfunction event", 
                               "Hospitalized at the end of follow-up (Day 30)", 
                               "Discharged from hospital before Day 30")
)
HCE
```

Patients in the category `I` are compared using the timing of the event, with an earlier event being a worse outcome (are assigned a lower rank). Similarly, in the category `III` the timing of the event is used for ranking patients within this category. In the category `II` patients are compared using the number of events with a higher number signifying a worse outcome. Patients in the category `IV` - hospitalized at the end of follow-up without previous worsening events - are further ranked according to oxygen support requirements at the hospital (`IV.1` on high flow oxygen devices, `IV.2` requiring supplemental oxygen, `IV.3` not requiring supplemental oxygen, with a higher rank being a better outcome). Patients in the category `V` are compared using the timing of the event, but, the hospital discharge being a favorable outcome, here the earlier event signifies a better outcome than the late event (reverse of the ranking in categories `I` and `III`).

The simplest case of a COVID-19 HCE is an endpoint with ordinal scale outcomes assessed at a given timepoint. The endpoint uses 1-8 categories for assessing the physical limitations of hospitalized patients with COVID-19 after 15 or 30 days of treatment. But unlike the DARE-19 HCE, within each category it does not use the timing of events to reduce the ties in a paiwrise comparison of patients in the active group with patients in the control group. See, for example, `COVID-19` and `COVID-19b` datasest ordinal scale outcomes [@beigel2020remdesivir].

```{r}
table(COVID19)
```

The function `hce::summaryWO()` provides the number of wins, losses, and ties by categories. We can calculate the probability of ties from the provided numbers.

```{r}
COVID19HCE <- hce(GROUP = COVID19$GROUP, TRTP = COVID19$TRTP)
SUM <- summaryWO(COVID19HCE, ref = "Placebo")$summary
SUM$Ptie <- round(SUM$TIE/SUM$TOTAL, 2)
SUM
```

### Kidney HCE

The kidney HCE defined in @khce2 has the following ordinal outcomes (for the review of the topic see @khce1).


```{r echo=FALSE}
HCE2 <- data.frame(Order = c("I", "II", "III", "IV", "V", "VI", "VII"), 
                  Category = c("Death", 
                               "Dialysis or kidney transplantation", 
                               "Sustained GFR < 15 ml/min per 1.73 m2", 
                               "Sustained GFR decline from baseline of >= 57%", 
                               "Sustained GFR decline from baseline of >= 50%",
                               "Sustained GFR decline from baseline of >= 40%",
                               "Individual GFR slope")
)
HCE2
```

The dataset `KHCE` contains data on a kidney HCE outcomes

```{r}
dat <- KHCE
Order <- c("Death (adj)", "Chronic dialysis (adj) >=90 days", 
           "Sustained eGFR<15 (mL/min/1.73 m2)", "Sustained >=57% decline in eGFR", 
           "Sustained >=50% decline in eGFR", "Sustained >=40% decline in eGFR", "eGFR slope")   
dat$GROUP <- factor(dat$GROUP, levels = Order)
table(dat$GROUP, dat$TRTP)
```

This dataset is derived from `ADSL` which contains baseline characteristics, 

```{r echo=FALSE}
knitr::kable(head(ADSL), caption = "ADSL dataset structure\\label{adsl}")
```

`ADLB` laboratory measurements of kidney function, 

```{r echo=FALSE}
knitr::kable(head(ADLB), caption = "ADLB dataset structure\\label{adlb}")
```

and `ADET` for the time-to-event outcomes with their timing. 

```{r echo=FALSE}
knitr::kable(head(ADET), caption = "ADET dataset structure\\label{adet}")
```

For the detailed derivation see the [Technical Appendix](https://cdn-links.lww.com/permalink/jsn/e/jsn_2023_09_29_heerspink_1_sdc1.pdf) in @khce2.




### Heart Failure

In the Heart Failure population (see @kondo2023use) the following HCE was considered

```{r echo=FALSE}
HCE3 <- data.frame(Order = c("I", "II", "III", "IV"), 
                  Category = c("Cardiovascular death", 
                               "Total (first and recurrent) HF hospitalizations", 
                               "Total urgent HF visits", 
                               "Improvement/deterioration in KCCQ-TSS"))
HCE3
```


## References

---
title: "Introduction"
date: "`r format(as.Date('2026-06-30'), '%d %B, %Y')`"
author:
  - name: "Samvel B. Gasparyan"
    affiliation: https://gasparyan.co/
output:
  rmarkdown::html_document:
    theme: "darkly"
    highlight: "zenburn"
    toc: true
    toc_float: true
link-citations: true
bibliography: REFERENCES.bib
biblio-style: science
vignette: >
  %\VignetteIndexEntry{Introduction}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

<style>
a[role="doc-biblioref"] {
  color: green !important;
}

a[role="doc-biblioref"]:visited {
  color: green !important;
}

a[role="doc-biblioref"]:hover {
  color: lightgreen !important;
  text-decoration: underline;
}
</style>

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
R <- function() knitr::include_graphics("Rlogo.png", dpi = 5000)
```

## `hce` package intro

### Background

```{r echo=FALSE, out.width = '33%'}
knitr::include_graphics("hex-hce.png")
```


The package supports the simulation and analysis of **univariate hierarchical composite endpoints (HCEs)**. The rationale for constructing a univariate endpoint, in contrast to the multivariate version of Generalized Pairwise Comparisons, which typically yields summary statistics rather than a single endpoint, is described in Gasparyan, Koch, and Brunner [@gasparyan2026univariate]. The framework is broad: univariate HCEs can be derived from different outcome types, including time-to-event, continuous, and binary outcomes. However, it applies only to fixed follow-up designs without early dropout.

In this setting, several approaches can be used to derive a patient-level ordinal outcome, which is the essence of a univariate HCE. Gasparyan, Koch, and Brunner [@gasparyan2026univariate] derived the distribution for the setting with multiple time-to-event outcomes and a single continuous (or ordinal) outcome assessed at the end of follow-up; in that construction, the event times also contribute to the HCE. Accordingly, our definition of an HCE focuses on this setting. Although the theoretical distribution is not required for estimation, understanding the distribution remains important for interpreting summary statistics in greater detail and for avoiding the **Condorcet non-transitivity paradox** [@de2014essai].

These endpoints have been implemented in clinical trials across multiple therapeutic areas. For example, see @gasparyan2022design for an implementation in a COVID-19 setting and for practical considerations in constructing hierarchical composite endpoints. For chronic kidney disease (**CKD**) applications, see [@khce1; @khce2; @khce3].

The primary analysis method is **win odds**. Other win statistics, including **win ratio** and **net benefit** [@dong2023win], are also implemented when there is no censoring. Win odds relies on the **DeLong-DeLong-Clarke-Pearson** formula [@delong1988comparing] for the variance of the win proportion and is based on the **Brunner-Munzel** test [@brunner2000nonparametric]. We also include the **Brunner-Konietschke** version [@brunner2025unbiased] of the **Bamber** estimator [@bamber1975area] for the variance of the win proportion. In addition, the package provides **Wilson-type, range-preserving confidence intervals** for the win proportion and win odds, following Schüürhuis, Konietschke, and Brunner [@schuurhuis2025new].

The power and sample size formulas cover several classes of alternatives: `"shifted"`, `"ordered"`, and `"max"` for the maximum value of the standard deviation. All formulas are derived from Bamber [@bamber1975area]. By default, **Noether's formula** [@noether1987sample] is used for shifted distributions. Additional discussion of power calculations for shifted distributions is available in [@gasparyan2021power; @gasparyan2022comments].

For reviews of HCE design in clinical trials, see [@gasparyan2022design], [@gasparyanhierarchical], and [@khce1]. The **Basic Data Structure (BDS)**, which follows **Analysis Data Model (ADaM)** [@CDISC] principles for hierarchical composite endpoints, is described in @gasp2024bds. For visualization, the **maraca plot** [@karpefors2023maraca] can be used.

Stratified and adjusted win odds are calculated using **randomization-based covariate adjustment theory** [@koch1998issues]; for a review, see [@gasparyan2021adjusted].

Thresholds for pairwise comparisons involving the continuous component of the HCE are implemented according to the theory in Gasparyan, Koch, and Brunner [@gasparyan2026univariate].

All implementations are **rank-based**. For a methodological review, see [@brunner2018rank].


### Setup

Load the package `hce` and check the version

```{r eval = TRUE}
library(hce)
packageVersion("hce")
```

For citing the package, run `citation("hce")` [@hce].

### Contents

List the functions and the datasets in the package

```{r}
ls("package:hce")
```

The package contains the following **Datasets**: use `data(package = "hce")` for the list of all datasets included in the package.

```{r eval=FALSE}
HCE1; HCE2; HCE3; HCE4
COVID19; COVID19b; COVID19plus
ADET; ADLB; ADSL; KHCE
```

- **Simulated datasets**: `HCE1 - HCE4` contain two treatment groups and analysis values `AVAL` for a hierarchical composite endpoint.

- **COVID-19 datasets**: `COVID19`, `COVID19b`, and `COVID19plus` contain COVID-19 ordinal scale outcomes [@beigel2020remdesivir; @kalil2021baricitinib].

- **Synthetic kidney disease datasets**: `ADET` contains event-time data, `ADLB` contains laboratory data, and `ADSL` contains subject-level baseline characteristics. Based on these data, `KHCE` provides the derived kidney hierarchical composite endpoint for the same patients [@khce2].

## In Brief

### Calculations

```{r eval=FALSE}
calcWO()
calcWINS()
summaryWO()
regWO()
stratWO()
IWP()
```

The main functions for calculating the so-called win statistics—**win probability**, **win odds**, **win ratio**, and **net benefit**—are `calcWO()` for win odds and `calcWINS()` for all win statistics, including win odds. To summarize wins, losses, and ties, use `summaryWO()`, which also reports the win probability, win odds, and win ratio, together with the standard errors of the win probability and win odds. By default, all of these functions use the **DeLong-DeLong-Clarke-Pearson** formula [@delong1988comparing] to estimate the standard error of the win probability and win odds.

In `calcWINS()`, the `SE_WP_Type` argument can be set to `unbiased` to use the **Brunner-Konietschke** version [@brunner2025unbiased] of the **Bamber** estimator [@bamber1975area] for the variance of the win proportion, or to `biased` to use the **DeLong-DeLong-Clarke-Pearson** variance estimator, which is the same method used in `calcWO()` and may be biased in small samples. In the same function, the `CI_WP_Type` argument can be set to `Wilson` to obtain **Wilson-type, range-preserving confidence intervals** for the win proportion and win odds, following Schüürhuis, Konietschke, and Brunner [@schuurhuis2025new]. The function `calcWINS()` also implements **Goodman-Kruskal's gamma** [@goodman1954measures; @goodman1963measures]. All reported statistics include confidence intervals.

The functions `regWO()` and `stratWO()` compute adjusted and stratified win odds, respectively, based on the **randomization-based covariate adjustment theory** developed in Koch et al. [@koch1998issues]; for a review, see Gasparyan et al. [@gasparyan2021adjusted].

In principle, all of these functions operate on ordinal outcomes when each patient contributes a single analysis value. Therefore, the input data should be at the patient level. Any patient-level dataset with a numeric analysis value can be used to calculate these statistics. In this setting, the use of win statistics is equivalent to ordinal analysis. Specifically, win odds corresponds to the classical **Mann-Whitney odds**, win ratio is the number of concordances divided by the number of discordances and can be derived from **Goodman-Kruskal's gamma**, and net benefit corresponds to **Somers' D C/R**. Therefore, any software that computes these statistics should produce the same results. For example, although the implementation here is fully rank-based, the estimate of **Goodman-Kruskal's gamma** and its confidence interval agree with `DescTools::GoodmanKruskalGamma()` [@Desc].

All calculations are rank-based and therefore computationally efficient. The basic building blocks are the **individual win proportions**, defined for each patient as the proportion of wins plus one-half of the proportion of ties. The function `IWP()` computes these individual win proportions for each patient in the dataset and returns an updated dataset with a new column. Any win-odds-related calculation can be based on these quantities [@gasparyan2021adjusted].

### Univariate hierarchical composite endpoints

```{r eval=FALSE}
hce()
as_hce()
```


The idea behind **univariate hierarchical composite endpoints** [@gasparyan2026univariate] is to derive a single ordinal outcome from multiple outcomes of different types, such as time-to-event, continuous, and binary outcomes, by prioritizing the most clinically important outcome for each patient. The resulting ordinal outcome can then be used to calculate win statistics.

To support this, the package provides helper functions for deriving `hce` objects from different types of outcomes. In particular, `hce()` and `as_hce()` check the structure of the input data and attempt to convert it to an `hce` object. Once this conversion is complete, that is, once the univariate HCE has been derived, the subsequent theory and analysis are the same as for any other ordinal outcome.

### Thresholds

```{r eval=FALSE}
deltaWO()
```


The function `deltaWO()` calculates threshold-adjusted win odds [@gasparyan2026univariate]. 

### Simulations

```{r eval=FALSE}
simORD()
simHCE()
simTTE()
simKHCE()
```


### Power and sample size


```{r eval=FALSE}
powerWO()
sizeWO()
sizeWR()
minWO()
propWINS()
```

The functions `powerWO()`, `sizeWO()`, and `minWO()` provide tools for calculating **power**, **sample size**, and the **minimum detectable treatment effect** for win odds under different classes of alternatives: `"shifted"`, `"ordered"`, and `"max"`, where `"max"` refers to the maximum value of the standard deviation. All formulas are based on Bamber [@bamber1975area]. For shifted distributions, the default is **Noether's formula** [@noether1987sample]. See also [@gasparyan2021power; @gasparyan2022comments] for further discussion of shifted distributions. The function `sizeWR()` provides sample size calculations for the **win ratio** [@yu2022sample].

The function `propWINS()` is a convenient tool for deriving the proportions of **wins**, **losses**, and **ties** for each treatment group from the win odds and win ratio.

### Ordinal dominance graph and the maraca plot

```{r eval=FALSE}
plot.hce()
```

A plot method for `hce` objects (created by `as_hce()`) to provide the ordinal dominance graph [@bamber1975area].

### Extras

```{r eval=FALSE}
rweibullGF()
dGLL()
pGLL()
qGLL()
rGLL()
hGLL()
HGLL()
```

The **generalized log-logistic (GLL)** distributions arise as the marginal distributions of the **Weibull-Gamma frailty model**. For comparison, one can use `rweibullGF()`, which implements the random frailty formulation, and compare its output with the generalized log-logistic distribution functions: density `dGLL()`, distribution function `pGLL()`, quantile function `qGLL()`, random generation `rGLL()`, hazard function `hGLL()`, and cumulative hazard function `HGLL()`.

These distributions form the basis for constructing two-outcome hierarchical composite endpoints with death and hospitalization in the generalized illness-death model implemented in `simTTE()`.

### Helpers

```{r eval=FALSE}
print.hce_results()
plot.hce_results()
as_formulae.formula()
```


To facilitate visualization and review of sample size and power calculations, `print()` and `plot()` methods are implemented for `hce_results` objects generated by the functions `powerWO()`, `sizeWO()`, and `minWO()`.

The function `as_formulae()` is an internal helper that decomposes formula objects into treatment, response, covariate, and grouping variables using the standardized names `TRTP`, `AVAL`, `COVAR`, and `GROUP`, respectively.

## `hce` Objects


### `hce()` Function

`hce` objects can be constructed using the helper function `hce()`, which has the following arguments:

```{r}
args("hce")
```
 
We see that the required arguments are `GROUP`, which specifies the clinically most important event of a patient to be included in the analysis, and `TRTP`, which specifies the (planned) treatment group of a patient (exactly two treatment groups should be present). Note that:

- The `hce` structure assumes that only one event per patient is present for the analysis, meaning that the resulting `hce` object created by the `hce()` function is a patient-level dataset. The function `hce()` does not select the clinically most important event of the patient but requires it to be already done when calling it.

- The argument `TRTP` should have exactly two levels.

Consider the following example of ordinal outcomes 'I', 'II', and 'III':

```{r}
set.seed(2022)
n <- 100
dat <- hce(GROUP = rep(x = c("I", "II", "III"), each = 100), 
           TRTP = sample(x = c("Active", "Control"), size = n*3, replace = TRUE))
class(dat)
```

This dataset has the appropriate structure of `hce` objects, but its class inherits from an object of class `data.frame`. This means that all functions available for data frames can be applied to `hce` objects, for example, the function `head()`:

```{r}
head(dat)
```

We see that the dataset has a very specific structure. The column `PARAMN` shows how the function `hce()` generated the order of given events (it uses usual alphabetic order for the unique values in the `GROUP` column to determine the clinical importance of events):

```{r}
unique(dat[, c("GROUP", "PARAMN")])
```

In the class `hce`, higher values for the ordering mean clinically less important events. For example, death, which is the most important event, should always get the lowest ordinal value. If there is a need to specify the order of outcomes, then the `GROUP` argument can be provided as a factor with the levels specifying the necessary order:

```{r}
set.seed(2022)
n <- 100
GROUP = rep(x = c("I", "II", "III"), each = 100) 
GROUP <- factor(GROUP, levels = c("III", "II", "I"))
dat <- hce(GROUP = GROUP, 
           TRTP = sample(x = c("Active", "Control"), size = n*3, replace = TRUE))
unique(dat[, c("GROUP", "PARAMN")])
```

This means that the clinically most important event is 'III' instead of 'I'. The argument `AVAL0` is meant to help in cases where we want to introduce sub-ordering within each `GROUP` category. For example, if two events in the group 'I' can be compared based on other parameters, then the `AVAL0` argument can be specified to take that into account.

Below we use the built-in data frame `HCE1` to construct an `hce` object. Before specifying the order of events, it is a good idea to check what are the unique events included in the `GROUP` column:


```{r}
data(HCE1)
unique(HCE1$GROUP)
```

Therefore, we can construct the following object using the `hce()` function:

```{r}
HCE <- hce(GROUP = factor(HCE1$GROUP, levels = c("TTE1", "TTE2", "TTE3", "TTE4", "C")), 
           TRTP = HCE1$TRTP, AVAL0 = HCE1$AVAL0, PADY = 1080)
class(HCE)
head(HCE)
```

The object's class is `adhce`, which inherits from `hce`. This design indicates that the object includes additional columns.

### Create an `hce` Object from a Data Frame

Consider the dataset `HCE1`, which is part of the package `hce`:

```{r}
data(HCE1, package = "hce")
class(HCE1)
head(HCE1)
```

This dataset has the appropriate structure of `hce` objects, but its class is `data.frame.` A generic way of coercing data structures to an `hce` object is to use the function `as_hce()`. This function performs checks (using an internal validator function) and creates an `hce` object from the given data structure (using an internal constructor function). If coercion is not possible, it will throw an error explaining the issue.

```{r}
dat1 <- as_hce(HCE2)
str(dat1)
```


### Simulate `hce` Objects Using `simHCE()`

To simulate values from a hierarchical composite endpoint, we use the function `simHCE()`, which has the following arguments:

```{r}
args("simHCE")
```

- The vector arguments `TTE_A` and `TTE_P` specify the event rates per year for time-to-event outcomes in the active and control groups, respectively. The function assumes a Weibull survival function with the same shape parameter for simulating all time-to-event outcomes in both treatment groups (by default `shape = 1`, which assumes an exponential survival function). These two vectors should have the same length, which indicates the number of time-to-event outcomes. 

- By default, the event rates are presented per 100 patient-years (`pat = 100`), which can be changed using the argument `pat`. The function simulates event times in days, and the `yeardays = 360` argument can be used to change the number of days in a year (e.g., 365 or 365.25).

- The function simulates events during a fixed follow-up period only, and the `fixedfy` argument can be used to change the length of the follow-up (in years).

- The function simulates the continuous outcome from a normal (default) or log-normal (if `logC = TRUE`) distribution with given means and standard deviations for two treatment groups.

```{r}
Rates_A <- c(1.72, 1.74, 0.58, 1.5, 1) 
Rates_P <- c(2.47, 2.24, 2.9, 4, 6) 
dat3 <- simHCE(n = 2500, n0 = 1500, TTE_A = Rates_A, 
               TTE_P = Rates_P, 
               CM_A = -3, CM_P = -6, 
               CSD_A = 16, CSD_P = 15, 
               fixedfy = 3, seed = 2023)
```

```{r}
class(dat3)
head(dat3)
```




## Generics for `hce` Objects

As we see, the function `simHCE()` creates an object of type `hce`, which inherits from the built-in class `data.frame`. We can check all implemented methods for this new class as follows:
```{r}
methods(class = "hce")
```

The function `calcWO()` calculates the win odds and its confidence interval, while `summaryWO()` provides a more detailed calculation of win odds, including the number of wins, losses, and ties by `GROUP` categories.

```{r}
HCE <- hce(GROUP = factor(HCE3$GROUP, levels = c("TTE1", "TTE2", "TTE3", "TTE4", "C")), 
           TRTP = HCE3$TRTP, AVAL0 = HCE3$AVAL0, PADY = 1080)
calcWO(HCE)
calcWINS(HCE)  
HCE$TRTP <- factor(HCE$TRTP, levels = c("P", "A"))
plot(HCE, fill = TRUE, col = "#865A4F", type = 'l', lwd = 2)
abline(a = 0, b = 1, lwd = 2, col = "#999999", lty = 2)
```

To check the generic functions available for the `adhce` class by running the following command:

```{r}
methods(class = "adhce")
```

The main difference between `summaryWO.hce()` and `summaryWO.adhce()` is that `summaryWO.adhce()` also provides a group-wise summary, in addition to the overall summary:

```{r}
summaryWO(HCE)
```

An important addition to `summaryWO.adhce()` is the `cumsummary_by_GROUP` output, which reports cumulative wins, losses, and ties as outcomes are added sequentially according to the priority order:

```{r}
res0 <- summaryWO(HCE, ref = "P")
res <- res0$cumsummary_by_GROUP
barplot(PROP ~ WINS + GROUPN, data = res, 
col = c("darkgreen", "darkred", "darkblue"), 
xlab = "Proportions", xlim = c(0, 1), 
ylab = "Cumulative components by prioritization", 
legend.text = unique(res$WINS), beside = TRUE, horiz = TRUE)
grid()
```


## References

---
title: "Visualization of HCE using maraca plots"
date: "`r format(as.Date('2024-10-15'), '%d %B, %Y')`"
author:
  - name: "Samvel B. Gasparyan"
    affiliation: https://gasparyan.co/
output:
  rmarkdown::html_document:
          theme: "darkly"
          highlight: "zenburn"
          toc: true
          toc_float: true
          link-citations: true 
bibliography: REFERENCES.bib
vignette: >
  %\VignetteIndexEntry{maraca}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

## Visualization


### Setup

```{r echo=FALSE, out.width = '33%'}
knitr::include_graphics("hex-hce.png")
```

Load the package `hce` and check the version:

```{r setup}
library(hce)
packageVersion("hce")
```

For citing the package, run `citation("hce")` [@hce].

### The Maraca Plot

The *maraca* plot (named for its visual similarity to the musical instrument) has been recently introduced [@karpefors2023maraca] for the visualization of HCEs, which combine multiple dichotomous outcomes with a single continuous endpoint. The maraca plot visualizes the contribution of components of a hierarchical composite endpoint (HCE) over time. It is formed by adjoining, from left to right, uniformly scaled Kaplan–Meier plots of times to each dichotomous outcome among those without more severe outcomes, with a superimposed box/violin plot of the continuous outcome.

The maraca plot is implemented in the `maraca` package [@maraca], which depends on the `hce` package. The `maraca` package includes a `plot.hce()` method to visualize objects of type `hce`. Consider the following example:

```{r eval=FALSE}
library(maraca)
Rates_A <- 10
Rates_P <- 15
dat <- simHCE(n = 1000, n0 = 500, TTE_A = Rates_A, TTE_P = Rates_P, 
              CM_A = 0.2, CM_P = 0, seed = 2, shape = 0.35)
plot(dat)
```

```{r echo=FALSE}
knitr::include_graphics("maraca.png", dpi = 100)
```

The example illustrates a maraca plot with a single dishotomous outcome combined with a continuous outcome. The dischotmous outcomes over time are simulated from a Weibull distribution with `shape = 0.35` in both treatment groups. The `rate` parameter in the active group is 10 per 100 patients per year, and 15 in the control group (100 patients per year is the default value). 

## References

---
title: "Win statistics"
date: "`r format(as.Date('2026-03-15'), '%d %B, %Y')`"
author:
  - name: "Samvel B. Gasparyan"
    affiliation: https://gasparyan.co/
output:
  rmarkdown::html_document:
          theme: "darkly"
          highlight: "zenburn"
          toc: true
          toc_float: true
          link-citations: true 
bibliography: REFERENCES.bib
vignette: >
  %\VignetteIndexEntry{Wins}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

## Win Odds, Win Ratio, and Net Benefit


### Setup

```{r echo=FALSE, out.width = '33%'}
knitr::include_graphics("hex-hce.png")
```

Load the package `hce` and check the version:

```{r setup}
library(hce)
packageVersion("hce")
```

For citing the package, run `citation("hce")` [@hce].

### Terminology

The concept of win probability for binary and continuous outcomes has been described by @buyse2010generalized as the "proportion in favor of treatment" (see also @rauch2014opportunities), while in @verbeeck2020unbiasedness refers to it as the "probabilistic index".

The concept of "win ratio" was introduced by @poc2012. Unlike the win odds, the win ratio does not account for ties. The win odds, however, is the odds of winning, as described by @dong2020 (see also @peng2020use; @brunner2021win; @gasparyan2021power). The same statistic was named as the Mann-Whitney odds by @obrien2006. In @gasparyan2021adjusted, the "win ratio" was used as a general term and included ties in the definition. @dong2022winwinnet suggested considering win ratio, win odds, and net benefit together as win statistics.

The concept of winning for the active group is the same as concordance for the active group. For two variables, $X$ and $Y,$ a pair is concordant if the observation with the larger value of $X$ also has the better value of $Y$ [@agresti2013categorical]. If $X$ indicates the treatment group with the value 1 for the active group and 0 for the control group, and $Y$ is the ordinal value for the analysis, then concordance means that a patient in the active treatment group has a better value than a patient in the control group. Discordance means the patient in the active group has a worse value than the control patient. Therefore, the win ratio is the total number of concordances divided by the total number of discordances. The win ratio can be obtained from the Goodman-Kruskal gamma [@kruskal1954measures], $G$, as follows: 

$$WR=(1+G)/(1-G).$$

The net benefit is  Somers' D C/R [@somers1962new], while the win odds is the Mann-Whitney odds [@mann1947test]. Estimation of win statistics in the absence of censoring can be done using the theory of *U*-statistics [@Hoeff].

### Definitions

Two treatment groups are compared using an ordinal endpoint, and each comparison results in a win, loss, or tie for the patient in the active group compared to a patient in the placebo group. All possible (overall) combinations are denoted by $O$, with $W$ denoting the total wins for the active group, $L$ the total losses, and $T$ the total ties, so that $O=W+L+T.$ The following quantities are called **win statistics**:

- **Win Probability** defined as $WP=\frac{W+0.5T}{O}$, which is the total number of wins, adding half of the total number of ties, divided by the overall number of comparisons.

- **Number Needed to Treat** defined as $NNT=\frac{1}{2WP-1}=\frac{O}{W-L}$ (rounded up to the nearest natural number for interpretation).

- **Win Ratio** defined as $WR=\frac{W}{L}$.

- **Win Odds** defined as $WO=\frac{W+0.5T}{L+0.5T} = \frac{WP}{1-WP}$.

- **Net Benefit** defined as $NB=\frac{W - L}{O}=2WP-1=\frac{1}{NNT}$.

Given the overall number of comparisons $O,$ the win proportion $WP$, and the win ratio $WR$, it is possible to find the total number of wins and losses: 
\begin{align*}
&L = O*\frac{2WP-1}{WR-1},\nonumber\\
&W = WR*L = WR*O*\frac{2WP-1}{WR-1},\nonumber\\
&T=O-W-L = O*\left[1 - (WR+1)\frac{2WP-1}{WR-1}\right].
\end{align*}

To better understand the relationship between the win ratio and the win odds, observe that:
$$\frac{W + \frac{W}{W+L}T}{L + \frac{L}{W+L}T}=\frac{W}{L}\frac{1 + \frac{T}{W+L}}{1 + \frac{T}{W+L}}=\frac{W}{L}\frac{W + L + T}{W + L + T}=\frac{W}{L}=WR.$$
Therefore, the win ratio essentially allocates tied outcomes proportionally: it assigns a fraction of ties to wins and losses based on the observed proportion of wins to losses, $\frac{W}{W+L}.$ This means that ties are distributed between the treatment groups in the same ratio as the actual wins and losses. In contrast, the win odds splits ties evenly between the active and control groups, making it a more conservative measure of treatment effect.


The function `propWINS()` implements the formula above:

```{r}
args("propWINS")
propWINS(WO = 1.5, WR = 2)
```

Suppose there are $n_1=120$ patients in the placebo group and $n_2=150$ in the active group. If the win ratio is 1.5 and the win odds is 1.25, then the number of wins and losses for the active group can be calculated using the `Overall` argument, which represents all possible comparisons.

```{r}
propWINS(WO = 1.25, WR = 1.5, Overall = 120*150)
```


This function call will provide the number of wins, losses, and ties for the active group based on the specified win odds and win ratio.

## References

---
title: "Hierarchical composite endpoints"
date: "`r format(as.Date('2026-03-15'), '%d %B, %Y')`"
author:
  - name: "Samvel B. Gasparyan"
    affiliation: https://gasparyan.co/
output:
  rmarkdown::html_document:
          theme: "darkly"
          highlight: "zenburn"
          toc: true
          toc_float: true
          link-citations: true 
bibliography: REFERENCES.bib
vignette: >
  %\VignetteIndexEntry{Hce}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

## Introduction


### Setup

```{r echo=FALSE, out.width = '33%'}
knitr::include_graphics("hex-hce.png")
```

Load the package `hce` and check the version:

```{r setup}
library(hce)
packageVersion("hce")
```

For citing the package run `citation("hce")` [@hce].

### Definitions

Hierarchical composite endpoints (HCE) are a class of endpoints that combine multiple clinical outcomes into a single composite while preserving the distinct nature of each outcome. In a particular case, outcomes within a fixed follow-up period are ranked by clinical importance, and the patient’s most important outcome is used for analysis. HCEs are typically analyzed using win odds and related win statistics [@gasparyanhierarchical].

A straightforward example of an HCE is an endpoint defined on an ordinal scale assessed at a specific timepoint. For instance, the COVID-19 HCE uses an 8-category ordinal scale to evaluate physical limitations in hospitalized patients with COVID-19 at 15 or 30 days post-treatment. See the `COVID-19` and `COVID-19b` dataset ordinal outcomes as examples [@beigel2020remdesivir].

```{r}
table(COVID19)
```
Simple HCEs are implemented in the `hce` package via the `hce` class, which inherits from `data.frame`. These objects must include the ordinal analysis values in the `AVAL` column and a two-level treatment group in the `TRTP` column. The `hce::summaryWO()` function can be used to provide the number of wins, losses, and ties by category. From these counts, one can compute the probability of ties.

```{r}
COVID19HCE <- hce(GROUP = COVID19$GROUP, TRTP = COVID19$TRTP)
SUM0 <- summaryWO(COVID19HCE, ref = "Placebo")
SUM <- SUM0$summary
SUM$Ptie <- round(SUM$TIE/SUM$TOTAL, 2)
SUM
```

While the win odds and its 95% confidence interval are calculated as follows:

```{r}
WO <- calcWO(COVID19HCE, ref = "Placebo")[, c("WO", "LCL", "UCL")]
round(WO, 2)
```

A more complex case arises when multiple events per patient are observed. Consider two binary outcomes: death and hospitalization. Each patient can experience up to two events (e.g., hospitalization followed by death), and the HCE must account for the hierarchy between them (typically prioritizing death over hospitalization) when determining wins, losses, and ties.

```{r}
set.seed(1)
n <- 100
dat0 <- data.frame(TRTP = rep(c("A", "P"), each = n), 
                  DEATH = c(rbinom(n = n, size = 1, prob = 0.6), 
                            rbinom(n = n, size = 1, prob = 0.8)),
                  HOSP = c(rbinom(n = n, size = 1, prob = 0.5), 
                            rbinom(n = n, size = 1, prob = 0.8)))
```

Based on these outcomes, several methods can be used to construct hierarchical composite endpoints. Below are two approaches with their implementations.

1. The most-important outcome approach

Select a single outcome per patient—the highest-priority one. If a patient experienced death, select death (regardless of hospitalization). If no death occurred, select hospitalization (only if it did not lead to subsequent death). Otherwise, select no outcome. This yields three categories: I. Death, II. Hospitalization (without subsequent death), III. No outcome.

This can be implemented as:

```{r}
dat1 <- dat0
dat1$AVAL <- ifelse(dat1$DEATH == 1, 1, ifelse(dat1$HOSP == 1, 2, 3))
dat1 <- as_hce(dat1)
summaryWO(dat1)$summary
```

2. The move-down approach (see @poc2012)

Compare patients first on death. The active (control) patient wins if they are alive while the patient in the control (active) group died. If both patients died, it is a tie on death and the comparison proceeds to the next outcome (hospitalization), using the same winner logic. If no winner is determined after evaluating hospitalization, declare a tie (there are no further outcomes to break ties).

In this case, we can also derive a single patient-level categorical variable that encodes the above comparison logic. From worst to best, the categories are: patients with both hospitalization and death; patients with death without prior hospitalization; patients with hospitalization without death; and patients with no events. This can be implemented using the following ordering:


```{r}
dat2 <- dat0
dat2$ORD <- 2*dat2$DEATH + dat2$HOSP
unique(dat2[, c("DEATH", "HOSP", "ORD")])
```

The multiplicative factor of 2 is used purely for ordering and can be replaced by any monotone transformation. Here, a higher `ORD` indicates a more prioritized (worse) outcome—i.e., outcomes that would cause the patient to lose in a pairwise comparison. Because the `hce` package convention is that higher values denote better outcomes, we simply reverse the ordering in `ORD` to obtain the analysis variable `AVAL`:

```{r}
dat2$AVAL <- max(dat2$ORD) - dat2$ORD 
dat2 <- as_hce(dat2)
unique(dat2[, c("DEATH", "HOSP", "ORD", "AVAL")])
summaryWO(dat2)$summary
```

The number of ties is more than halved, leading to a larger estimated treatment effect. This occurs because, in this case, the second method more efficiently breaks ties by sequentially comparing outcomes (death first, then hospitalization), reducing indeterminate pairings. 


## Examples

Here we provide examples of HCE using in clinical trials from different therapeutic areas. General considerations for creating HCEs can be found in @gasparyan2022design, @khce1.


### Kidney HCE

The kidney HCE defined in @khce2 has the following ordinal outcomes (for the review of the topic see @khce1).


```{r echo=FALSE}
HCE2 <- data.frame(Order = c("I", "II", "III", "IV", "V", "VI", "VII"), 
                  Category = c("Death", 
                               "Dialysis or kidney transplantation", 
                               "Sustained GFR < 15 ml/min per 1.73 m2", 
                               "Sustained GFR decline from baseline of >= 57%", 
                               "Sustained GFR decline from baseline of >= 50%",
                               "Sustained GFR decline from baseline of >= 40%",
                               "Individual GFR slope")
)
HCE2
```

The dataset `KHCE` contains data on a kidney HCE outcomes

```{r}
dat <- KHCE
Order <- c("Death (adj)", "Chronic dialysis (adj) >=90 days", 
           "Sustained eGFR<15 (mL/min/1.73 m2)", "Sustained >=57% decline in eGFR", 
           "Sustained >=50% decline in eGFR", "Sustained >=40% decline in eGFR", "eGFR slope")   
dat$GROUP <- factor(dat$GROUP, levels = Order)
table(dat$GROUP, dat$TRTP)
```

This dataset is derived from `ADSL` which contains baseline characteristics, `ADLB` laboratory measurements of kidney function, and `ADET` for the time-to-event outcomes with their timing. For the detailed derivation see the [Technical Appendix](https://cdn-links.lww.com/permalink/jsn/e/jsn_2023_09_29_heerspink_1_sdc1.pdf) in @khce2.


### COVID-19

The DARE-19 [@kosiborod2021effects;@kosiborod2021dapagliflozin] trial used an HCE to assess outcomes in patients hospitalized for COVID-19 and treated for 30 days. The COVID-19 HCE is presented below. It combines death, in hospital organ dysfunction events with clinical status at Day 30 for patients alive, still hospitalized but without previous organ dysfunction events, and hospital discharge as the most favorable outcome for patients discharging without organ dysfunction events and being alive at Day 30. 

Below a higher category signifies a better outcome. Patients are ranked into one and only one category based on their clinically most severe event. For example, patients experiencing an in-hospital new or worsening organ dysfunction event then dying will be included in the category `I`.

```{r echo=FALSE}
HCE <- data.frame(Order = c("I", "II", "III", "IV", "V"), 
                  Category = c("Death", 
                               "More than one new or worsened organ dysfunction events", 
                               "One new or worsened organ dysfunction event", 
                               "Hospitalized at the end of follow-up (Day 30)", 
                               "Discharged from hospital before Day 30")
)
HCE
```

Patients in the category `I` are compared using the timing of the event, with an earlier event being a worse outcome (are assigned a lower rank). Similarly, in the category `III` the timing of the event is used for ranking patients within this category. In the category `II` patients are compared using the number of events with a higher number signifying a worse outcome. Patients in the category `IV` - hospitalized at the end of follow-up without previous worsening events - are further ranked according to oxygen support requirements at the hospital (`IV.1` on high flow oxygen devices, `IV.2` requiring supplemental oxygen, `IV.3` not requiring supplemental oxygen, with a higher rank being a better outcome). Patients in the category `V` are compared using the timing of the event, but, the hospital discharge being a favorable outcome, here the earlier event signifies a better outcome than the late event (reverse of the ranking in categories `I` and `III`).

### Heart Failure

In the Heart Failure population (see @kondo2023use) the following HCE was considered

```{r echo=FALSE}
HCE3 <- data.frame(Order = c("I", "II", "III", "IV"), 
                  Category = c("Cardiovascular death", 
                               "Total (first and recurrent) HF hospitalizations", 
                               "Total urgent HF visits", 
                               "Improvement/deterioration in KCCQ-TSS"))
HCE3
```

## Simulations

### The kidney HCE

This section describes the function `simKHCE(),` which simulates a kidney HCE composed of six components, evaluated in the following order: 

1. Kidney Failure Replacement Therapy (KFRT) 

1. Sustained eGFR < 15 

1. Sustained $\geq 57\%$ decline in eGFR 

1. Sustained $\geq 50\%$ decline in eGFR

1. Sustained $\geq 40\%$ decline in eGFR

1. Change in eGFR

```{r eval=FALSE}
simKHCE(
  n,
  CM_A,
  CM_P = -4,
  n0 = n,
  TTE_A = 1000,
  TTE_P = TTE_A,
  fixedfy = 2,
  Emin = 20,
  Emax = 100,
  sigma = NULL,
  Sigma = 3,
  m = 10,
  theta = -0.4605,
  phi = 0,
  two_meas = c("no", "base", "postbase", "both")
)
```



The simulation assumes a fixed follow-up interval $[0, T]$, where $T$ (in years) is set by `fixedfy` (fixed follow-up years). Each patient has $m$ equally spaced visits over this interval. The active arm has sample size $n;$ the control arm has size $n_0$. Measurements are generated from a linear mixed-effects model with a independent random intercepts and slopes:
$$y_{ij}=\alpha_i+\{\beta_i + [b^1 - (\beta_i)^-\varphi]x_i\}t_j+\varepsilon_{ij}, \ \ t_j=\frac{T}{m}j,\,j=0,1,\cdots,m,\,i=1,\cdots,N=n_0+n.$$

Here, $x_i$ is the active-arm indicator (1 for active, 0 for control). 

- **Baseline eGFR:** True baselines are sampled as $\alpha_i\sim {\mathcal U}[E_{\min}, E_{\max}].$ 

- **Random slope:** For the placebo group, $\beta_i\sim {\mathcal N}(b^0, \Sigma^2),$ where the mean placebo slope $b^0$ is given by `CM_P.` The negative part is $(\beta_i)^-=\max(-\beta_i, 0).$ For active-arm patients with a negative slope, treatment combines multiplicative and additive effects:
$$\beta_i + [b^1 - (\beta_i)^-\varphi]x_i=\beta_i (1-\varphi)+b^1.$$
For patients with a positive slope, the treatment effect is purely additive:

$$\beta_i + [b^1 - (\beta_i)^-\varphi]x_i=\beta_i +b^1.$$
The additive effect $b^1$ is specified in `CM_A,` and the proportionality coefficient $\varphi\in[0,1]$ is set by `phi.` The between-patient slope variability is `Sigma` (i.e., $\Sigma$). 

- **Residual variability:** Measurement error follows $\varepsilon_{ij}\sim {\mathcal N}(0, \sigma_{ij}^2).$ If `sigma` is provided, it sets the within-patient standard deviation (SD). By default (`sigma = NULL`), the SD is time-varying and depends on the predicted eGFR:

$$\sigma_{ij}=\sqrt{0.67*\textit{predicted eGFR}}=\sqrt{0.67(\alpha_i+\{\beta_i + [b^1 - (\beta_i)^-\varphi]x_i\}t_j)}.$$

- **KFRT events:** KFRT times are simulated using a patient-specific piecewise-constant hazard on each $[t_{j-1}, t_j],$ with
$$\lambda_{ij}=e^{c_0+c_1x_i}e^{\theta(\alpha_i+\{\beta_i + [b^1 - (\beta_i)^-\varphi]x_i\}t_j)}.$$

Let $dt=t_j-t_{j-1}=\frac{T}{m}.$ The cumulative hazard is
$$\Lambda_i(t)=\sum_{j=1}^{k-1}dt \lambda_{ij}+\lambda_{ik}(t - t_{k-1}), \ \ t\in[t_{k-1}, t_k].$$
If $U_i\sim{\mathcal U}[0, 1]$ drawn per patient, the event time $X_i$ satisfies $S_i(X_i)=\exp(-\Lambda_i(X_i))=U_i,$ yielding,
$$X_i = \frac{-\log(U_i)-\sum_{j=1}^{k-1}dt \lambda_{ij}}{\lambda_{ik}}+t_{k-1}.$$
Here $k$ is the minimum for which $X_i\leq t_k,$ or, equivalently, $k$ is the minimum for which $\Lambda_i(t_k)\geq -\log(U_i).$

Note: a single uniform variate is sampled per patient, not per visit.

The parameters $c_0$ and $c_1$ control treatment-independent and treatment-dependent components of the baseline hazard. Let `TTE_P` be the placebo time-to-event scaling and `TTE_A` the active-arm scaling. Then:

- $c_0=\log(TTE_P)$
- $c_1=\log(TTE_A/TTE_P)$

By default `TTE_A=TTE_P,` so $c_1=0$, meaning no treatment effect on KFRT beyond its effect through eGFR. Thus, treatment reduces KFRT risk only by slowing eGFR decline.

 - **Calibration of TTE and theta:** `TTE_A` and `theta` are set so that the event rate is approximately 1 per patient-year at eGFR = 15 and 0.01 per patient-year at eGFR = 25. This yields `TTE_A=1000` and `theta=-0.4605.`

- **Additional constraints based on observed data:** In the current setup, KFRT risk depends on the true eGFR trajectory. In practice, however, the observed eGFR plays an important role in initiating KFRT. To reflect this, event rates are further adjusted based on observed eGFR: when observed eGFR is above 30, the patient’s event rate is set to a very low value (10E-7); when observed eGFR is at or below 7, the event rate is set to a very high value (10E5).


### Dependent Time-To-Event Outcomes

To model dependent outcomes, several methods are available:

1. **Joint Distribution Modeling Using Copulas:** This method employs copulas to model the joint distribution of outcomes, capturing their dependence.

1. **Random Frailty Modeling:** This approach captures patient-level dependence between outcomes using a random frailty model.

1. **Conditional Distribution Specification Through Multi-State Modeling:** This technique uses multi-state models to describe the conditional distribution of outcomes.

#### Joint Distribution Modeling Using Copulas

Sklar's theorem [@sklar1959fonctions] shows that multivariate distribution functions can be expressed using a copula and univariate distributions. For a random vector $X^d=(X_1,\cdots,X_d)$ with a multivariate distribution function $H(x_1,\cdots,x_d),$ Sklar's theorem states that there is a copula $C(\cdot)$ such that:
$$H(x_1,\cdots,x_d)=C(F_1(x_1),\cdots,F_d(x_d)),$$
where each component $X_j$ has the univariate distribution $F_j.$ A copula is essentially a multivariate distribution function where each univariate marginal distribution is uniform, describing the dependency structure of the multivariate distribution function $H(\cdot)$. To construct the multivariate distribution function, one combines each variable's univariate distributions $F_j$ with the copula. 

If $X_j$ has distribution function $F_j,$ then $U_j=F_j(X_j)$ is uniformly distributed,  allowing random variables $X_j\sim F_j$ to be simulated by generating uniform random variables $U_j$ and applying the inverse transformation:

$$F_j^{-1}(y)=\inf\{x\in {\mathbf R}: \ \ F_j(x)\geq y\}, \ \ \inf\varnothing=\infty.$$
Thus, if one has simulated a uniform random vector $U^d=(U_1,\cdots, U_d)$ from the copula $C(\cdot)$, the random vector $X^d=(X_1,\cdots,X_d)$ can be simulated as:
$$(X_1,\cdots,X_d)=(F_1^{-1}(U_1),\cdots,F_d^{-1} (U_d)).$$
The main challenge remains in simulating from the given copula.

An *Archimedean copula* [@nelsen2006introduction] is one where:

$$C(u^d;\varphi)=\varphi(\varphi^{-1}(u_1)+\cdots+\varphi^{-1}(u_d)).$$
The function $\varphi:[0,+\infty]\rightarrow [0,1]$ is a generator - continuous, decreasing, with $\varphi(0)=1$ and $\lim_{t\rightarrow+\infty}\varphi(t)=0.$ When $\varphi(t)=e^{-t^\theta},\ \ \theta>1,$ the copula is called a **Gumbel** copula.


##### The Marshall-Olkin algorithm

By Bernstein's theorem, completely monotone Archimedean generators coincide with Laplace-Stieltjes transforms of distribution functions $F,$ determined by $\varphi=LS[F].$ The **Marshall-Olkin** algorithm [@marshall1988families] for sampling from an Archimedean copula involves:

1. Sampling $V\sim F=LS^{-1}[\varphi].$
2. Sampling $R_j\sim Exp(1),\,j\in\{1,\cdots,d\}.$
3. Setting $U_j=\varphi\left(\frac{R_j}{V}\right),\,j\in\{1,\cdots,d\}.$

The vector $U=(U_1,\cdots, U_d)$ is then a random vector from the Archimedean copula with generator $\varphi$. For the Gumbel copula, one needs to use the inverse Laplace-Stieltjes transform of a stable distribution  [@nolan2020univariate; @hofert2011nested]:
$$F\sim S(1/\theta, 1, \cos^\theta(\pi/(2\theta)), {\mathbf I}_{\{\theta=1\}},1)$$
modifying the first step to sample from a stable distribution. 

##### Chambers-Mallows-Stuck method for simulating stable random variables

Chambers-Mallows-Stuck method [@chambers1976method] efficiently simulates stable variables with:

1. Generating independent uniform and exponential random variables
$$\Theta\sim U\left[-\frac{\pi}{2},\frac{\pi}{2}\right] \text{ and } W\sim Exp(1).$$ 
2. Defining $\alpha=1/\theta,$ setting $b_{\tan}=\beta\tan\left(\frac{\alpha\pi}{2}\right),$ and $\theta_0=\arctan(b_{\tan})/\alpha,$ with
$$C_{\tan}=(1+b_{\tan}^2)^\frac{1}{2\alpha}.$$

3. Utilizing the transformations:
$$Z(\theta) = \frac{\sin(a_0)  C_{\tan}}{\cos(\Theta)^\frac{1}{\alpha}}\left(\frac{\cos(a_0-\Theta)}{W}\right)^\frac{1-\alpha}{\alpha}, \ \ a_0=\alpha(\Theta+\theta_0),\,\theta>1.$$
$$Z(1)=\frac{2}{\pi}\left(\pi_\beta\tan(\Theta)-\beta\log\left(\frac{\pi}{2}W\frac{\cos(\Theta)}{\pi_\beta}\right)\right), \ \ \pi_\beta=\frac{\pi}{2}+\beta\Theta.$$
Finally, $\gamma Z + \delta$ has the desired distribution with $\gamma =[\cos(\pi/(2\theta))]^\theta$ and $\delta={\mathbf I}_{\{\theta=1\}}$ (and one needs to set $\beta=1$).

##### Note

In the original Chambers-Mallows-Stuck formula, the term $$\frac{1}{[\cos(\alpha\theta_0)\cos(\Theta)]^\frac{1}{\alpha}}$$ is replaced by
$\frac{C_{\tan}}{[\cos(\Theta)]^\frac{1}{\alpha}},$
as suggested by the `copula` package [@hofert2025copula], which is based on the fact that $C_{\tan} = 1/(\cos(\alpha\theta_0))^{1/\alpha}.$ Indeed, one needs to show that

$$(1+b_{\tan}^2)^\frac{1}{2\alpha}=1/(\cos(\alpha\theta_0))^{1/\alpha},$$
which is equivalent to showing that
$$1+\left[\beta\tan\left(\frac{\alpha\pi}{2}\right)\right]^2=\frac{1}{[\cos(\alpha\theta_0)]^2}.$$
And this is true because of the trigonometric identity 
$$1+[\tan(y)]^2=\frac{1}{[\cos(y)]^2}.$$
Here we have set $y=\alpha\theta_0=\arctan(b_{\tan})=\arctan\left(\beta\tan\left(\frac{\alpha\pi}{2}\right)\right)$ and hence $\tan(y)=\beta\tan\left(\frac{\alpha\pi}{2}\right).$


##### Implementation

The function `simHCE()` provides the implementation above with the argument `theta` specifying the dependence of outcomes.

```{r}
Rates_A <- c(10, 20)
Rates_P <- c(20, 20)
dat1 <- simHCE(n = 2500, TTE_A = Rates_A, TTE_P = Rates_P, 
CM_A = -3, CM_P = -6, CSD_A = 15, fixedfy = 3, theta = 1, seed = 1)
dat2 <- simHCE(n = 2500, TTE_A = Rates_A, TTE_P = Rates_P, 
CM_A = -3, CM_P = -6, CSD_A = 15, fixedfy = 3, theta = 1.0001, seed = 1)
dat3 <- simHCE(n = 2500, TTE_A = Rates_A, TTE_P = Rates_P, 
CM_A = -3, CM_P = -6, CSD_A = 15, fixedfy = 3, theta = 10, seed = 1)
calcWO(dat1)
calcWO(dat2)
calcWO(dat3)
```



## References

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/propWINS.R
\name{propWINS}
\alias{propWINS}
\title{Proportion of wins/losses/ties given the win odds and the win ratio}
\usage{
propWINS(WO, WR, Overall = 1, alpha = NULL, N = NULL)
}
\arguments{
\item{WO}{win odds.}

\item{WR}{win ratio.}

\item{Overall}{number of comparisons, the sample size of
the active treatment multiplied by the sample size of the placebo.
The default is 1, hence gives the proportion.}

\item{alpha}{significance level for the win ratio confidence interval. The default is \code{NULL} hence the confidence interval is not produced.}

\item{N}{the combined sample size of two treatment groups. The default is \code{NULL}. If \code{alpha} is specified then either \code{N} should be specified or \code{Overall > 1}.
For given \code{Overall}, the pooled sample size is calculated as \code{N = 2*sqrt(Overall)}.}
}
\value{
a data frame with a number (or proportion if \code{Overall = 1}) of wins/losses/ties. If \code{alpha} is specified returns also \code{WR} confidence interval.
}
\description{
Proportion of wins/losses/ties given the win odds and the win ratio
}
\details{
\figure{propWINS.png}{Calculation formula}
}
\examples{
# Example 1
propWINS(WR = 2, WO = 1.5)
# Example 2 - Back-calculation 
COVID19HCE <- hce(GROUP = COVID19$GROUP, TRTP = COVID19$TRTP)
res <- calcWINS(COVID19HCE)
WR <- res$WR1$WR
WO <- res$WO$WO
Overall <- res$summary$TOTAL
propWINS(WR = WR, WO = WO, Overall = Overall)
## Verify 
res$summary
# Example 3 - Confidence interval
 propWINS(WR = 1.4, WO = 1.3, alpha = 0.05, Overall = 2500)
 propWINS(WR = 2, WO = 1.5, alpha = 0.01, N = 500)
}
\references{
\itemize{
\item For the relationship between win odds and win ratio see
\cr \cr Gasparyan SB et al. "Hierarchical Composite Endpoints in COVID-19: The DARE-19 Trial". Case Studies in Innovative Clinical Trials, Chapter 7 (2023): 95-148. Chapman and Hall/CRC. \url{doi:10.1201/9781003288640-7}.
\item The win ratio CI uses the standard error presented in
\cr \cr Yu RX, Ganju J. (2022) "Sample size formula for a win ratio endpoint." Statistics in Medicine 41.6: 950-63. \url{doi:10.1002/sim.9297}.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{COVID19plus}
\alias{COVID19plus}
\title{COVID-19 ordinal scale dataset for a combination therapy.}
\format{
a data frame with 1033 rows and 4 variables:
\describe{
\item{ID}{patient identifiers, numeric}
\item{TRTP}{treatment values, A Active or P Placebo, character}
\item{GROUP}{type of the event, ordinal outcomes 1-8, where a higher value means a better outcome}
\item{BASE}{baseline ordinal values}
}
}
\source{
Kalil AC et al. "Baricitinib plus Remdesivir for Hospitalized Adults with Covid-19." New England Journal of Medicine 384.9 (2021): 795-807. \url{doi:10.1056/NEJMoa2031994}.
}
\usage{
COVID19plus
}
\description{
A dataset with COVID-19 ordinal scale outcomes for 1033 patients.
}
\examples{
COVID19HCE <- hce(GROUP = COVID19plus$GROUP, TRTP = COVID19plus$TRTP)
# Summary wins, losses, and ties with win odds
summaryWO(COVID19HCE, ref = "P")
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/regWO_formula.R
\name{regWO.formula}
\alias{regWO.formula}
\title{Win Odds Regression Using a Formula Syntax}
\usage{
\method{regWO}{formula}(x, data, ...)
}
\arguments{
\item{x}{an object of class formula.}

\item{data}{a data frame.}

\item{...}{additional parameters.}
}
\value{
a data frame containing the calculated win odds and its confidence interval, including:
\itemize{
\item WO_beta adjusted win odds.
\item LCL lower confidence limit for adjusted WO.
\item UCL upper confidence limit for adjusted WO.
\item SE standard error of the adjusted win odds.
\item WOnull win odds of the null hypothesis (specified in the \code{WOnull} argument).
\item alpha two-sided significance level for calculating the confidence interval (specified in the \code{alpha} argument).
\item Pvalue p-value associated with testing the null hypothesis.
\item N total number of patients in the analysis.
\item beta adjusted win probability.
\item LCL_beta lower confidence limit for adjusted win probability.
\item UCL_beta upper confidence limit for adjusted win probability.
\item SE_beta standard error for the adjusted win probability.
\item SD_beta standard deviation for the adjusted win probability.
\item WP (non-adjusted) win probability.
\item SE_WP standard error of the non-adjusted win probability.
\item SD_WP standard deviation of the non-adjusted win probability.
\item WO non-adjusted win odds.
\item COVAR_MEAN_DIFF mean difference between two treatment groups of the numeric covariate.
\item COVAR_VAR sum of variances of two treatment groups of the numeric covariate.
\item COVAR_COV covariance between the response and the numeric covariate.
\item formula returning the specified formula in the \code{x} argument.
\item ref showing how the reference group was selected. Can be modifying by specifying the \code{ref} argument.
}
}
\description{
This function performs regression analysis for the win odds using a single numeric covariate.
}
\examples{
regWO(AVAL ~ TRTP, data = KHCE)
regWO(AVAL ~ TRTP + EGFRBL, data = KHCE)
}
\references{
Gasparyan SB et al. (2021) "Adjusted win ratio with stratification: calculation methods and interpretation." Statistical Methods in Medical Research 30.2: 580-611. \url{doi:10.1177/0962280220942558}.
}
\seealso{
\code{\link[=regWO]{regWO()}}, \code{\link[=regWO.data.frame]{regWO.data.frame()}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sizeWR.R
\name{sizeWR}
\alias{sizeWR}
\title{Sample size calculation for the win ratio test (with WR = 1 null hypothesis)}
\usage{
sizeWR(WR, power, WO = NULL, Pties = NULL, k = 0.5, alpha = 0.05)
}
\arguments{
\item{WR}{a numeric vector of win odds values.}

\item{power}{the given power. A numeric vector of length 1.}

\item{WO}{win odds. Should be specified only if \code{Pties} is not specified. A numeric vector of length 1.}

\item{Pties}{probability of ties. A numeric vector of length 1.}

\item{k}{proportion of active group in the overall sample size. Default is 0.5 (balanced randomization). A numeric vector of length 1.}

\item{alpha}{the significance level for the 2-sided test. Default is 0.05. A numeric vector of length 1.}
}
\value{
a data frame containing the sample size with input values.
}
\description{
Sample size calculation for the win ratio test (with WR = 1 null hypothesis)
}
\examples{
sizeWR(WR = 1.35, Pties = 0.125, power = 0.8)
sizeWR(WR = 1.35, WO = 1.3, power = seq(0.5, 0.9, 0.05))
}
\references{
Yu RX, Ganju J. (2022) "Sample size formula for a win ratio endpoint." Statistics in Medicine, 41.6: 950-63. \url{doi:10.1002/sim.9297}.
}
\seealso{
\code{\link[=sizeWO]{sizeWO()}} for WO sample size calculation.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summaryWO_adhce.R
\name{summaryWO.adhce}
\alias{summaryWO.adhce}
\title{Win odds summary for \code{adhce} objects}
\usage{
\method{summaryWO}{adhce}(x, ...)
}
\arguments{
\item{x}{an \code{adhce} object.}

\item{...}{additional parameters.}
}
\value{
a list containing the summary of wins, losses, and ties. It contains the following named objects:
\itemize{
\item summary a data frame containing number of wins, losses, and ties by treatment group and the overall number of comparisons.
\item summary_by_GROUP a summary data frame by \code{GROUP}.
\item WO calculated WO (win odds) and WP (win probability) and their standard errors.
\item cumsummary_by_GROUP a cumulative summary data frame by \code{GROUP}.
}
}
\description{
Win odds summary for \code{adhce} objects
}
\examples{
## Example 1 - using an `hce` object
HCE5 <- HCE4
HCE5$PADY <- NULL
dat <- as_hce(HCE5)
## `PADY` is not present in the dataset, hence converts it to an `hce` object 
## instead of an `adhce` object.
## Example 2 - Using an `adhce` object
class(dat)
summaryWO(dat, ref = "P")
## The class is `adhce` hence will use the variable `GROUP`.
HCE5$PADY <- 1080
dat <- as_hce(HCE4) 
class(dat)
summaryWO(dat, ref = "P")
## Example 3 - Plotting cumulative components of an `adhce` object
dat <- as_hce(KHCE)
res0 <- summaryWO(dat, ref = "P")
res <- res0$cumsummary_by_GROUP
barplot(PROP ~ WINS + GROUPN, data = res, 
col = c("darkgreen", "darkred", "darkblue"), 
xlab = "Proportions", xlim = c(0, 1), 
ylab = "Cumulative components by prioritization", 
legend.text = unique(res$WINS), beside = TRUE, horiz = TRUE)
grid()
}
\seealso{
\code{\link[=calcWO]{calcWO()}}, \code{\link[=summaryWO]{summaryWO()}}, \code{\link[=summaryWO.data.frame]{summaryWO.data.frame()}}, \code{\link[=summaryWO.formula]{summaryWO.formula()}}, \code{\link[=summaryWO.hce]{summaryWO.hce()}} methods.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summaryWO_hce.R
\name{summaryWO.hce}
\alias{summaryWO.hce}
\title{Win odds summary for \code{hce} objects}
\usage{
\method{summaryWO}{hce}(x, ...)
}
\arguments{
\item{x}{an \code{hce} object.}

\item{...}{additional parameters.}
}
\value{
a list containing the summary of wins, losses, and ties. It contains the following named objects:
\itemize{
\item summary a data frame containing number of wins, losses, and ties by treatment group and the overall number of comparisons.
\item WO calculated WO (win odds) and WP (win probability) and their standard errors.
}
}
\description{
Win odds summary for \code{hce} objects
}
\examples{
COVID19HCE <- hce(GROUP = COVID19$GROUP, TRTP = COVID19$TRTP)
summaryWO(COVID19HCE, ref = "Placebo")
}
\seealso{
\code{\link[=calcWO]{calcWO()}}, \code{\link[=summaryWO]{summaryWO()}}, \code{\link[=summaryWO.data.frame]{summaryWO.data.frame()}}, \code{\link[=summaryWO.formula]{summaryWO.formula()}}, \code{\link[=summaryWO.adhce]{summaryWO.adhce()}} methods.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calcWINS.R
\name{calcWINS}
\alias{calcWINS}
\title{A generic function for calculating win statistics}
\usage{
calcWINS(x, ...)
}
\arguments{
\item{x}{an object used to select a method.}

\item{...}{further arguments passed to or from other methods.}
}
\value{
a data frame containing calculated values.
}
\description{
A generic function for calculating win statistics
}
\seealso{
\code{\link[=calcWINS.hce]{calcWINS.hce()}}, \code{\link[=calcWINS.formula]{calcWINS.formula()}}, \code{\link[=calcWINS.data.frame]{calcWINS.data.frame()}}  methods.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hce.R
\name{hce}
\alias{hce}
\title{Helper function for \code{hce} objects}
\usage{
hce(GROUP, TRTP, AVAL0 = NULL, PADY = NULL)
}
\arguments{
\item{GROUP}{a character vector or a factor containing events.
If a factor, its levels are used to define the hierarchy. Otherwise, the vector is
converted to a factor.}

\item{TRTP}{a character vector of the same length as \code{GROUP}, indicating assigned treatment groups.}

\item{AVAL0}{a numeric vector of the same length as \code{GROUP}, indicating containing analysis values within each category. The default is 0.}

\item{PADY}{numeric specifying the length of follow-up in years.}
}
\value{
an object of class \code{hce} or \code{adhce} (if \code{AVAL0} is provided). The result
is a subject-level data frame, where each row corresponds to one subject,
}
\description{
Helper function for \code{hce} objects
}
\examples{
# Example 1 - Both `AVAL0` and `PADY` are provided. The output is an `adhce` object.
GROUP <- COVID19$GROUP
TRTP <- rep(c("A", "P"), each = 531)
dat <- hce(GROUP, TRTP, PADY = 10, AVAL0 = rnorm(1062))
class(dat)
calcWO(dat)
summaryWO(dat) # Uses the `GROUP` variable for summary.
# Example 2 - Only `AVAL0` is provided, `PADY` is calculated as the maximum of `AVAL0`. 
# The output is an `adhce` object.
set.seed(2022)
d <- hce(GROUP = sample(x = c("A", "B", "C"), size = 10, replace = TRUE), 
TRTP = rep(c("Active", "Control"), each = 5), 
AVAL0 = c(rnorm(5, mean = 1), rnorm(5)))
calcWO(d, ref = "Control")
## modify the hierarchy by proving a factor for the GROUP variable. 
## calcWO() applied to an hce rederives `AVAL` based on the `GROUP` variable.
d$GROUP <- factor(d$GROUP, levels = c("C", "B", "A"))
calcWO(d, ref = "Control")
# Example 3 - Provide only `PADY` and not `AVAL0` will not make any difference.
GROUP <- COVID19$GROUP
TRTP <- rep(c("A", "P"), each = 531)
dat <- hce(GROUP, TRTP, PADY = 10)
class(dat)
calcWO(dat)
dat <- hce(GROUP, TRTP)
class(dat)
calcWO(dat)
}
\seealso{
\code{\link[=as_hce]{as_hce()}} for coercing to \code{hce} objects.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{KHCE}
\alias{KHCE}
\title{Kidney Hierarchical Composite Endpoint dataset.}
\format{
a data frame with 1500 rows and 11 variables:
\describe{
\item{ID}{patient identifiers, numeric}
\item{TRTPN}{treatment values, 1 Active or 2 Placebo, numeric}
\item{AVAL0}{original values for each type of the event, time for TTE outcomes 1-6, numeric values for Continuous outcome 7, numeric}
\item{AVAL}{\code{AVAL = AVAL0 + GROUPN}, ordinal analysis values for the HCE analysis, numeric, but caution NOT to apply numeric operations; will give meaningless results}
\item{GROUP}{name of the event, character}
\item{GROUPN}{ordinal outcomes corresponding to \code{PARAMN} values, numeric}
\item{PARAMCD}{coded name of the event, character}
\item{PARAMN}{severity of the event, outcomes 1-7, where a higher value means a better outcome, character}
\item{STRATAN}{strata 1-4, higher value means more severe kidney disease, numeric}
\item{EGFRBL}{Baseline GFR values of patients, numeric}
\item{TRTP}{treatment values, A Active or P Placebo, character}
\item{PADY}{primary analysis day (in years), length of the fixed follow-up, numeric}
}
}
\source{
Heerspink HL et al "Development and validation of a new hierarchical composite endpoint for clinical trials of kidney disease progression." Journal of the American Society of Nephrology (2023): \url{doi:10.1681/ASN.0000000000000243}.
}
\usage{
KHCE
}
\description{
A dataset with kidney ordinal scale outcomes of 1500 patients in the \code{ADSL} dataset.
}
\examples{
# Adjusted win odds
res <- regWO(x = KHCE, AVAL = "AVAL", TRTP = "TRTP", COVAR = "STRATAN", ref = "P")
res
# Convert the dataset to an adhce object.
## First check that `GROUP` is a factor with the correct ordering of outcomes.
class(KHCE$GROUP) # "factor"
levels(KHCE$GROUP)
dat1 <- as_hce(KHCE)
class(dat1)
calcWO(dat1)
## Re-derive individual patient eGFR slopes using a linear regression model, 
## based on the eGFR measurements in the `ADLB` dataset
dat2 <- KHCE
l <- lapply(
split(ADLB, ADLB$ID), 
function(x) coef(lm(AVAL ~ ADAY, data = x))[2])
new_slopes <- do.call(rbind, l)
new_slopes <- as.data.frame(new_slopes)
names(new_slopes) <- "LINEAR"
new_slopes$ID <- as.numeric(row.names(new_slopes))
dat2 <- merge(KHCE, new_slopes, by = "ID", all.x = TRUE)
dat2$AVAL0[dat2$PARAMCD == "eGFR"] <- dat2$LINEAR[dat2$PARAMCD == "eGFR"]
dat2$AVAL0[is.na(dat2$AVAL0)] <- 0 
dat2 <- as_hce(dat2)
calcWO(dat2)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simHCE.R
\name{simHCE}
\alias{simHCE}
\title{Simulate an \code{hce} object}
\usage{
simHCE(
  n,
  n0 = n,
  TTE_A,
  TTE_P,
  CM_A,
  CM_P,
  CSD_A = 1,
  CSD_P = CSD_A,
  fixedfy = 1,
  yeardays = 360,
  pat = 100,
  shape = 1,
  theta = 1,
  logC = FALSE,
  seed = NULL,
  dec = 2,
  all_data = FALSE
)
}
\arguments{
\item{n}{sample size in the active treatment group.}

\item{n0}{sample size in the placebo group.}

\item{TTE_A}{event rates per year in the active group for the time-to-event outcomes.}

\item{TTE_P}{event rates per year in the placebo group for the time-to-event outcomes. Should have the same length as \code{TTE_A}.}

\item{CM_A}{mean value for the continuous outcome of the active group.}

\item{CM_P}{mean value for the continuous outcome of the placebo group.}

\item{CSD_A}{standard deviation for the continuous outcome of the active group.}

\item{CSD_P}{standard deviation for the continuous outcome of the placebo group.}

\item{fixedfy}{length of follow-up in years.}

\item{yeardays}{number of days in a year.}

\item{pat}{scale of provided event rates (per \code{pat}-years).}

\item{shape}{shape of the Weibull distribution for time-to-event outcomes. Default is exponential distribution with \code{shape = 1}.}

\item{theta}{Gumbel dependence coefficient of the Weibull distributions for time-to-event outcomes. Default is \code{theta = 1} which assumes independence of time-to-event outcomes. Must be above or equal to 1.}

\item{logC}{logical, whether to use log-normal distribution for the continuous outcome.}

\item{seed}{for generating random numbers.}

\item{dec}{decimal places for the continuous outcome used for rounding. The default is \code{dec = 2}.}

\item{all_data}{logical, whether to return source datasets \code{ADET} (an event-time dataset for all time-to-event outcomes per patient) and \code{BDS} (a basic data structure for the continuous outcome for all patients).}
}
\value{
an object of class \code{hce} containing the following columns:
\itemize{
\item ID subject identifier.
\item TRTP planned treatment group - "A" for active, "P" for Placebo.
\item GROUP type of the outcome, either "TTE" for time-to-event outcomes or "C" for continuous.
Only one continuous outcome is possible, but no restriction on the number of "TTE" outcomes.
\item GROUPN order of outcomes in \code{GROUP}, with a higher value signifying a better outcome.
\item AVALT the timing of the time-to-event outcomes.
\item AVAL0 numeric values of the continuous outcome and the timing of "TTE" outcomes.
\item AVAL analysis values derived as \code{AVAL0 + GROUPN}. For the continuous outcome the values of \code{AVAL0} are shifted to start always from 0.
\item seed the seed of the random sample. If not specified in \code{seed} argument will be selected based on system time.
\item PADY primary analysis day, the length of fixed follow-up in days calculated as \code{yeardays} multiplied by \code{fixedfy}.
}

If \code{all_data = TRUE}, the function returns a list containing the \code{hce} dataset, along with its source datasets: \code{ADET} (an event-time dataset for all time-to-event outcomes per patient) and \code{BDS} (a basic data structure for the continuous outcome for all patients).
}
\description{
Simulate an \code{hce} object with multiple, possibly correlated (Hougaard copula) Weibull time‑to‑event outcomes and a single continuous endpoint (normal or log‑normal)
}
\examples{
# Example 1
Rates_A <- c(1.72, 1.74, 0.58, 1.5, 1)
Rates_P <- c(2.47, 2.24, 2.9, 4, 6)
dat <- simHCE(n = 2500, TTE_A = Rates_A, TTE_P = Rates_P,
              CM_A = -3, CM_P = -6, CSD_A = 16, CSD_P = 15, fixedfy = 3)
head(dat)

# Example 2
Rates_A <- 10
Rates_P <- 15
dat <- simHCE(n = 1000, n0 = 500, TTE_A = Rates_A, TTE_P = Rates_P, 
              CM_A = 0.1, CM_P = 0, seed = 5, shape = 0.2, logC = TRUE, dec = 0)
summaryWO(dat)

# Example 3: Comparison of dependent and independent outcomes
Rates_A <- c(10, 20)
Rates_P <- c(20, 20)
dat1 <- simHCE(n = 2500, TTE_A = Rates_A, TTE_P = Rates_P, 
CM_A = -3, CM_P = -6, CSD_A = 15, fixedfy = 3, theta = 1, seed = 1)
dat2 <- simHCE(n = 2500, TTE_A = Rates_A, TTE_P = Rates_P, 
CM_A = -3, CM_P = -6, CSD_A = 15, fixedfy = 3, theta = 1.0001, seed = 1)
calcWO(dat1)
calcWO(dat2)
}
\seealso{
\code{\link[=hce]{hce()}}, \code{\link[=as_hce]{as_hce()}} for the helper a coerce function to \code{hce} objects.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{HCE3}
\alias{HCE3}
\title{\code{HCE1}, \code{HCE2}, \code{HCE3}, \code{HCE4} datasets for 1000 patients with different treatment effects.}
\format{
a data frame with 1000 rows and 6 variables:
\describe{
\item{ID}{subject ID, numbers from 1 to 1000}
\item{TRTP}{treatment values, A Active or P Placebo, character}
\item{GROUP}{type of the event, either Time-To-Event (TTE) or Continuous (C), character}
\item{GROUPN}{type of the event, for the ordering of outcomes in the \code{GROUP} variable, numeric}
\item{AVALT}{the timing of the time-to-event outcomes, numeric}
\item{AVAL0}{original values for each type of the event, time for TTE outcomes, numeric values for Continuous outcomes, numeric}
\item{AVAL}{\code{AVAL = AVAL0 + GROUPN}, ordinal analysis values for the HCE analysis. For the continuous outcome the values of \code{AVAL0} are shifted to start always from 0. Numeric, but caution NOT to apply numeric operations; will give meaningless results}
\item{PADY}{ primary analysis day, the length of fixed follow-up in days, numeric}
}
}
\usage{
HCE3
}
\description{
A simulated dataset containing the ordinal values and other attributes for 1000 patients.
HCE3
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simTTE.R
\name{simTTE}
\alias{simTTE}
\title{Simulate an \code{adhce} dataset with two correlated outcomes (illness - death model)}
\usage{
simTTE(
  n,
  n0 = n,
  TTE_A,
  TTE_P = TTE_A,
  shape = 1,
  shape0 = shape,
  fixedfy = 2,
  theta = 1,
  alpha0 = 1,
  alpha = 1,
  rHR = 1,
  m = Inf,
  hce_type = c("mi", "md")
)
}
\arguments{
\item{n}{sample size in the active treatment group.}

\item{n0}{sample size in the placebo treatment group.}

\item{TTE_A}{event rates in the active group for the time-to-event outcomes; a numeric vector of length two.}

\item{TTE_P}{event rates in the placebo group for the time-to-event outcomes; a numeric vector of length two.}

\item{shape}{shape parameter of the Weibull distribution for time-to-event outcomes in the active group. Default is 1 (exponential distribution).}

\item{shape0}{shape parameter of the Weibull distribution for time-to-event outcomes in the placebo group. Default is 1 (exponential distribution).}

\item{fixedfy}{length of follow-up.}

\item{theta}{heterogeneity coefficient for the first event, modeled via a gamma distribution with mean 1; \code{theta} controls the variance. When \code{theta = 0}, there is no heterogeneity, which implies that death and hospitalization are independent.}

\item{alpha0}{exponential heterogeneity coefficient for modeling the heterogeneity of risk of death as the first event.}

\item{alpha}{exponential heterogeneity coefficient for modeling the heterogeneity of risk of death after hospitalization; the heterogeneity of the second event is the inverse of the time of the first event.}

\item{rHR}{recurrence hazard ratio comparing the active group to the control group for the second event, based on gap time measured from the first event.}

\item{m}{categorization number used for to discretize time to death and time to hospitalization. Defaults to \code{Inf} (no discretization).}

\item{hce_type}{the type of the hierarchical composite endpoint: use \code{mi} for the most-important outcome or \code{md} for the move-down approach. This parameter only affects results when \code{m} is finite.}
}
\value{
an object of class \code{adhce}.
}
\description{
Simulate an \code{adhce} dataset with two correlated outcomes - death and hospitalization - from a heterogeneous population. The correlation between these outcomes arises from population heterogeneity. Models the risk of death following hospitalization as dependent on the timing
of the hospitalization, reflecting strong dependence between the times to the first and second events (i.e., event clustering).
}
\details{
The default setting assumes \code{TTE_A = TTE_P}. Both \code{TTE_A} and \code{TTE_P} must be numeric vectors of length two, corresponding to the event rates
(Weibull distribution) for the first event of hospitalization and death. The parameters \code{shape} and \code{shape0} identify the shape parameters
of Weibull distributions for the first event, simulated from a distribution with a cumulative hazard of
\deqn{\gamma\cdot rate\cdot t^{shape}}
for hospitalization and
\deqn{\gamma^{\alpha_0}\cdot rate\cdot t^{shape}}
for death, where \eqn{\gamma} (\code{gamma}) is a patient-specific frailty drawn from a gamma distribution with mean 1 and
variance \eqn{\theta}, shared between death and hospitalization for a given patient. The parameter \eqn{\theta} (\code{theta}) represents population heterogeneity and also induces
correlation between death and hospitalization as competing first events. The parameter \eqn{\alpha_0} (\code{alpha0}) controls the heterogeneity of time to death through its
effect on heterogeneity. Death after hospitalization is simulated from an exponential distribution with a constant hazard that depends on the timing \eqn{t_1}
of the first event (hospitalization) as
\deqn{\frac{TTE_A[2] + TTE_P[2]}{2}\cdot \left(\frac{t_1}{fixedfy}\right)^{\alpha}\cdot \gamma^{\alpha_0}}
for the placebo arm and
\deqn{\frac{TTE_A[2] + TTE_P[2]}{2}\cdot rHR \cdot \left(\frac{t_1}{fixedfy}\right)^{\alpha}\cdot \gamma^{\alpha_0}}
for the active arm where \code{rHR} is the recurrence hazard ratio. When \eqn{\alpha < 0}, earlier hospitalization (smaller \eqn{t_1})
increases the risk of death following hospitalization.

By default, events are simulated in continuous time. When \code{m} is specified as a positive numeric value, the event times are discretized into \code{m} intervals over the follow-up period.
}
\examples{
## Example 1 - positive correlation
i <- 1764002323
set.seed(i)
PADY <- 2
D <- simTTE(n = 1000, TTE_A = c(0.1, 0.04), 
TTE_P = c(.15, 0.045), theta = 4, alpha0 = 2, alpha = -1, shape = 2, 
fixedfy = PADY, rHR = 1)
####### Summary of first events by treatment group ########
table(D$EVENT1, D$TRTP)
####### Summary of second events by treatment group ########
table(D$EVENT2, D$TRTP)
######## Calculate win odds #########################
calcWO(D, ref = "P")
## Plot the ordinal dominance graph ######
D$TRTP <- factor(D$TRTP, levels = c("P", "A"))
plot(D, type = "l", col = 2, fill = TRUE)
abline(a = 0, b = 1, lwd = 2, lty = 3, col = "darkgreen")
grid()
################################################################
## Example 2 - Move-down approach (discrete‑time case only)
PADY <- 2
# Continuous-time
set.seed(2)
D <- simTTE(n = 1000, TTE_A = c(0.1, 0.04), 
TTE_P = c(.15, 0.045), theta = 4, alpha0 = 2, alpha = -1, shape = 2, 
fixedfy = PADY, rHR = 1, m = Inf)
summaryWO(D, ref = "P")$summary_by_GROUP
# Discrete-time (more-ties)
D0 <- simTTE(n = 1000, TTE_A = c(0.1, 0.04), 
TTE_P = c(.15, 0.045), theta = 4, alpha0 = 2, alpha = -1, shape = 2, 
fixedfy = PADY, rHR = 1, m = 5)
summaryWO(D0, ref = "P")$summary_by_GROUP
# Discrete-time and move-down approach (less ties on death)
D1 <- simTTE(n = 1000, TTE_A = c(0.1, 0.04), 
TTE_P = c(.15, 0.045), theta = 4, alpha0 = 2, alpha = -1, shape = 2, 
fixedfy = PADY, rHR = 1, m = 5, hce_type = "md")
summaryWO(D1, ref = "P")$summary_by_GROUP

}
\seealso{
\code{\link[=simHCE]{simHCE()}} for a general \code{adhce} dataset simulation, and \code{\link[=simKHCE]{simKHCE()}} for kidney disease-specific \code{adhce} simulation.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print_results.R
\name{print.hce_results}
\alias{print.hce_results}
\title{A print method for \code{hce_results} objects}
\usage{
\method{print}{hce_results}(x, ...)
}
\arguments{
\item{x}{an object of class \code{hce_results}.}

\item{...}{additional arguments to be passed to \code{\link[base:print]{base::print()}} function.}
}
\value{
no return value, called for printing.
}
\description{
A print method for \code{hce_results} objects
}
\examples{
print(powerWO(N = 1000, WO = 1.2))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simKHCE.R
\name{simKHCE}
\alias{simKHCE}
\title{Simulate a kidney disease \code{hce} dataset}
\usage{
simKHCE(
  n,
  CM_A,
  CM_P = -4,
  n0 = n,
  TTE_A = 1000,
  TTE_P = TTE_A,
  fixedfy = 2,
  Emin = 20,
  Emax = 100,
  sigma = NULL,
  Sigma = 3,
  m = 10,
  theta = -0.4605,
  phi = 0,
  two_meas = c("no", "base", "postbase", "both"),
  df_sigma = Inf
)
}
\arguments{
\item{n}{sample size in the active treatment group.}

\item{CM_A}{annualized eGFR slope in the active group.}

\item{CM_P}{annualized eGFR slope in the control group.}

\item{n0}{sample size in the control treatment group.}

\item{TTE_A}{event rate per year in the active group for KFRT.}

\item{TTE_P}{event rate per year in the placebo group for KFRT.}

\item{fixedfy}{length of follow-up in years.}

\item{Emin}{lower limit of eGFR at baseline.}

\item{Emax}{upper limit of eGFR at baseline.}

\item{sigma}{within-patient standard deviation.}

\item{Sigma}{between-patient standard deviation.}

\item{m}{number of equidistant visits.}

\item{theta}{coefficient of dependence of eGFR values and the risk of KFRT.}

\item{phi}{coefficient of proportionality (between 0 and 1) of the treatment effect. The case of 0 corresponds to the uniform treatment effect.}

\item{two_meas}{determines whether to use duplicate measurements at baseline and/or at \code{fixedfy}. The default is to use a single measurement.}

\item{df_sigma}{degrees of freedom for the measurement error distribution. The default \code{Inf} gives normal measurement errors; must be more than 2.}
}
\description{
Simulate a kidney disease \code{hce} dataset, capturing eGFR (Estimated Glomerular Filtration Rate) progression over time, along with
a competing and dependent terminal event: KFRT (Kidney Failure Replacement Therapy)
}
\details{
The default setting is \code{TTE_A = TTE_P} because, conditional on eGFR level,
the treatment effect does not influence the event rate of KFRT. In this model,
the effect of treatment on KFRT operates entirely through its impact on eGFR decline.

Let \code{CM_A} and \code{CM_P} be denoted by \eqn{\beta_A} and \eqn{\beta_P}, respectively.
Let \code{TTE_A} and \code{TTE_P} be denoted by \eqn{\lambda_A} and \eqn{\lambda_P}, respectively.
Let \code{theta} be denoted by \eqn{\theta}, \code{phi} by \eqn{\phi}, \code{sigma} by \eqn{\sigma},
\code{Sigma} by \eqn{\Sigma}, and \code{df_sigma} by \eqn{\nu}.

The parameters \code{TTE_A} and \code{theta} are chosen so that when GFR is 15, the event rate
is 1 per patient per year, and when GFR is 25, the event rate is 0.01 per patient per year. These
parameter values are obtained by solving
\deqn{
\lambda_g \exp(\theta \cdot \mathrm{GFR}) = \mathrm{rate}
}
for the group-specific baseline rate \eqn{\lambda_g} and \eqn{\theta}. When the observed eGFR is above 30,
the event rate is set to a very low value (\code{10e-7}), while when the observed eGFR is below or equal to 7,
the event rate is set to a very high value (\code{10e5}). This ensures that patients with observed low eGFR values
always experience KFRT, while those with high eGFR values do not.

By default, the within-patient standard deviation, \code{sigma}, is set to \code{NULL}. When left as \code{NULL}, \code{sigma}
is calculated as
\deqn{
\sigma_{ij} = \sqrt{0.67 \cdot \mu_{ij}},
}
where \eqn{\mu_{ij}} denotes the predicted eGFR for patient \eqn{i} at visit \eqn{j}. This yields
time-dependent measurement variability, with lower predicted eGFR values corresponding to lower variability.
In the implementation, to prevent the variance from becoming negative or too small, the quantity
\eqn{0.67 \cdot \mu_{ij}} is truncated below at \code{0.1} before taking the square root.

When \code{df_sigma = Inf} (the default), measurement errors are normally distributed:
\deqn{
\varepsilon_{ij} \sim N(0, \sigma_{ij}^2).
}
When \code{df_sigma} is finite and greater than 2, measurement errors follow a Student t distribution
with heavier tails:
\deqn{
\varepsilon_{ij} = \sigma_{ij}\sqrt{\frac{\nu - 2}{\nu}} T_{\nu},
}
where \eqn{\nu =} \code{df_sigma} and \eqn{T_{\nu}} is a standard t random variable with \eqn{\nu}
degrees of freedom. This scaling preserves the same time-dependent standard deviation defined by \code{sigma},
so that
\deqn{
\mathrm{Var}(\varepsilon_{ij}) = \sigma_{ij}^2.
}

Given the overall effect \code{Delta} and the placebo progression rate \code{CM_P}, a fully uniform (purely additive)
treatment effect—meaning the same average effect applies to all patients regardless of baseline progression—is
implemented by setting \eqn{\phi = 0} and
\deqn{
\beta_A = \Delta + \beta_P.
}
A fully proportional treatment effect—no additive component, the effect scales with the baseline rate—is
implemented by setting
\deqn{
\beta_A = \beta_P
}
and
\deqn{
\phi = \frac{\Delta}{|\beta_P|}.
}
A more relativistic intermediate effect (half additive and half proportional) is obtained by setting
\deqn{
\phi = \frac{\Delta}{2|\beta_P|}
}
and
\deqn{
\beta_A = \frac{\Delta}{2} + \beta_P.
}

The kidney hierarchical composite endpoint is defined in the following order:
(1) Kidney Failure Replacement Therapy (KFRT); (2) Sustained eGFR < 15;
(3) Sustained 57 percent or more decline in eGFR; (4) Sustained 50 percent or more decline in eGFR;
(5) Sustained 40 percent or more decline in eGFR; and (6) Change in eGFR. In practice, because KFRT is
frequently initiated when true eGFR is very low, sustained eGFR < 15 events are rarely observed.

If \code{two_meas} is not \code{"no"}, a second measurement is generated using the same latent baseline eGFR and
subject-specific slope, but with an independent measurement error draw from the same distribution as above.
The two measurements are then averaged at baseline when \code{two_meas = "base"}, at the final visit when
\code{two_meas = "postbase"}, or at both baseline and the final visit when \code{two_meas = "both"}. This implementation
preserves correlation between the duplicate measurements because they share the same underlying true trajectory.
}
\examples{
# Example - Specifying the most important variables
set.seed(2022)
## The overall treatment effect
Delta <- 0.75
## The placebo progression rate
CM_P <- - 4.5
## Intermediate effect (half additive and half proportional)
delta <- Delta/2
CM_A <- delta + CM_P 
phi <- Delta / (2*abs(CM_P))
L <- simKHCE(n = 1000, CM_A = CM_A, CM_P = CM_P, 
fixedfy = 4, Emin = 25, Emax = 75, phi = phi)
dat <- L$HCE
calcWO(dat)
}
\seealso{
\code{\link[=simHCE]{simHCE()}} for a general function of simulating \code{hce} datasets.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summaryWO_formula.R
\name{summaryWO.formula}
\alias{summaryWO.formula}
\title{Win odds summary using formula syntax}
\usage{
\method{summaryWO}{formula}(x, data, ...)
}
\arguments{
\item{x}{an object of class formula.}

\item{data}{a data frame.}

\item{...}{additional parameters.}
}
\value{
a list containing the summary of wins, losses, and ties. It contains the following named objects:
\itemize{
\item summary a data frame containing number of wins, losses, and ties by treatment group and the overall number of comparisons.
\item WO calculated WO (win odds) and WP (win probability) and their standard errors.
\item formula returning the specified formula in the \code{x} argument.
\item ref showing how the reference group was selected. Can be modifying by specifying the \code{ref} argument.
}
}
\description{
Win odds summary using formula syntax
}
\examples{
# Example 1
summaryWO(data = COVID19, GROUP ~ TRTP)
summaryWO(data = COVID19, GROUP ~ TRTP, GROUP = "GROUP", ref = "Placebo")
# Example 2 - Individual wins, losses, and ties
dat <- COVID19
dat$ID <- 1:nrow(dat)
summaryWO(data = dat, GROUP ~ TRTP, GROUP = "ID", ref = "Placebo")
}
\seealso{
\code{\link[=calcWO]{calcWO()}}, \code{\link[=summaryWO]{summaryWO()}}, \code{\link[=summaryWO.data.frame]{summaryWO.data.frame()}}, \code{\link[=summaryWO.hce]{summaryWO.hce()}}  methods.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/minWO.R
\name{minWO}
\alias{minWO}
\title{Minimum detectable or WO for alternative hypothesis for given power (no ties)}
\usage{
minWO(N, power = 0.5, SD = NULL, k = 0.5, alpha = 0.05, WOnull = 1, digits = 2)
}
\arguments{
\item{N}{a numeric vector of sample size values (two arms combined).}

\item{power}{the given power. The default is 0.5 corresponding to the minimum detectable win odds. A numeric vector of length 1.}

\item{SD}{assumed standard deviation of the win proportion. By default uses the conservative SD. A numeric vector of length 1.}

\item{k}{proportion of active group in the overall sample size. Default is 0.5 (balanced randomization). A numeric vector of length 1.}

\item{alpha}{the significance level for the 2-sided test. Default is 0.05. A numeric vector of length 1.}

\item{WOnull}{the win odds value of the null hypothesis (default is 1). A numeric vector of length 1.}

\item{digits}{precision to use for reporting calculated win odds.}
}
\value{
a data frame containing the calculated WO with input values.
}
\description{
Minimum detectable or WO for alternative hypothesis for given power (no ties)
}
\examples{
minWO(N = 100, digits = 5)
minWO(N = 1200, power = 0.9, k = 0.75)
# Compare the minimum detectable win odds from shifted alternatives to max and ordered alternatives
WO <- minWO(N = 1200, k = 0.5, power = 0.67, digits = 7)$WO
powerWO(N = 1200, WO = WO, k = 0.5, alternative = "shift")
powerWO(N = 1200, WO = WO, k = 0.5, alternative = "ordered")
powerWO(N = 1200, WO = WO, k = 0.5, alternative = "max")
}
\references{
Gasparyan SB et al. (2021) "Power and sample size calculation for the win odds test: application to an ordinal endpoint in COVID-19 trials." Journal of Biopharmaceutical Statistics 31.6: 765-787. \url{doi:10.1080/10543406.2021.1968893}.
}
\seealso{
\code{\link[=powerWO]{powerWO()}}, \code{\link[=sizeWO]{sizeWO()}}  for WO power and sample size calculation.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{COVID19b}
\alias{COVID19b}
\title{COVID-19 ordinal scale dataset (preliminary report).}
\format{
a data frame with 844 rows and 2 variables:
\describe{
\item{GROUP}{type of the event, ordinal outcomes 1-8, where a higher value means a better outcome}
\item{TRTP}{treatment values, Active or Placebo, character}
}
}
\source{
Beigel JH et al. "Remdesivir for the treatment of Covid-19-final report." New England Journal of Medicine 383.19 (2020): 1813-1836. \url{doi:10.1056/NEJMoa2007764}.
}
\usage{
COVID19b
}
\description{
A dataset with COVID-19 ordinal scale outcomes for 844 patients.
}
\examples{
#Frequencies
table(COVID19b)
mosaicplot(table(COVID19b), col = c(1, 8, 6, 2, 4, 5, 3, 7), 
xlab = "Treatment", ylab = "Ordinal Scale", main = "COVID-19 ordinal scale")
# Calculate win statistics
calcWINS(x = COVID19b, AVAL = "GROUP", TRTP = "TRTP", ref = "Placebo")
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/regWO.R
\name{regWO}
\alias{regWO}
\title{A generic function for win odds regression}
\usage{
regWO(x, ...)
}
\arguments{
\item{x}{an object used to select a method.}

\item{...}{further arguments passed to or from other methods.}
}
\value{
a data frame containing calculated values.
}
\description{
A generic function for win odds regression
}
\seealso{
\code{\link[=regWO.data.frame]{regWO.data.frame()}}, \code{\link[=regWO.formula]{regWO.formula()}} methods.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summaryWO_data_frame.R
\name{summaryWO.data.frame}
\alias{summaryWO.data.frame}
\title{Win odds summary for a data frame}
\usage{
\method{summaryWO}{data.frame}(x, AVAL, TRTP, ref, GROUP = NULL, ...)
}
\arguments{
\item{x}{a data frame containing subject-level data.}

\item{AVAL}{variable in the data with ordinal analysis values.}

\item{TRTP}{the treatment variable in the data.}

\item{ref}{the reference treatment group.}

\item{GROUP}{an optional variable for grouping.}

\item{...}{additional parameters.}
}
\value{
a list containing the summary of wins, losses, and ties. It contains the following named objects:
\itemize{
\item summary a data frame containing number of wins, losses, and ties by treatment group and the overall number of comparisons.
\item summary_by_GROUP (if \code{GROUP} variable is specified) a summary data frame by \code{GROUP}.
\item WO calculated WO (win odds) and WP (win probability) and their standard errors.
}
}
\description{
Win odds summary for a data frame
}
\examples{
summaryWO(x = HCE3, AVAL = "AVAL", TRTP = "TRTP", ref = "P", GROUP = "GROUP")
}
\seealso{
\code{\link[=calcWO]{calcWO()}}, \code{\link[=summaryWO]{summaryWO()}}, \code{\link[=summaryWO.formula]{summaryWO.formula()}}, \code{\link[=summaryWO.hce]{summaryWO.hce()}}   methods.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calcWO_hce.R
\name{calcWO.hce}
\alias{calcWO.hce}
\title{Win odds calculation for \code{hce} objects}
\usage{
\method{calcWO}{hce}(x, ...)
}
\arguments{
\item{x}{an \code{hce} object.}

\item{...}{additional parameters.}
}
\value{
a data frame containing the win odds and its confidence interval. It contains the following columns:
\itemize{
\item WO calculated win odds.
\item LCL lower confidence limit.
\item UCL upper confidence limit.
\item SE standard error of the win odds.
\item WOnull win odds of the null hypothesis (specified in the \code{WOnull} argument).
\item alpha two-sided significance level for calculating the confidence interval (specified in the \code{alpha} argument).
\item Pvalue p-value associated with testing the null hypothesis.
\item WP calculated win probability.
\item LCL_WP lower confidence limit for \code{WP}.
\item UCL_WP upper confidence limit for \code{WP}.
\item SE_WP standard error of the win probability.
\item SD_WP standard deviation of the win probability, calculated as \code{SE_WP} multiplied by \code{sqrt(N)}.
\item N total number of patients in the analysis.
}
}
\description{
Win odds calculation for \code{hce} objects
}
\examples{
Rates_A <- c(1, 1.5) 
Rates_P <- c(2, 2) 
dat <- simHCE(n = 500, TTE_A = Rates_A, TTE_P = Rates_P, CM_A = 1.25, CM_P = 1)
calcWO(dat)
calcWO(dat, ref = "A", WOnull = 1, alpha = 0.01)
}
\references{
Gasparyan SB et al. "Adjusted win ratio with stratification: calculation methods and interpretation." Statistical Methods in Medical Research 30.2 (2021): 580-611. \url{doi:10.1177/0962280220942558}.
}
\seealso{
\code{\link[=calcWO]{calcWO()}}, \code{\link[=calcWO.formula]{calcWO.formula()}}, \code{\link[=calcWO.data.frame]{calcWO.data.frame()}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as_hce.default.R
\name{as_hce.default}
\alias{as_hce.default}
\title{Coerce a data frame to an \code{hce} object}
\usage{
\method{as_hce}{default}(x, ...)
}
\arguments{
\item{x}{an object.}

\item{...}{additional parameters.}
}
\value{
an \code{hce} object.
}
\description{
Coerce a data frame to an \code{hce} object
}
\examples{
dat <- KHCE
class(dat) <- "moo" # non-existent class
as_hce(dat) # tries to convert to an hce object
## It still works because the inheritance converted it to a data frame
}
\seealso{
\code{\link[=as_hce]{as_hce()}}, \code{\link[=as_hce.data.frame]{as_hce.data.frame()}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stratWO_data_frame.R
\name{stratWO.data.frame}
\alias{stratWO.data.frame}
\title{Stratified win odds with adjustment}
\usage{
\method{stratWO}{data.frame}(
  x,
  AVAL,
  TRTP,
  STRATA,
  ref,
  COVAR = NULL,
  alpha = 0.05,
  WOnull = 1,
  ...
)
}
\arguments{
\item{x}{a data frame containing subject-level data.}

\item{AVAL}{variable in the data with ordinal analysis values.}

\item{TRTP}{the treatment variable in the data.}

\item{STRATA}{a character variable for stratification.}

\item{ref}{the reference treatment group.}

\item{COVAR}{a numeric covariate.}

\item{alpha}{the reference treatment group.}

\item{WOnull}{the null hypothesis. The default is 1.}

\item{...}{additional parameters.}
}
\value{
a data frame containing the following columns:
\itemize{
\item WO stratified (or adjusted/stratified) win odds.
\item LCL lower confidence limit for adjusted (or adjusted/stratified) WO.
\item UCL upper confidence limit for adjusted (or adjusted/stratified) WO.
\item SE standard error of the adjusted (or adjusted/stratified) win odds.
\item WOnull win odds of the null hypothesis (specified in the \code{WOnull} argument).
\item alpha two-sided significance level for calculating the confidence interval (specified in the \code{alpha} argument).
\item Pvalue p-value associated with testing the null hypothesis.
\item WP adjusted (or adjusted/stratified) win probability.
\item LCL_WP lower confidence limit for adjusted (or adjusted/stratified) WP.
\item UCL_WP upper confidence limit for adjusted (or adjusted/stratified) WP.
\item SE_WP standard error for the adjusted (or adjusted/stratified) win probability.
\item SD_WP standard deviation of the adjusted (or adjusted/stratified) win probability.
\item N total number of patients in the analysis.
\item Type "STRATIFIED" or "STRATIFIED/ADJUSTED" depending on whether \code{COVAR} is specified.
}
}
\description{
Stratified win odds with adjustment
}
\examples{
# Stratified win odds
res <- stratWO(x = KHCE, AVAL = "AVAL", TRTP = "TRTP", 
               STRATA = "STRATAN", ref = "P")
res
## Compare with the non-stratified win odds
res0 <- calcWO(AVAL ~ TRTP, data = KHCE,  ref = "P")
res0
## Compare with the win odds in each stratum separately
l <- lapply(split(KHCE, KHCE$STRATAN), calcWO, AVAL = "AVAL", TRTP = "TRTP", ref = "P")
l <- do.call(rbind, l)
l <- l[, c("WO", "LCL", "UCL", "N")]
l$STRATA <- as.numeric(row.names(l))
plot(y = l$WO, x = l$STRATA, ylim = c(0.5, 2.5), log = "y", xlim = c(0, 6),
      ylab = "Win Odds", xlab = "", xaxt = "n")
axis(1, at = 1:6, labels = c(paste0("STR = ", l$STRATA), "Stratified", "Non-stratified"))
arrows(l$STRATA, l$LCL, l$STRATA, 
       l$UCL, angle = 90, code = 3, length = 0.05, col = "darkgreen")
points(5, res$WO)
arrows(5, res$LCL, 5, res$UCL, angle = 90, code = 3, 
       length = 0.05, col = "darkblue")
abline(h = c(1, res$WO), col = "red", lty = 4)
points(6, res0$WO)
arrows(6, res0$LCL, 6, res0$UCL, angle = 90, code = 3, 
       length = 0.05, col = "darkred")
# Stratified and adjusted win odds
res <- stratWO(x = KHCE, AVAL = "AVAL", COVAR = "EGFRBL", 
              TRTP = "TRTP", STRATA = "STRATAN", ref = "P")
res

}
\references{
Gasparyan SB et al. (2021) "Adjusted win ratio with stratification: calculation methods and interpretation." Statistical Methods in Medical Research 30.2: 580-611. \url{doi:10.1177/0962280220942558}.
}
\seealso{
\code{\link[=stratWO]{stratWO()}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_results.R
\name{plot.hce_results}
\alias{plot.hce_results}
\title{A print method for \code{hce_results } objects}
\usage{
\method{plot}{hce_results}(x, ...)
}
\arguments{
\item{x}{an object of class \code{hce_results}.}

\item{...}{additional arguments to be passed to \code{\link[base:plot]{base::plot()}} function.}
}
\value{
no return value, called for plotting.
}
\description{
A print method for \code{hce_results } objects
}
\examples{
WO <- minWO(N = 100:1000)
plot(WO)
POW <- powerWO(N = 100:1000, WO = 1.2)
plot(POW, ylim = c(0, 1))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calcWINS_hce.R
\name{calcWINS.hce}
\alias{calcWINS.hce}
\title{Win statistics calculation for \code{hce} objects}
\usage{
\method{calcWINS}{hce}(x, ...)
}
\arguments{
\item{x}{an \code{hce} object.}

\item{...}{additional parameters.}
}
\value{
a list containing win statistics and their confidence intervals. It contains the following named data frames:
\itemize{
\item summary a data frame containing number of wins, losses, and ties of the active treatment group and the overall number of comparisons.
\item WP a data frame containing the win probability and its confidence interval.
\item NetBenefit a data frame containing the net benefit and its confidence interval. This is just a \verb{2x-1} transformation of WP and its CI.
\item WO a data frame containing the win odds and its confidence interval.
\item WR1 a data frame containing the win ratio and its confidence interval, using the transformed standard error of the \code{gamma} statistic.
\item WR2 a data frame containing the win ratio and its confidence interval, using the standard error calculated using \code{Pties}.
\item gamma a data frame containing Goodman Kruskal's \code{gamma} and its confidence interval.
\item SE a data frame containing standard errors used to calculated the Confidence intervals for win statistics.
}

When \code{SE_WP_Type = "unbiased"}, the \code{WP}, \code{WO} and \code{NetBenefit} estimators use the unbiased variance estimator of \code{WP}. Additionally, a Wilson-type range-preserving confidence interval is provided:
\itemize{
\item WP_W a data frame containing the win probability and its range-preserving confidence interval.
\item NetBenefit_W a data frame containing the net benefit and its range-preserving confidence interval.
\item WO_W a data frame containing the win odds and its range-preserving confidence interval.
}
}
\description{
Win statistics calculation for \code{hce} objects
}
\examples{
# Example 1
COVID19HCE <- hce(GROUP = COVID19$GROUP, TRTP = COVID19$TRTP)
calcWINS(COVID19HCE)
# Example 2
COVID19bHCE <- hce(GROUP = COVID19b$GROUP, TRTP = COVID19b$TRTP)
calcWINS(COVID19bHCE, ref = "Placebo", WOnull = 1.1, alpha = 0.01)
# Example 3
calcWINS(COVID19HCE, SE_WP_Type = "unbiased")$WP
calcWINS(COVID19HCE, SE_WP_Type = "biased")$WP
}
\references{
The theory of win statistics is covered in the following papers:
\itemize{
\item Win proportion and win odds confidence interval calculation:
\cr \cr Somers RH (1962) “A New Asymmetric Measure of Association for Ordinal Variables." American Sociological Review 27.6: 799-811. \url{doi:10.2307/2090408}.
\cr \cr Bamber D (1975) "The area above the ordinal dominance graph and the area below the receiver operating characteristic graph." Journal of Mathematical Psychology 12.4: 387-415. \url{doi:10.1016/0022-2496(75)90001-2}.
\cr \cr DeLong ER et al. (1988) "Comparing the Areas Under Two or More Correlated Receiver Operating Characteristic Curves: A Nonparametric Approach." Biometrics 44.3: 837-845. \url{doi:10.2307/2531595}.
\cr \cr Brunner E et al. (2021) "Win odds: an adaptation of the win ratio to include ties." Statistics in Medicine 40.14: 3367-3384. \url{doi:10.1002/sim.8967}.
\cr \cr Gasparyan SB et al. (2021) "Adjusted win ratio with stratification: calculation methods and interpretation." Statistical Methods in Medical Research 30.2: 580-611. \url{doi:10.1177/0962280220942558}.
\cr \cr Gasparyan SB et al. (2021) "Power and sample size calculation for the win odds test: application to an ordinal endpoint in COVID-19 trials." Journal of Biopharmaceutical Statistics 31.6: 765-787. \url{doi:10.1080/10543406.2021.1968893}.
\cr \cr Brunner E, Konietschke F. (2025) "An unbiased rank-based estimator of the Mann–Whitney variance including the case of ties." Statistical Papers 66.20. \url{doi:10.1007/s00362-024-01635-0}.
\item Win ratio: the first CI utilizes the standard error derived from the \code{gamma} statistic standard error as outlined by:
\cr \cr Gasparyan SB, Kowalewski EK, Buenconsejo J, Koch GG. (2023) "Hierarchical Composite Endpoints in COVID-19: The DARE-19 Trial." In Case Studies in Innovative Clinical Trials, Chapter 7, 95–148. Chapman; Hall/CRC. \url{doi:10.1201/9781003288640-7}.
\item Win ratio: the second CI utilizes the standard error presented by:
\cr \cr Yu RX, Ganju J. (2022) "Sample size formula for a win ratio endpoint." Statistics in Medicine 41.6: 950-63. \url{doi:10.1002/sim.9297}.
\item Goodman Kruskal's \code{gamma} and CI: matches implementation in \code{DescTools::GoodmanKruskalGamma()} and based on:
\cr \cr Agresti A. (2002) Categorical Data Analysis. John Wiley & Sons, pp. 57-59. \url{doi:10.1002/0471249688}.
\cr \cr Brown MB, Benedetti JK. (1977) "Sampling Behavior of Tests for Correlation in Two-Way Contingency Tables." Journal of the American Statistical Association 72, 309-315. \url{doi:10.1080/01621459.1977.10480995}.
\cr \cr Goodman LA, Kruskal WH. (1954) "Measures of association for cross classifications." Journal of the American Statistical Association 49, 732-764. \url{doi:10.1080/01621459.1954.10501231}.
\cr \cr Goodman LA, Kruskal WH. (1963) "Measures of association for cross classifications III: Approximate sampling theory." Journal of the American Statistical Association 58, 310-364. \url{doi:10.1080/01621459.1963.10500850}.
\item Unbiased variance estimator for \code{WP} and Wilson-type range -preserving confidence intervals are based on:
\cr \cr Brunner E, Konietschke F. (2025) "An unbiased rank-based estimator of the Mann–Whitney variance including the case of ties." Statistical Papers 66: 20. \url{doi:10.1007/s00362-024-01635-0}.
\cr \cr Schüürhuis S, Konietschke F, Brunner E. (2025) "A New Approach to the Nonparametric Behrens–Fisher Problem With Compatible Confidence Intervals." Biometrical Journal 67.6. \url{doi:10.1002/bimj.70096}.
}
}
\seealso{
\code{\link[=calcWINS]{calcWINS()}}, \code{\link[=calcWINS.formula]{calcWINS.formula()}}, \code{\link[=calcWINS.data.frame]{calcWINS.data.frame()}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sizeWO.R
\name{sizeWO}
\alias{sizeWO}
\title{Sample size calculation for the win odds test (no ties)}
\usage{
sizeWO(
  WO,
  power,
  SD = NULL,
  k = 0.5,
  alpha = 0.05,
  WOnull = 1,
  alternative = c("shift", "max", "ordered")
)
}
\arguments{
\item{WO}{a numeric vector of win odds values.}

\item{power}{the given power. A numeric vector of length 1.}

\item{SD}{assumed standard deviation of the win proportion. By default uses the conservative SD. A numeric vector of length 1.}

\item{k}{proportion of active group in the overall sample size. Default is 0.5 (balanced randomization). A numeric vector of length 1.}

\item{alpha}{the significance level for the 2-sided test. Default is 0.05. A numeric vector of length 1.}

\item{WOnull}{the win odds value of the null hypothesis (default is 1). A numeric vector of length 1.}

\item{alternative}{a character string specifying the class of the alternative hypothesis, must be one of \code{"shift"} (default), \code{"max"} or \code{"ordered"}. You can specify just the initial letter.}
}
\value{
a data frame containing the sample size with input values.
}
\description{
Sample size calculation for the win odds test (no ties)
}
\details{
\code{alternative = "max"} refers to the maximum variance of the win proportion across all possible
alternatives. The maximum variance equals \code{WP*(1 - WP)/k} where the win probability is calculated as \verb{WP = WO/(WO + 1).}
\code{alternative = "shift"} specifies the variance across alternatives from a shifted family of distributions (Wilcoxon test). The variance formula, as suggested by Noether, is calculated based on the null hypothesis as follows \verb{1/(12*k*(1 - k)).}
\code{alternative = "ordered"} specifies the variance across alternatives from stochastically ordered distributions which include shifted distributions.
}
\examples{
sizeWO(WO = 1.25, power = 0.9)
sizeWO(WO = 1.25, power = 0.9, k = 0.75)
sizeWO(WO = seq(1.05, 1.5, 0.05), power = 0.9)
# Comparison of different alternatives
x <- seq(1.05, 5, 0.05)
N1 <- sizeWO(WO = x, power = 0.9, alternative = "m")$SampleSize
N2 <- sizeWO(WO = x, power = 0.9, alternative = "o")$SampleSize
N3 <- sizeWO(WO = x, power = 0.9, alternative = "s")$SampleSize
d <- data.frame(WO = x, N_m = N1, N_o = N2, N_s = N3)
## Check the power for the ordered alternative
check <- c()
for(i in seq_along(x)){
check[i] <- powerWO(N = d[i, "N_o"], WO = d[i, "WO"], alternative = "o")$power
}
d$power_check_o <- check
print(d)
}
\references{
\itemize{
\item All formulas were presented in
\cr \cr Bamber D (1975) "The area above the ordinal dominance graph and the area below the receiver operating characteristic graph." Journal of Mathematical Psychology 12.4: 387-415. \url{doi:10.1016/0022-2496(75)90001-2}.
\item Noether's formula for shifted alternatives
\cr \cr Noether GE (1987) "Sample size determination for some common nonparametric tests." Journal of the American Statistical Association 82.398: 645-7. \url{doi:10.1080/01621459.1987.10478478}.
\item For shift alternatives see also
\cr \cr Gasparyan SB et al. (2021) "Power and sample size calculation for the win odds test: application to an ordinal endpoint in COVID-19 trials." Journal of Biopharmaceutical Statistics 31.6: 765-787. \url{doi:10.1080/10543406.2021.1968893}.
}
}
\seealso{
\code{\link[=powerWO]{powerWO()}}, \code{\link[=minWO]{minWO()}} for WO power or minimum detectable WO calculation.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calcWO.R
\name{calcWO}
\alias{calcWO}
\title{A generic function for calculating win odds}
\usage{
calcWO(x, ...)
}
\arguments{
\item{x}{an object used to select a method.}

\item{...}{further arguments passed to or from other methods.}
}
\value{
a data frame containing calculated values.
}
\description{
A generic function for calculating win odds
}
\seealso{
\code{\link[=calcWO.hce]{calcWO.hce()}}, \code{\link[=calcWO.formula]{calcWO.formula()}}, \code{\link[=calcWO.data.frame]{calcWO.data.frame()}}  methods.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/IWP.R
\name{IWP}
\alias{IWP}
\title{Calculates patient-level individual win proportions}
\usage{
IWP(data, AVAL, TRTP, ref)
}
\arguments{
\item{data}{a data frame containing subject-level data.}

\item{AVAL}{variable in the data with ordinal analysis values.}

\item{TRTP}{the treatment variable in the data.}

\item{ref}{the reference treatment group.}
}
\value{
the input data frame with a new column of individual win proportions named using the input \code{AVAL} value with \verb{_}.
}
\description{
Calculates patient-level individual win proportions
}
\examples{
KHCE1 <- IWP(data = KHCE, AVAL = "EGFRBL", TRTP = "TRTPN", ref = 2)
WP <- tapply(KHCE1$EGFRBL_, KHCE1$TRTPN, mean)
VAR <- tapply(KHCE1$EGFRBL_, KHCE1$TRTPN, function(x) (length(x)-1)*var(x)/length(x))
N <- tapply(KHCE1$EGFRBL_, KHCE1$TRTPN, length)
SE <- sqrt(sum(VAR/N))
c(WP = WP[[1]], SE = SE)
calcWO(EGFRBL ~ TRTP, data = KHCE)[c("WP", "SE_WP")]
}
\references{
Gasparyan SB et al. "Adjusted win ratio with stratification: calculation methods and interpretation." Statistical Methods in Medical Research 30.2 (2021): 580-611. \url{doi:10.1177/0962280220942558}.
}
\seealso{
\code{\link[=calcWO]{calcWO()}}, \code{\link[=calcWO.hce]{calcWO.hce()}}, \code{\link[=calcWO.formula]{calcWO.formula()}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as_hce.data.frame.R
\name{as_hce.data.frame}
\alias{as_hce.data.frame}
\title{Coerce a data frame to an \code{hce} object}
\usage{
\method{as_hce}{data.frame}(x, ...)
}
\arguments{
\item{x}{a data frame.}

\item{...}{additional parameters.}
}
\value{
an \code{hce} object.
}
\description{
Coerce a data frame to an \code{hce} object
}
\examples{
# The case when all required variables `AVAL0`, `GROUP`, `PADY`, and `TRTP` are present.
KHCE <- as_hce(KHCE)
## Converts to an `adhce` object
class(KHCE)
calcWO(KHCE)
# The case when only `AVAL` and `TRTP`.
## Converts to an `hce` object
dat <- KHCE[, c("TRTP", "AVAL")]
dat <- as_hce(dat)
class(dat)
summaryWO(dat)
}
\seealso{
\code{\link[=as_hce]{as_hce()}}, \code{\link[=as_hce.default]{as_hce.default()}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/powerWO.R
\name{powerWO}
\alias{powerWO}
\title{Power calculation for the win odds test (no ties)}
\usage{
powerWO(
  N,
  WO,
  SD = NULL,
  k = 0.5,
  alpha = 0.05,
  WOnull = 1,
  alternative = c("shift", "max", "ordered")
)
}
\arguments{
\item{N}{a numeric vector of sample size values.}

\item{WO}{the given win odds for the alternative hypothesis. A numeric vector of length 1.}

\item{SD}{assumed standard deviation of the win proportion. By default uses the conservative SD. A numeric vector of length 1.}

\item{k}{proportion of active group in the overall sample size. Default is 0.5 (balanced randomization). A numeric vector of length 1.}

\item{alpha}{the significance level for the 2-sided test. Default is 0.05. A numeric vector of length 1.}

\item{WOnull}{the win odds value of the null hypothesis (default is 1). A numeric vector of length 1.}

\item{alternative}{a character string specifying the class of the alternative hypothesis, must be one of \code{"shift"} (default), \code{"max"} or \code{"ordered"}. You can specify just the initial letter.}
}
\value{
a data frame containing the calculated power with input values.
}
\description{
Power calculation for the win odds test (no ties)
}
\details{
\code{alternative = "max"} refers to the maximum variance of the win proportion across all possible
alternatives. The maximum variance equals \code{WP*(1 - WP)/k} where the win probability is calculated as \verb{WP = WO/(WO + 1).}
\code{alternative = "shift"} specifies the variance across alternatives from a shifted family of distributions (Wilcoxon test). The variance formula, as suggested by Noether, is calculated based on the null hypothesis as follows \verb{1/(12*k*(1 - k)).}
\code{alternative = "ordered"} specifies the variance across alternatives from stochastically ordered distributions which include shifted distributions.
}
\examples{
# Example 1- Use the default standard deviation
powerWO(N = 1000, WO = 1.2)
powerWO(N = seq(500, 1500, 100), WO = 1.2)
# Example 2 - Use data-driven win odds and standard deviation from the COVID19 dataset
res <- calcWO(x = COVID19, AVAL = "GROUP", TRTP = "TRTP", ref = "Placebo")
print(res)
powerWO(N = 500, WO = res$WO, SD = res$SD_WP)
powerWO(N = 500, WO = res$WO) # power with the default standard deviation for the win proportion.
# Example 3 - Non-balanced 3:1 randomization
powerWO(N = 1000, WO = 1.2, k = 0.75)
# Example 4 - Comparison of different alternatives
powerWO(N = 1000, WO = 1.2, alternative = "m")
powerWO(N = 1000, WO = 1.2, alternative = "s")
powerWO(N = 1000, WO = 1.2, alternative = "o")
}
\references{
\itemize{
\item All formulas were presented in
\cr \cr Bamber D (1975) "The area above the ordinal dominance graph and the area below the receiver operating characteristic graph." Journal of Mathematical Psychology 12.4: 387-415. \url{doi:10.1016/0022-2496(75)90001-2}.
\item Noether's formula for shifted alternatives
\cr \cr Noether GE (1987) "Sample size determination for some common nonparametric tests." Journal of the American Statistical Association 82.398: 645-7. \url{doi:10.1080/01621459.1987.10478478}.
\item For shift alternatives see also
\cr \cr Gasparyan SB et al. (2021) "Power and sample size calculation for the win odds test: application to an ordinal endpoint in COVID-19 trials." Journal of Biopharmaceutical Statistics 31.6: 765-787. \url{doi:10.1080/10543406.2021.1968893}.
}
}
\seealso{
\code{\link[=sizeWO]{sizeWO()}}, \code{\link[=minWO]{minWO()}} for WO sample size or minimum detectable WO calculation.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{ADET}
\alias{ADET}
\title{Event-Time dataset for kidney outcomes.}
\format{
a data frame with 604 rows (events) and 6 variables:
\describe{
\item{ID}{patient identifiers, numeric}
\item{AVAL}{occurence time of the event, numeric}
\item{PARAM}{name of the event, character}
\item{PARAMCD}{coded name of the event, character}
\item{PARAMN}{type of the event, outcomes 1-7, where a higher value means a better outcome, numeric}
\item{TRTPN}{treatment values, 1 Active or 2 Placebo, numeric}
}
}
\source{
Heerspink HL et al "Development and validation of a new hierarchical composite endpoint for clinical trials of kidney disease progression." Journal of the American Society of Nephrology (2023): \url{doi:10.1681/ASN.0000000000000243}.
}
\usage{
ADET
}
\description{
A dataset with multiple kidney outcomes over time scale outcomes of 1500 patients in the \code{ADSL} dataset.
}
\examples{
head(ADET)
# Number of unique patients
length(unique(ADET$ID)) 
# Number of events per event type
barplot(table(ADET$PARAM))
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as_hce.R
\name{as_hce}
\alias{as_hce}
\title{A generic function for coercing data structures to \code{hce} objects}
\usage{
as_hce(x, ...)
}
\arguments{
\item{x}{an object used to select a method.}

\item{...}{additional parameters.}
}
\value{
an \code{hce} object.
}
\description{
A generic function for coercing data structures to \code{hce} objects
}
\examples{
### data frames 
data(HCE1)
HCE <- as_hce(HCE1)
calcWINS(HCE)
}
\seealso{
\code{\link[=as_hce.data.frame]{as_hce.data.frame()}}, \code{\link[=as_hce.default]{as_hce.default()}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deltaWO.R
\name{deltaWO}
\alias{deltaWO}
\title{A generic function for calculating win odds based on a threshold}
\usage{
deltaWO(x, delta, ...)
}
\arguments{
\item{x}{an object used to select a method.}

\item{delta}{a numeric threshold.}

\item{...}{further arguments passed to or from other methods.}
}
\value{
a data frame containing calculated values.
}
\description{
A generic function for calculating win odds based on a threshold
}
\seealso{
\code{\link[=deltaWO.adhce]{deltaWO.adhce()}} method.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{ADLB}
\alias{ADLB}
\title{Laboratory dataset for Glomerular Filtration Rate (GFR) measurements.}
\format{
a data frame with 13980 rows and 8 variables:
\describe{
\item{ID}{patient identifiers, numeric}
\item{TRTPN}{treatment values, 1 Active or 2 Placebo, numeric}
\item{AVAL}{measurement value, numeric}
\item{ADAY}{measurement day in the study, numeric}
\item{AVISITN}{hospital visit number, numeric}
\item{PARAM}{name of the event, GFR measurements, character}
\item{PARAMCD}{coded name of the event, GFR, character}
\item{PARAMN}{type of the event is set to 7 for all measurements, numeric}
}
}
\source{
Heerspink HL et al "Development and validation of a new hierarchical composite endpoint for clinical trials of kidney disease progression." Journal of the American Society of Nephrology (2023): \url{doi:10.1681/ASN.0000000000000243}.
}
\usage{
ADLB
}
\description{
A dataset of laboratory measurements of kidney function over time for the 1500 patients in the \code{ADSL} dataset.
}
\examples{
head(ADLB)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{ADSL}
\alias{ADSL}
\title{Baseline characteristics dataset of patients with kidney function assessments.}
\format{
a data frame with 1500 rows and 4 variables:
\describe{
\item{ID}{patient identifiers, numeric}
\item{TRTPN}{treatment values, 1 Active or 2 Placebo, numeric}
\item{EGFRBL}{Baseline GFR values of patients, numeric}
\item{STRATAN}{strata 1-4, higher value means a higher risk for kidney disease progression, numeric}
}
}
\source{
Heerspink HL et al "Development and validation of a new hierarchical composite endpoint for clinical trials of kidney disease progression." Journal of the American Society of Nephrology (2023): \url{doi:10.1681/ASN.0000000000000243}.
}
\usage{
ADSL
}
\description{
A data frame with baseline characteristics for 1500 patients used to derive \code{KHCE} dataset.
}
\examples{
head(ADSL)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simORD.R
\name{simORD}
\alias{simORD}
\title{Simulate ordinal variables for two treatment groups using categorization of beta distributions}
\usage{
simORD(n, n0 = n, M, alpha1 = 8, beta1 = 7, alpha0 = 5, beta0 = 5)
}
\arguments{
\item{n}{sample size in the active treatment group.}

\item{n0}{sample size in the placebo group.}

\item{M}{number of ordinal values to be simulated.}

\item{alpha1}{shape1 parameter for the beta distribution in the active group.}

\item{beta1}{shape2 parameter for the beta distribution in the active group.}

\item{alpha0}{shape1 parameter for the beta distribution in the placebo group.}

\item{beta0}{shape2 parameter for the beta distribution in the placebo group.}
}
\value{
a data frame containing the following columns:
\itemize{
\item ID subject identifier.
\item TRTP planned treatment group - "A" for active, "P" for Placebo.
\item GROUPN ordinal values. The number of unique values is specified by the variable \code{M0}.
\item tau the theoretical win odds.
\item theta the theoretical win probability.
}
}
\description{
Simulate ordinal variables for two treatment groups using categorization of beta distributions
}
\examples{
# Example 1
set.seed(2024)
alpha1 <- 8
beta1 <- 8
alpha0 <- 4
beta0 <- 5
d <- simORD(n = 1500, n0 = 1500, M = 5, alpha1 = alpha1, beta1 = beta1, 
alpha0 = alpha0, beta0 = beta0)
x <- seq(0, 1, 0.01)
plot(x, dbeta(x, shape1 = alpha1, shape2 = beta1), 
type = "l", ylab = "Density of beta distribution", col = 2)
lines(x, dbeta(x, shape1 = alpha0, shape2 = beta0), col = 3, lty = 2)
legend("topleft", lty = c(1, 2), col = c(2, 3), legend = c("Control", "Active"))
D <- hce(GROUP = d$GROUPN, TRTP = d$TRTP)
table(D$TRTP, D$GROUP)
calcWO(D)
# Example 2
set.seed(2024)
d <- simORD(n = 100, n0 = 50, M = 2)
d_hce <- hce(GROUP = d$GROUPN, TRTP = d$TRTP)
calcWO(d_hce)
### compare with the theoretical values of the continuous distributions 
c(tau = unique(d$tau), theta = unique(d$theta))
# Example 2 - Convergence of the win odds to its theoretical value
set.seed(2024)
N <- NULL
size <- c(seq(10, 500, 1))
for(i in size){
  d <- simORD(n = i, M = 2)
  d_hce <- hce(GROUP = d$GROUPN, TRTP = d$TRTP)
  TAU <- calcWO(d_hce)
  D <- data.frame(WO = TAU$WO, n = i, tau = unique(d$tau))
  N <- rbind(N, D)
}
plot(N$n, N$WO, log = "y", ylim = c(0.5, 2), ylab = "Win Odds", xlab = "Sample size", type = "l")
lines(N$n, N$tau, col = "darkgreen", lty = 2, lwd = 2)
abline(h = 1, lty = 4, col = "red")
legend("bottomright", legend = c("Theoretical Win Odds", "Null", "Win Odds Estimate"), 
lty = c(4, 2, 1), col = c("darkgreen", "red", "black"))
title("Convergence of the win odds to its theoretical value")
}
\seealso{
\code{\link[=simHCE]{simHCE()}} for simulating \code{hce} objects.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/regWO_data_frame.R
\name{regWO.data.frame}
\alias{regWO.data.frame}
\title{Win Odds Regression Using a Data Frame}
\usage{
\method{regWO}{data.frame}(x, AVAL, TRTP, COVAR, ref, alpha = 0.05, WOnull = 1, ...)
}
\arguments{
\item{x}{a data frame containing subject-level data.}

\item{AVAL}{a variable in the data with ordinal analysis values.}

\item{TRTP}{the treatment variable in the data.}

\item{COVAR}{a numeric covariate.}

\item{ref}{the reference treatment group.}

\item{alpha}{the significance level, with a default value of 0.05.}

\item{WOnull}{the null hypothesis value for win odds. The default is 1.}

\item{...}{additional parameters.}
}
\value{
a data frame containing the calculated win odds and its confidence interval, including:
\itemize{
\item WO_beta adjusted win odds.
\item LCL lower confidence limit for adjusted WO.
\item UCL upper confidence limit for adjusted WO.
\item SE standard error of the adjusted win odds.
\item WOnull win odds of the null hypothesis (specified in the \code{WOnull} argument).
\item alpha two-sided significance level for calculating the confidence interval (specified in the \code{alpha} argument).
\item Pvalue p-value associated with testing the null hypothesis.
\item N total number of patients in the analysis.
\item beta adjusted win probability.
\item LCL_beta lower confidence limit for adjusted win probability.
\item UCL_beta upper confidence limit for adjusted win probability.
\item SE_beta standard error for the adjusted win probability.
\item SD_beta standard deviation for the adjusted win probability.
\item WP (non-adjusted) win probability.
\item SE_WP standard error of the non-adjusted win probability.
\item SD_WP standard deviation of the non-adjusted win probability.
\item WO non-adjusted win odds.
\item COVAR_MEAN_DIFF mean difference between two treatment groups of the numeric covariate.
\item COVAR_VAR sum of variances of two treatment groups of the numeric covariate.
\item COVAR_COV covariance between the response and the numeric covariate.
}
}
\description{
This function performs regression analysis for the win odds using a single numeric covariate.
}
\examples{
# A baseline covariate that is highly correlated with the outcome
set.seed(2023)
dat <- COVID19
n <- nrow(dat)
dat$Severity <- ifelse(dat$GROUP > 4, rnorm(n, 0), rnorm(n, 100))
tapply(dat$Severity, dat$TRTP, mean)
regWO(x = dat, AVAL = "GROUP", TRTP = "TRTP", COVAR = "Severity", ref = "Placebo")
# Without adjustment
calcWO(x = dat, AVAL = "GROUP", TRTP = "TRTP", ref = "Placebo")
}
\references{
Gasparyan SB et al. (2021) "Adjusted win ratio with stratification: calculation methods and interpretation." Statistical Methods in Medical Research 30.2: 580-611. \url{doi:10.1177/0962280220942558}.
}
\seealso{
\code{\link[=regWO]{regWO()}}, \code{\link[=regWO.formula]{regWO.formula()}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rweibullGF.R
\name{rweibullGF}
\alias{rweibullGF}
\title{Simulate random numbers from a Weibull distribution with gamma frailty}
\usage{
rweibullGF(n, rate, shape = 1, theta = 1)
}
\arguments{
\item{n}{number of observations.}

\item{rate}{rate parameter of the Weibull distribution.}

\item{shape}{shape parameter of the Weibull distribution, with a default value of 1 (exponential).}

\item{theta}{variance parameter of the gamma frailty distribution, with a default value of 1.}
}
\value{
a vector of random numbers of length \code{n}.
}
\description{
Simulate random numbers from a Weibull distribution with gamma frailty
}
\details{
Let \eqn{\gamma} denote a frailty term following a gamma distribution with
\eqn{shape = 1 / \theta} and \eqn{scale = \theta}. Then \eqn{\gamma} has mean 1
and variance \eqn{\theta}.

Conditional on \eqn{\gamma}, the survival function is
\deqn{
S(t \mid \lambda, \alpha, \gamma) =
\exp\left(-\gamma \lambda t^\alpha\right), \qquad \alpha > 0,\ \lambda > 0,
}
where \eqn{\alpha} is the shape parameter and \eqn{\lambda} is the rate parameter.

Integrating over the gamma frailty distribution gives the marginal survival
function
\deqn{
S(t \mid \lambda, \alpha, \theta) =
\left(1 + \theta \lambda t^\alpha\right)^{-1 / \theta},
\qquad \alpha > 0,\ \lambda > 0,\ \theta > 0.
}
This corresponds to the generalized log-logistic distribution (GLL) as implemented in \code{\link[=GLL]{GLL()}}.

As \eqn{\theta \to 0}, this reduces to the standard Weibull distribution
without frailty, with survival function
\deqn{
S(t \mid \lambda, \alpha) = \exp\left(-\lambda t^\alpha\right).
}
This corresponds to the Weibull distribution in \code{\link[stats:pweibull]{stats::pweibull()}} with \code{shape = shape} and \code{scale = rate^(-1 / shape)}.

When \eqn{\theta = 1}, the survival function becomes
\deqn{
S(t \mid \lambda, \alpha, 1) = \left(1 + \lambda t^\alpha\right)^{-1},
}
which is the survival function of a log-logistic distribution.
}
\examples{
## Example 1: each patient has a sampled time with potentially different
## shape, rate, and theta values.
set.seed(123)
n <- 1000
d <- data.frame(ID = 1:(2*n), TRTP = rep(c("A", "P"), each = n), 
rate = rep(c(0.1, 0.15), each = n), shape = 1.5, theta = 1)
d$time1 <- rweibullGF(n = 2*n, rate = d$rate, shape = d$shape, theta = d$theta)
tapply(d$time1, d$TRTP, summary)
## Example 2: all patients share the same parameter values.
d$time2 <- rweibullGF(n = 2*n, rate = 0.1, theta = 0)
tapply(d$time2, d$TRTP, summary)
}
\references{
Wienke A. "Frailty Models in Survival Analysis." Chapman and Hall/CRC (2010).
}
\seealso{
\code{\link[=simTTE]{simTTE()}} for simulation of a two-event time-to-event endpoint
from this distribution under the illness-death model.

\code{\link[=pGLL]{pGLL()}} provides the cumulative distribution function of the
generalized log-logistic distribution.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stratWO.R
\name{stratWO}
\alias{stratWO}
\title{A generic function for stratified win odds with adjustment}
\usage{
stratWO(x, ...)
}
\arguments{
\item{x}{an object used to select a method.}

\item{...}{further arguments passed to or from other methods.}
}
\value{
a list containing the stratified results and results by strata.
}
\description{
A generic function for stratified win odds with adjustment
}
\seealso{
\code{\link[=stratWO.data.frame]{stratWO.data.frame()}}  methods.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_hce.R
\name{plot.hce}
\alias{plot.hce}
\title{A plot method for \code{hce} objects}
\usage{
\method{plot}{hce}(x, fill = FALSE, ...)
}
\arguments{
\item{x}{an object of class \code{hce} created by \code{as_hce()}.}

\item{fill}{logical; if \code{TRUE} fill the area above the graph.}

\item{...}{additional arguments to be passed to \code{\link[base:plot]{base::plot()}} function.}
}
\value{
no return value, called for plotting.
}
\description{
Ordinal dominance graph for \code{hce} objects
}
\examples{
d <- as_hce(KHCE)
d$TRTP <- factor(d$TRTP, levels = c("P", "A"))
res <- calcWO(AVAL ~ TRTP, data = d)
# Ordinal Dominance Graph
plot(d, col = 3, type = 'l')
grid()
# Area above the Ordinal Dominance Graph
plot(d, fill = TRUE, col = "#865A4F", type = 'l', 
     lwd = 2, xlab = "Control", ylab = "Active")
legend("bottomright", legend = paste0("WP = ", round(res$WP, 5)))
abline(a = 0, b = 1, lwd = 2, lty = 2, col = "#999999")
}
\references{
Bamber D. "The area above the ordinal dominance graph and the area below the receiver operating characteristic graph." Journal of Mathematical Psychology 12.4 (1975): 387-415. \url{doi:10.1016/0022-2496(75)90001-2}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deltaWO_adhce.R
\name{deltaWO.adhce}
\alias{deltaWO.adhce}
\title{Win odds calculation based on a threshold for \code{adhce} objects}
\usage{
\method{deltaWO}{adhce}(x, delta, ref = unique(x$TRTP)[1], alpha = 0.05, WOnull = 1, ...)
}
\arguments{
\item{x}{an \code{adhce} object.}

\item{delta}{a numeric threshold.}

\item{ref}{the reference treatment group.}

\item{alpha}{significance level. The default is 0.05.}

\item{WOnull}{the null hypothesis. The default is 1.}

\item{...}{additional parameters.}
}
\value{
a data frame containing the win odds and its confidence interval. It contains the following columns:
\itemize{
\item WO calculated win odds.
\item LCL lower confidence limit.
\item UCL upper confidence limit.
\item SE standard error of the win odds.
\item WOnull win odds of the null hypothesis (specified in the \code{WOnull} argument).
\item alpha two-sided significance level for calculating the confidence interval (specified in the \code{alpha} argument).
\item Pvalue p-value associated with testing the null hypothesis.
\item WP calculated win probability.
\item LCL_WP lower confidence limit for \code{WP}.
\item UCL_WP upper confidence limit for \code{WP}.
\item SE_WP standard error of the win probability.
\item SD_WP standard deviation of the win probability, calculated as \code{SE_WP} multiplied by \code{sqrt(N)}.
\item N total number of patients in the analysis.
\item ref the reference group.
\item delta the threshold.
}
}
\description{
Win odds calculation based on a threshold for \code{adhce} objects
}
\examples{
# Example using the kidney dataset
dat <- as_hce(KHCE)
calcWO(dat, ref = "P")
## The exact same result
deltaWO(dat, delta = 0, ref = "P")
## A large threshold
deltaWO(dat, delta = 10, ref = "P")

}
\seealso{
\code{\link[=deltaWO]{deltaWO()}}, \code{\link[=calcWO]{calcWO()}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calcWINS_data_frame.R
\name{calcWINS.data.frame}
\alias{calcWINS.data.frame}
\title{Win statistics calculation using a data frame}
\usage{
\method{calcWINS}{data.frame}(
  x,
  AVAL,
  TRTP,
  ref,
  alpha = 0.05,
  WOnull = 1,
  SE_WP_Type = c("biased", "unbiased"),
  ...
)
}
\arguments{
\item{x}{a data frame containing subject-level data.}

\item{AVAL}{variable in the data with ordinal analysis values.}

\item{TRTP}{the treatment variable in the data.}

\item{ref}{the reference treatment group.}

\item{alpha}{2-sided significance level. The default is 0.05.}

\item{WOnull}{the null hypothesis. The default is 1.}

\item{SE_WP_Type}{biased or unbiased standard error for win probability. The default is biased.}

\item{...}{additional parameters.}
}
\value{
a list containing win statistics and their confidence intervals. It contains the following named data frames:
\itemize{
\item summary a data frame containing number of wins, losses, and ties of the active treatment group and the overall number of comparisons.
\item WP a data frame containing the win probability and its confidence interval.
\item NetBenefit a data frame containing the net benefit and its confidence interval. This is just a \verb{2x-1} transformation of WP and its CI.
\item WO a data frame containing the win odds and its confidence interval.
\item WR1 a data frame containing the win ratio and its confidence interval, using the transformed standard error of the \code{gamma} statistic.
\item WR2 a data frame containing the win ratio and its confidence interval, using the standard error calculated using \code{Pties}.
\item gamma a data frame containing Goodman Kruskal's \code{gamma} and its confidence interval.
\item SE a data frame containing standard errors used to calculated the Confidence intervals for win statistics.
}

When \code{SE_WP_Type = "unbiased"}, the \code{WP}, \code{WO} and \code{NetBenefit} estimators use the unbiased variance estimator of \code{WP}. Additionally, a Wilson-type range-preserving confidence interval is provided:
\itemize{
\item WP_W a data frame containing the win probability and its range-preserving confidence interval.
\item NetBenefit_W a data frame containing the net benefit and its range-preserving confidence interval.
\item WO_W a data frame containing the win odds and its range-preserving confidence interval.
}
}
\description{
Win statistics calculation using a data frame
}
\details{
When \code{SE_WP_Type = "unbiased"}, the calculations for win proportion, net benefit, and win odds utilize the unbiased standard error from Brunner-Konietschke (2025) paper which is a reformulation of the original formula proposed by Bamber (1975). In this case, Wilson-type confidence intervals are calculated for the win probability, net benefit, and win odds, following the approach proposed by Schüürhuis, Konietschke, and Brunner (2025).
}
\examples{
# Example 1 - Simple use
calcWINS(x = COVID19b, AVAL = "GROUP", TRTP = "TRTP", ref = "Placebo")
calcWINS(x = COVID19b, AVAL = "GROUP", TRTP = "TRTP", ref = "Placebo", SE_WP_Type = "unbiased")
# Example 2 - Different variance estimators  
FREQ <- c(16, 5, 0, 1, 0, 4, 1, 5, 7, 2)
dat0 <- data.frame(AVAL = rep(5:1, 2), TRTP = rep(c('A', 'P'), each = 5))
dat <- dat0[rep(row.names(dat0), FREQ),]
## By default, the variance estimator applies a 1/n weighting to the coefficients
## This approach matches the Somers' D (C|R) estimator, where `C|R` indicates that 
## the column variable Y is treated as the independent variable and the row
## variable X is treated as the dependent variable.
calcWINS(AVAL ~ TRTP, data = dat)$WP
## The Brunner-Konietschke estimator
UNB <- calcWINS(AVAL ~ TRTP, data = dat,  SE_WP_Type = "unbiased")
cbind(UNB$WP, SE = UNB$SE$WP_SE)
## The Brunner-Munzel test, based on the DeLong-Clarke-Pearson formula for the variance estimation,
## applies 1/(n - 1) weighting to the coefficients.
dat1 <- IWP(data = dat, AVAL = "AVAL", TRTP = "TRTP", ref = "P")
WP <- tapply(dat1$AVAL_, dat1$TRTP, mean)
VAR <- tapply(dat1$AVAL_, dat1$TRTP, var)
N <- tapply(dat1$AVAL_, dat1$TRTP, length)
SE <- sqrt(sum(VAR/N))
c(WP = WP[[1]], SE = SE)
# Example 3 - Simulations: Biased vs unbiased vs Wilson confidence intervals for Win Probability
set.seed(1)
n0 <- 5; n1 <- 7; p0 <- 0.2; p1 <- 0.5; x <- 1:20; delta <- 0.15
WP0 <- (p1 - p0)/2 + 0.5
DAT <- NULL
for(i in x){
  dat <- data.frame(AVAL = c(rbinom(n1, size = 1, p1), rbinom(n0, size = 1, p0)), 
  TRTP = c(rep("A", n1), rep("P", n0)))
  CL1 <- calcWINS(x = dat, AVAL = "AVAL", TRTP = "TRTP", ref = "P")$WP
  CL1$Type <- "biased"
  fit <- calcWINS(x = dat, AVAL = "AVAL", TRTP = "TRTP", 
                ref = "P", SE_WP_Type = "unbiased")
  CL2 <- fit$WP
  CL2$Type <- "unbiased"
  CL3 <- fit$WP_W
  CL3$Type <- "Wilson"
  DAT <- rbind(DAT, CL1, CL2, CL3)
  }
WP <- DAT$WP[DAT$Type == "unbiased"]
plot(x, WP, pch = 19, xlab = "Simulations", ylab = "Win Probability", 
            ylim = c(0., 1.1), xlim = c(0, max(x) + 1))
points(x + delta, WP, pch = 19)
points(x + 2*delta, WP, pch = 19)
arrows(x, DAT$LCL[DAT$Type == "unbiased"], 
       x, DAT$UCL[DAT$Type == "unbiased"], angle = 90, code = 3, length = 0.05, "green")
       arrows(x + delta, DAT$LCL[DAT$Type == "biased"], 
       x + delta, DAT$UCL[DAT$Type == "biased"], angle = 90, code = 3, 
       length = 0.05, col = "orange")
arrows(x + 2*delta, DAT$LCL[DAT$Type == "Wilson"], 
       x + 2*delta, DAT$UCL[DAT$Type == "Wilson"], angle = 90, code = 3, 
       length = 0.05, col = "blue")
abline(h = c(WP0, 1), col = c("darkgreen", "darkred"), lty = c(3, 4))
legend("bottomleft", legend = c("True WP", "UnBiased", "Biased", "Wilson", "Null"), 
       col = c("darkgreen", "green", "orange", "blue", "darkred"), 
       lty = c(3, 1, 1, 1, 4), cex = 0.75, ncol = 3)
       title("Win Probability: Biased vs Unbiased vs Wilson CI")
# End of Example 3
}
\references{
The theory of win statistics is covered in the following papers:
\itemize{
\item Win proportion and win odds confidence interval calculation:
\cr \cr Somers RH (1962) “A New Asymmetric Measure of Association for Ordinal Variables." American Sociological Review 27.6: 799-811. \url{doi:10.2307/2090408}.
\cr \cr Bamber D (1975) "The area above the ordinal dominance graph and the area below the receiver operating characteristic graph." Journal of Mathematical Psychology 12.4: 387-415. \url{doi:10.1016/0022-2496(75)90001-2}.
\cr \cr DeLong ER et al. (1988) "Comparing the Areas Under Two or More Correlated Receiver Operating Characteristic Curves: A Nonparametric Approach." Biometrics 44.3: 837-845. \url{doi:10.2307/2531595}.
\cr \cr Brunner E et al. (2021) "Win odds: an adaptation of the win ratio to include ties." Statistics in Medicine 40.14: 3367-3384. \url{doi:10.1002/sim.8967}.
\cr \cr Gasparyan SB et al. (2021) "Adjusted win ratio with stratification: calculation methods and interpretation." Statistical Methods in Medical Research 30.2: 580-611. \url{doi:10.1177/0962280220942558}.
\cr \cr Gasparyan SB et al. (2021) "Power and sample size calculation for the win odds test: application to an ordinal endpoint in COVID-19 trials." Journal of Biopharmaceutical Statistics 31.6: 765-787. \url{doi:10.1080/10543406.2021.1968893}.
\cr \cr Brunner E, Konietschke F. (2025) "An unbiased rank-based estimator of the Mann–Whitney variance including the case of ties." Statistical Papers 66.20. \url{doi:10.1007/s00362-024-01635-0}.
\item Win ratio: the first CI utilizes the standard error derived from the \code{gamma} statistic standard error as outlined by:
\cr \cr Gasparyan SB, Kowalewski EK, Buenconsejo J, Koch GG. (2023) "Hierarchical Composite Endpoints in COVID-19: The DARE-19 Trial." In Case Studies in Innovative Clinical Trials, Chapter 7, 95–148. Chapman; Hall/CRC. \url{doi:10.1201/9781003288640-7}.
\item Win ratio: the second CI utilizes the standard error presented by:
\cr \cr Yu RX, Ganju J. (2022) "Sample size formula for a win ratio endpoint." Statistics in Medicine 41.6: 950-63. \url{doi:10.1002/sim.9297}.
\item Goodman Kruskal's \code{gamma} and CI: matches implementation in \code{DescTools::GoodmanKruskalGamma()} and based on:
\cr \cr Agresti A. (2002) Categorical Data Analysis. John Wiley & Sons, pp. 57-59. \url{doi:10.1002/0471249688}.
\cr \cr Brown MB, Benedetti JK. (1977) "Sampling Behavior of Tests for Correlation in Two-Way Contingency Tables." Journal of the American Statistical Association 72, 309-315. \url{doi:10.1080/01621459.1977.10480995}.
\cr \cr Goodman LA, Kruskal WH. (1954) "Measures of association for cross classifications." Journal of the American Statistical Association 49, 732-764. \url{doi:10.1080/01621459.1954.10501231}.
\cr \cr Goodman LA, Kruskal WH. (1963) "Measures of association for cross classifications III: Approximate sampling theory." Journal of the American Statistical Association 58, 310-364. \url{doi:10.1080/01621459.1963.10500850}.
\item Unbiased variance estimator for \code{WP} and Wilson-type range -preserving confidence intervals are based on:
\cr \cr Brunner E, Konietschke F. (2025) "An unbiased rank-based estimator of the Mann–Whitney variance including the case of ties." Statistical Papers 66: 20. \url{doi:10.1007/s00362-024-01635-0}.
\cr \cr Schüürhuis S, Konietschke F, Brunner E. (2025) "A New Approach to the Nonparametric Behrens–Fisher Problem With Compatible Confidence Intervals." Biometrical Journal 67.6. \url{doi:10.1002/bimj.70096}.
}
}
\seealso{
\code{\link[=calcWINS]{calcWINS()}}, \code{\link[=calcWINS.hce]{calcWINS.hce()}}, \code{\link[=calcWINS.formula]{calcWINS.formula()}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GLL.R
\name{GLL}
\alias{GLL}
\alias{dGLL}
\alias{pGLL}
\alias{qGLL}
\alias{rGLL}
\alias{hGLL}
\alias{HGLL}
\title{The Generalized Log-Logistic Distribution}
\usage{
dGLL(x, rate, shape = 1, theta = 0, log = FALSE)

pGLL(q, rate, shape = 1, theta = 0)

qGLL(p, rate, shape = 1, theta = 0)

rGLL(n, rate, shape = 1, theta = 0)

hGLL(x, rate, shape = 1, theta = 0)

HGLL(x, rate, shape = 1, theta = 0)
}
\arguments{
\item{x, q}{vector of quantiles.}

\item{rate}{positive rate parameter of the distribution.}

\item{shape}{positive shape parameter of the distribution.}

\item{theta}{non-negative additional parameter of the distribution, with default
value \code{0}.}

\item{log}{logical; if \code{TRUE}, densities are given as logarithms.}

\item{p}{vector of probabilities.}

\item{n}{number of observations. Must be a single non-negative integer.}
}
\value{
For \code{dGLL()}, \code{pGLL()}, \code{qGLL()}, \code{hGLL()}, and \code{HGLL()}, a numeric vector of
the same length as the main input (\code{x}, \code{q}, or \code{p}). For \code{rGLL()}, a numeric
vector of length \code{n}.
}
\description{
Density, distribution function, quantile function, hazard, cumulative hazard,
and random generation for the generalized log-logistic distribution.
}
\details{
The generalized log-logistic distribution with rate parameter \eqn{\lambda > 0},
shape parameter \eqn{\alpha > 0}, and parameter \eqn{\theta \ge 0} has survival
function
\deqn{
S(t \mid \lambda, \alpha, \theta) =
\left(1 + \theta \lambda t^\alpha\right)^{-1 / \theta},
\qquad \theta > 0.
}

The corresponding cumulative distribution function is
\deqn{
F(t \mid \lambda, \alpha, \theta) =
1 - \left(1 + \theta \lambda t^\alpha\right)^{-1 / \theta}.
}

The quantile function is
\deqn{
Q(p \mid \lambda, \alpha, \theta) =
\left(
\frac{(1 - p)^{-\theta} - 1}{\theta \lambda}
\right)^{1 / \alpha},
\qquad 0 \le p \le 1,\ \theta > 0.
}

The cumulative hazard function is
\deqn{
H(t \mid \lambda, \alpha, \theta) =
\frac{\log\left(1 + \theta \lambda t^\alpha\right)}{\theta}.
}

The density function is
\deqn{
f(t \mid \lambda, \alpha, \theta) =
\lambda \alpha t^{\alpha - 1}
\left(1 + \theta \lambda t^\alpha\right)^{-1 / \theta - 1}.
}

For \code{dGLL(..., log = TRUE)}, the logarithm of the density is returned. For
\eqn{\theta > 0},
\deqn{
\log f(t \mid \lambda, \alpha, \theta) =
\log(\lambda) + \log(\alpha) + (\alpha - 1)\log(t)
- \left(\frac{1}{\theta} + 1\right)\log\left(1 + \theta \lambda t^\alpha\right).
}

The hazard function is
\deqn{
h(t \mid \lambda, \alpha, \theta) =
\frac{\lambda \alpha t^{\alpha - 1}}
{1 + \theta \lambda t^\alpha}.
}

Random generation is performed by inverse transform sampling. If
\eqn{U \sim \mathrm{Uniform}(0,1)}, then
\deqn{
T = Q(U \mid \lambda, \alpha, \theta)
}
has the generalized log-logistic distribution.

As \eqn{\theta \to 0}, this reduces to the Weibull distribution with survival
function
\deqn{
S(t \mid \lambda, \alpha) = \exp\left(-\lambda t^\alpha\right).
}
The corresponding quantile function is
\deqn{
Q(p \mid \lambda, \alpha) =
\left(
\frac{-\log(1 - p)}{\lambda}
\right)^{1 / \alpha}.
}
In this case, \code{dGLL(..., log = TRUE)} returns the corresponding Weibull
log-density.
This corresponds to the Weibull distribution in \code{\link[stats:dweibull]{stats::dweibull()}},
\code{\link[stats:pweibull]{stats::pweibull()}}, \code{\link[stats:qweibull]{stats::qweibull()}}, and \code{\link[stats:rweibull]{stats::rweibull()}} with
\code{shape = shape} and \code{scale = rate^(-1 / shape)}.

When \eqn{\theta = 1}, the survival function becomes
\deqn{
S(t \mid \lambda, \alpha, 1) = \left(1 + \lambda t^\alpha\right)^{-1},
}
corresponding to the log-logistic distribution.
}
\examples{
## Example 1: Compare the density of GLL with different values of `theta`
x <- seq(0, 10, length.out = 100)
y1 <- pGLL(x, rate = 0.5, shape = 2, theta = 1)
y0 <- pGLL(x, rate = 0.5, shape = 2, theta = 0)
plot(x, y1, type = "l", ylab = "Cumulative distribution function")
lines(x, y0, col = "red", lty = 2)
}
\references{
Wienke A. \emph{Frailty Models in Survival Analysis}. Chapman and Hall/CRC (2010).
}
\seealso{
\code{\link[=rweibullGF]{rweibullGF()}} for simulation of a Weibull distribution with gamma frailty.
}
